﻿'All program codes are developed in the Biosimulation laboratory, Faculty of Life Sciences, Ritsumeikan University
'Reference; Publications
'A human ventricular myocyte model with refined representation of excitation-contraction coupling 
'Running title;                        'CICR in human ventricular cell model
'Authors;    'Y. Himeno†, K. Asakura†‡, C.Y. Cha†1, H. Memida†, T. Powell2, A. Amano†, and A. Noma†. 
'†Biosimulation Project, College of Life Sciences, Ritsumeikan University,
'Noji Higashi 1-1-1 Kusatsu-City, Shiga-prefecture, Japan
'‡Nippon Shinyaku, Co., Ltd., 14, Nishinosho-Monguchi-cho, Kisshoin, Minami-ku, Kyoto, Japan
'1Oxford Centre for Diabetes Endocrinology and Metabolism,
'University of Oxford, Churchill Hospital, Oxford, OX37 LJ, UK.
'2Department of Pharmacology, University of Oxford, Oxford OX1 3QT, UK

'Solution explorer lists all subclasses.
'Calculate.vb provides tools for Disk Write and Read
'Form.APClamp.vb ;   protocol of the Action potetial clamp experiment
'Form.General.vb;       protocol of the current clamp experiments
'Form.ICaLkinetics.vb ;   protocol to draw kinetics of LCC
'Form.RyRtransient.vb;  protocol of the state transition of the couplon model at the onset of action potential
'Form.scaleI1I2;  protocol for simulation of EAD  by varying the inactivation rate of INaL by using the scaling factor for kI1I2, and DAD by injecting 1~1.5 mM Ca into SRup compartment
'Form.VmClampIV.vb;  protocol of Vm clamp experiment
'Graph.vb; home-made tools for Graphics
'Graph2014.vb  home-made tools for Graphics in 2014
'Model.vb;    All equations of HuVEC model are described in this module class
'
'To select a 'Form' to run the protocol of interest, use 'My Project'  and select a Start up form (O) from the pull-down list in the solution explorer.



Imports System.IO

Public Class Form_APClamp

    '********** prepared pens **********
    Public BlackPen As Pen = New Pen(Color.Black, 1)
    Public BlackPen3 As Pen = New Pen(Color.Black, 5)
    Public BluePen As Pen = New Pen(Color.Blue, 1)
    Public BluePen3 As Pen = New Pen(Color.Blue, 5)
    Public RedPen As Pen = New Pen(Color.Red, 1)
    Public RedPen3 As Pen = New Pen(Color.Red, 5)
    Public ChocolatePen As Pen = New Pen(Color.Chocolate, 1)
    Public ChocolatePen3 As Pen = New Pen(Color.Chocolate, 5)
    Public GreenPen As Pen = New Pen(Color.Green, 1)
    Public GreenPen3 As Pen = New Pen(Color.Green, 5)
    Public OlivePen As Pen = New Pen(Color.Olive, 1)
    Public LimePen As Pen = New Pen(Color.Lime, 1)
    Public GreenYellowPen As Pen = New Pen(Color.GreenYellow, 1)
    Public LightCoralPen As Pen = New Pen(Color.LightCoral, 1)
    Public MagentaPen As Pen = New Pen(Color.Magenta, 1)
    Public VioletPen As Pen = New Pen(Color.Violet, 1)
    Public GrayPen As Pen = New Pen(Color.Gray, 1)
    Public GrayPen3 As Pen = New Pen(Color.Gray, 3)
    Public DarkCyanPen As Pen = New Pen(Color.DarkCyan, 1)
    Public DarkMagentaPen As Pen = New Pen(Color.DarkMagenta, 1)
    Public OrangePen As Pen = New Pen(Color.Orange, 1)
    Public cyanPen As Pen = New Pen(Color.Cyan, 1)
    Public yellowPen As Pen = New Pen(Color.Yellow, 1)

    Dim HV As New HuVECIII

    '********** Graph option **********
    Dim graphVm1 As Graph2014
    Dim graphVm2 As Graph2014
    Dim graphIm1 As Graph2014
    Dim graphIm2 As Graph2014

    'Dim graphConc3 As Graph
    'Dim graphGate4 As Graph
    'Dim graphTimeChange6 As Graph
    'Dim graphGate7 As Graph
    'Dim graphGate8 As Graph
    'Dim graphGate9 As Graph

    ''******************************   Graph parameters **********************************
    Dim MinX1 As Double = 0                               'msec, Abscissa of the graph
    Dim MaxX1 As Double = 500
    Dim IntervalX1 As Double = 100

    Dim MinY1 As Double = -90                          'mV, ordinate, graph of membrane potential
    Dim MaxY1 As Double = 50
    Dim IntervalY1 As Double = 10

    Dim MinY2 As Double = -1               '-11         'pA/pF, or uM, ordinate , graph of Current
    Dim MaxY2 As Double = 1.6
    Dim IntervalY2 As Double = 0.2

    Dim MinY3 As Double = 0                                  'mM, or uM, ordinate , graph of Concentrations                                '
    Dim MaxY3 As Double = 10
    Dim IntervalY3 As Double = 1

    Dim MinX4 As Double = 20                                 'unitless, Gating variable
    Dim MaxX4 As Double = 40
    Dim IntervalX4 As Double = 5

    Dim MinY4 As Double = 0                                  'unitless, Gating variable
    Dim MaxY4 As Double = 0.2
    Dim IntervalY4 As Double = 0.05

    Dim MinX5 As Double = 0
    Dim MaxX5 As Double = 1
    Dim IntervalX5 As Double = 0.1

    Dim MinY5 As Double = -1                                'mN/cm2
    Dim MaxY5 As Double = 0
    Dim IntervalY5 As Double = 0.1

    Dim MinX6 As Double = -40                          'mV,  Vtest
    Dim MaxX6 As Double = 50                           ' mV, Vtest
    Dim IntervalX6 As Double = 10                     'mV

    Dim MinY6 As Double = -1
    Dim MaxY6 As Double = 1
    Dim IntervalY6 As Double = (MaxY6 - MinY6) / 10

    Dim MinY10 As Double = -1
    Dim MaxY10 As Double = 0.1
    Dim IntervalY10 As Double = 0.1


    '********** File to hold time variables **********
    Dim FileName As String = "GBP07.csv"

    '********** Membrane current component **********
    Public onset As Double = 50     'start time of external stimulation
    Public offset As Double = 53    'end time of external stimulation

    '********** Time step **********
    Public RepeatN As Double                       'Number of repeat the sweeps
    Public elapsedTime As Double               'msec, experimental time
    Public sweepTime As Double                 'msec   time for each sweep
    Public dt As Double                                   'ms,  time step for integration
    Dim TtoPlot As Double                            'msec timing for plot the point
    Dim maxTtoPlot As Double = 0.01           '
    '**********     counter     *****************************************
    Public numSweep As Double                 'counter of sweeps

    Dim fileNo As Integer                              'File handler
    'Dim valueArray() As String

    Dim originalDataV(400000) As Double      'memory array for the original Vm
    Dim Num As Long                                      'address of the memory array
    Dim lastNum As Long                                'The last memory address
    '***************************************** save data during simulation *******************************************************
    Dim IsRecord As Boolean
    Dim FileName1 As String
    Dim sw1 As StreamWriter
    Dim RecordTimeStart As Double
    Dim RecordDuration As Double
    Dim RecordInterval As Double
    Dim TtoRecord As Double

    Public Tfreeze As Double = 0
    Dim IsFreezeSlowV As Boolean = False
    Dim IsAddCaNa As Boolean = False
    Dim IsFreezeSR As Boolean = False

    Dim maxICaL As Double
    Dim finalICaL As Double
    Dim maxFb As Double
    Dim maxCablk As Double

    Dim preV As Double = 0
    Dim slopeVmax As Double = 0
    Dim Results(100, 10) As Double

    Dim ICaLzero As Double
    Dim Cablkzero As Double
    Dim Fbzero As Double
    Dim FirstSweep As Boolean = True

    Private Sub ButtonStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonStart.Click
        ButtonStart.Visible = False       'conceal the button

        If FirstSweep = True Then
            DefineGraph()                                           'define graphs
            Clear()                                                       'clear the graphs
            FirstSweep = False
        End If

        ReadForm()                                     'Stimulus strength & Sweeptime max

        DrawAxes()                                      'prepare the graph grids

        '************ Time dt ********************
        dt = 0.002                                   'ms,   dt used in this experiment

        HV.VtestOn = lastNum * dt       'always the command V = Vhold
        HV.VtestOff = lastNum * dt

        '**************   Record the control AP ****************
        HV.PCaL = HV.oPCaL * 1
        HV.GK1 = HV.oGK1 * 1
        HV.GIKr = HV.oGIKr * 1
        HV.fixCaconc = False                            'normal Ca dynamics

        HV.DefaultValues()                                'obtain the initial set of variables
        HV.clampMode = "Iclamp"                      'under the current clamp condition
        Num = 0                                                  'address of the memory array for the Vm
        sweepTime = 0                                        'reset sweepTime = 0
        Do
            My.Application.DoEvents()

            HV.ModelCell(sweepTime, dt)             'determination of dydt(N) by calculating the model
            HV.EulerIntegration(dt)                      'integrate ODE using Euler Method,  renew the value in y(x)

            'HV.calculateModel(sweepTime, dt)             'determination of dydt(N) by calculating the model
            'HV.EulerIntegration(dt)                      'integrate ODE using Euler Method,  renew the value in y(x)

            originalDataV(Num) = HV.Vm           'store the Vm for the following action potential clamp experiments.

            graphVm1.PlotPoint(sweepTime, HV.Vm, BlackPen)                'Vm
            graphIm1.PlotPoint(sweepTime, HV.ICaL, RedPen)              'ICaL
            graphIm1.PlotPoint(sweepTime, HV.IKr, BlackPen)                 'IKr
            graphIm1.PlotPoint(sweepTime, HV.IK1, BluePen)                  'IK1
            graphIm1.PlotPoint(sweepTime, HV.INCX, ChocolatePen)       'whole cell current

            Num = Num + 1
            sweepTime = sweepTime + dt                                  'msec, proceed with the experimental time within one sweep
        Loop While sweepTime < MaxX1
        lastNum = Num - 1                                          'final last address of the memory array

        '***************** control Action potential Clamp  ****************
        HV.DefaultValues()                         'obtain the initial set of variables
        Labelintact.Visible = True              'the lavel is made visible
        HV.PCaL = HV.oPCaL * 1                  'control PICaL
        HV.GK1 = HV.oGK1 * 1                      'recover the control GK1
        HV.GIKr = HV.oGIKr * 1
        HV.fixCaconc = False                          'normal Ca dynamics

        HV.clampMode = "Vclamp"                  'only the stimulus current
        Num = 0
        sweepTime = 0                                                    'reset sweepTime = 0
        Do While sweepTime < MaxX1                           'Calculate individual sweep of the simulation
            My.Application.DoEvents()                             'check events
            HV.Vhold = originalDataV(Num + 1)             '  command potential = stored record 

            HV.ModelCell(sweepTime, dt)                'determination of dydt(N) by calculating the model 
            HV.EulerIntegration(dt)                          'integrate ODE using Euler Method,  renew the value in y(x)
            graphVm2.PlotPoint(sweepTime, HV.Vm, BlackPen)
            graphIm2.PlotPoint(sweepTime, HV.Icompensation, BlackPen)      'reflect only the capacitive current

            Num = Num + 1                                        'proceed with the addoress of the memory array
            sweepTime = sweepTime + dt                   'msec, proceed with the experimental time within one sweep
        Loop

        'lastNum = Num - 1

        ' ****************** AP Clamp After blocking IK1   ********************************************
        HV.DefaultValues()                         'obtain the initial set of variables
        LabelIK1.Visible = True              'the lavel is made visible
        HV.PCaL = HV.oPCaL * 1            'control ICaL
        HV.GK1 = HV.oGK1 * 0                'IK1 is blocked
        HV.GIKr = HV.oGIKr * 1             'control IK1
        HV.fixCaconc = False                    'intact Ca dynamics

        HV.clampMode = "Vclamp"
        graphVm2.Clear()                                                         'Clear panel C
        graphVm2.DrawAxis(IntervalX1, IntervalY1, "", 12)   'draw axis of panel C
        Num = 0
        sweepTime = 0                                                      'reset sweepTime = 0
        Do While sweepTime < MaxX1                           'Calculate individual sweep of the simulation
            My.Application.DoEvents()                              'check events
            HV.Vhold = originalDataV(Num)                    ' Vcommand is  the recorded Vm on the memory array

            HV.ModelCell(sweepTime, dt)
            HV.EulerIntegration(dt)
            graphVm2.PlotPoint(sweepTime, HV.Vm, BlackPen)
            graphIm2.PlotPoint(sweepTime, HV.Icompensation, BlackPen)

            Num = Num + 1
            sweepTime = sweepTime + dt                        'msec, proceed with the experimental time within one sweep
        Loop

        ' ****************** AP Clamp After blocking IKr   ********************************************
        HV.DefaultValues()                         'obtain the initial set of variables
        LabelIKr.Visible = True              'the lavel is made visible
        HV.PCaL = HV.oPCaL * 1
        HV.GK1 = HV.oGK1 * 1                     '
        HV.GIKr = HV.oGIKr * 0                   'IKr is blocked
        HV.fixCaconc = False                           'intact Ca dynamics

        HV.clampMode = "Vclamp"
        graphVm2.Clear()
        graphVm2.DrawAxis(IntervalX1, IntervalY1, "", 12)
        Num = 0
        sweepTime = 0                                                      'reset sweepTime = 0
        Do While sweepTime < MaxX1                           'Calculate individual sweep of the simulation
            My.Application.DoEvents()                              'check events
            HV.Vhold = originalDataV(Num)                          ' Vcommand is the recorded Vm in the memory array

            HV.ModelCell(sweepTime, dt)
            HV.EulerIntegration(dt)
            graphVm2.PlotPoint(sweepTime, HV.Vm, BlackPen)
            graphIm2.PlotPoint(sweepTime, HV.Icompensation, BlackPen)

            Num = Num + 1
            sweepTime = sweepTime + dt                        'msec, proceed with the experimental time within one sweep
        Loop

        ' ****************** AP Clamp After blocking ICaL   ********************************************
        HV.DefaultValues()                         'obtain the initial set of variables
        LabelICaL.Visible = True              'the lavel is made visible
        HV.PCaL = HV.PCaL * 0                          'ICaL is blocked
        HV.GK1 = HV.oGK1 * 1
        HV.GIKr = HV.oGIKr * 1
        HV.fixCaconc = True                                                 'no Ca release from SR

        HV.clampMode = "Vclamp"
        Num = 0
        graphVm2.Clear()
        graphVm2.DrawAxis(IntervalX1, IntervalY1, "", 12)
        sweepTime = 0                                                      'reset sweepTime = 0
        Do While sweepTime < MaxX1                           'Calculate individual sweep of the simulation
            My.Application.DoEvents()                              'check events
            HV.Vhold = originalDataV(Num)                     ' Vcommand is the recorded Vm in the memory array

            HV.ModelCell(sweepTime, dt)
            'HV.EulerIntegration(dt)

            graphVm2.PlotPoint(sweepTime, HV.Vm, BlackPen)
            graphIm2.PlotPoint(sweepTime, HV.Icompensation, BlackPen)
            graphIm2.PlotPoint(sweepTime, HV.INCX, RedPen)
            'graphIm2.PlotPoint(sweepTime, HV.ICaL, BlackPen)

            Num = Num + 1
            sweepTime = sweepTime + dt                        'msec, proceed with the experimental time within one sweep
        Loop

        ButtonStart.Visible = True
    End Sub

    Public Sub ReadForm()
        'txtInterval.Text = CStr(MaxX1)
        HV.Samplitude = -6              'CDbl(txtStimulus.Text)        'amplitude of the stimulus current in pA
        RepeatN = 1   'CDbl(txtPonset.Text)              'number of sweeps

        HV.clampMode = "Vclamp"         'always Current Clamp Mode
        'HV.testV = CDbl(txtVtest.Text)         'testV is a particular case of Vtest
        'HV.Vhold = CDbl(txtVhold.Text)
        'HV.VtestOn = CDbl(txtPonset.Text)
        'HV.VtestOff = CDbl(txtPoffset.Text)

        'If chkCaclamp.Checked = True Then
        '    HV.fixCaconc = True
        'Else
        '    HV.fixCaconc = False
        'End If

        HV.IsoMetric = True                                'isometric contraction
        HV.isometricL = 0.95   'CDbl(txthalfSL.Text)          'get the halfsarcomere length from the textbox
        HV.Fext = 10             'CDbl(txtFext.Text)                       'get Fext from the textbox

        HV.sclI1I2 = 1                  'scaling factor for kI1I2 of the late scattered mode Na channel

        'Tfreeze = CDbl(txtTfreeze.Text)           'time to freeze slow variables

        HV.IsMClamp = False                               'defalt value

        IsFreezeSR = False

        IsFreezeSlowV = False

        IsAddCaNa = False

        HV.fNaK = 1

    End Sub

    Private Sub DefineGraph()
        graphVm1 = New Graph2014(pbVm, MinX1, MaxX1, MinY1, MaxY1)
        graphIm1 = New Graph2014(pbNCX, MinX1, MaxX1, MinY2, MaxY2)
        graphVm2 = New Graph2014(pbCommand, MinX1, MaxX1, MinY1, MaxY1)
        graphIm2 = New Graph2014(pbCurrent, MinX1, MaxX1, MinY2, MaxY2)


    End Sub

    Private Sub DrawAxes()
        graphVm1.DrawAxis(IntervalX1, IntervalY1, "", 12)
        graphIm1.DrawAxis(IntervalX1, IntervalY2, "", 12)
        graphVm2.DrawAxis(IntervalX1, IntervalY1, "", 12)
        graphIm2.DrawAxis(IntervalX1, IntervalY2, "", 12)

    End Sub

    Private Sub Clear()
        graphVm1.Clear()
        graphIm2.Clear()
        graphVm2.Clear()
        graphIm1.Clear()

    End Sub

    Private Sub WritetxtBox()
        'txtmaxICaL.Text = Format(maxICaL, "###.#####")

    End Sub

    Private Sub cbErase_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbErase.Click
        graphVm1.Clear()
        graphIm2.Clear()
        graphVm2.Clear()
        graphIm1.Clear()

        DrawAxes()
    End Sub

    Private Sub ButtonEnd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonEnd.Click
        End
    End Sub

End Class