﻿'All program codes are developed in the Biosimulation laboratory, Faculty of Life Sciences, Ritsumeikan University
'Reference; Publications
'A human ventricular myocyte model with refined representation of excitation-contraction coupling 
'Running title;                        'CICR in human ventricular cell model
'Authors;    'Y. Himeno†, K. Asakura†‡, C.Y. Cha†1, H. Memida†, T. Powell2, A. Amano†, and A. Noma†. 
'†Biosimulation Project, College of Life Sciences, Ritsumeikan University,
'Noji Higashi 1-1-1 Kusatsu-City, Shiga-prefecture, Japan
'‡Nippon Shinyaku, Co., Ltd., 14, Nishinosho-Monguchi-cho, Kisshoin, Minami-ku, Kyoto, Japan
'1Oxford Centre for Diabetes Endocrinology and Metabolism,
'University of Oxford, Churchill Hospital, Oxford, OX37 LJ, UK.
'2Department of Pharmacology, University of Oxford, Oxford OX1 3QT, UK

'Solution explorer lists all subclasses.
'Calculate.vb provides tools for Disk Write and Read
'Form.APClamp.vb ;   protocol of the Action potetial clamp experiment
'Form.General.vb;       protocol of the current clamp experiments
'Form.ICaLkinetics.vb ;   protocol to draw kinetics of LCC
'Form.RyRtransient.vb;  protocol of the state transition of the couplon model at the onset of action potential
'Form.scaleI1I2;  protocol for simulation of EAD  by varying the inactivation rate of INaL by using the scaling factor for kI1I2, and DAD by injecting 1~1.5 mM Ca into SRup compartment
'Form.VmClampIV.vb;  protocol of Vm clamp experiment
'Graph.vb; home-made tools for Graphics
'Graph2014.vb  home-made tools for Graphics in 2014
'Model.vb;    All equations of HuVEC model are described in this module class
'
'To select a 'Form' to run the protocol of interest, use 'My Project'  and select a Start up form (O) from the pull-down list in the solution explorer.



Imports System.IO

Public Class Form_RyRtransient
    '********** prepared pens **********
    Public BlackPen As Pen = New Pen(Color.Black, 1)
    Public BlackPen2 As Pen = New Pen(Color.Black, 2)
    Public BlackPen3 As Pen = New Pen(Color.Black, 3)
    Public BluePen As Pen = New Pen(Color.Blue, 1)
    Public BluePen2 As Pen = New Pen(Color.Blue, 2)
    Public BluePen3 As Pen = New Pen(Color.Blue, 3)
    Public RedPen As Pen = New Pen(Color.Red, 1)
    Public RedPen3 As Pen = New Pen(Color.Red, 3)
    Public RedPen2 As Pen = New Pen(Color.Red, 2)
    Public ChocolatePen As Pen = New Pen(Color.Chocolate, 1)
    Public ChocolatePen2 As Pen = New Pen(Color.Chocolate, 2)
    Public ChocolatePen3 As Pen = New Pen(Color.Chocolate, 3)
    Public GreenPen As Pen = New Pen(Color.DarkGreen, 1)
    Public GreenPen3 As Pen = New Pen(Color.Green, 3)
    Public OlivePen As Pen = New Pen(Color.Olive, 1)
    Public LimePen As Pen = New Pen(Color.Lime, 1)
    Public LimePen3 As Pen = New Pen(Color.Lime, 3)
    Public GreenYellowPen As Pen = New Pen(Color.GreenYellow, 1)
    Public LightCoralPen As Pen = New Pen(Color.LightCoral, 1)
    Public MagentaPen As Pen = New Pen(Color.Magenta, 1)
    Public VioletPen3 As Pen = New Pen(Color.Violet, 3)
    Public VioletPen As Pen = New Pen(Color.Violet, 1)
    Public GrayPen As Pen = New Pen(Color.Gray, 1)
    Public GrayPen2 As Pen = New Pen(Color.Gray, 2)
    Public GrayPen3 As Pen = New Pen(Color.Gray, 3)
    Public DarkCyanPen As Pen = New Pen(Color.DarkCyan, 1)
    Public DarkMagentaPen As Pen = New Pen(Color.DarkMagenta, 1)
    Public OrangePen As Pen = New Pen(Color.Orange, 1)
    Public cyanPen As Pen = New Pen(Color.Cyan, 1)
    Public yellowPen As Pen = New Pen(Color.Yellow, 1)

    '********** Graph option **********
    Dim graphVm1 As Graph2014
    Dim graphCand2 As Graph2014
    Dim graphpO3 As Graph2014
    Dim graphCasr4 As Graph2014
    Dim graphRate5 As Graph2014

    ''******************************   Graph parameters **********************************
    Dim MinX1 As Double = 48                       'msec, Abscissa of the graph
    Dim MaxX1 As Double = 70
    Dim IntervalX1 As Double = 1

    Dim MinY1 As Double = -100                          'mV, ordinate, graph of membrane potential
    Dim MaxY1 As Double = 50
    Dim IntervalY1 As Double = 20

    Dim MinY2 As Double = 0                                  'uM, ordinate , graph of [Ca]nd
    Dim MaxY2 As Double = 300
    Dim IntervalY2 As Double = 50

    Dim MinY3 As Double = 0                                  'open probability of RyR
    Dim MaxY3 As Double = 1
    Dim IntervalY3 As Double = 0.1

    Dim MinY4 As Double = 0                                  'mM, [Ca]SRrl
    Dim MaxY4 As Double = 0.8
    Dim IntervalY4 As Double = 0.1

    Dim MinY5 As Double = 0                                  '/ms, state transition rate
    Dim MaxY5 As Double = 2.5
    Dim IntervalY5 As Double = 0.5

    '********** File to hold time variables **********
    Dim HV As New HuVECIII
    Dim CsvFileName As String = "CsvHuVEC.csv"
 
    Dim names1() As String = New String() {"Vm", "TnChCa", "CaMCa", "bufferSRCa", "Lb_jnc", "Lb_iz", "Hb_jnc", "Hb_iz", "Nai", "Ki", _
                                       "Catot_jnc", "Catot_iz", "Catot_blk", "Ca_SRup", "Catot_SRrl", "O_TM", "I2_TM", "Is_TM", "O_LSM", _
                                       "I1_LSM", "I2_LSM", "Is_LSM", "Yco_iz", "Yoc_iz", "Yoo_iz", "Yco_blk", "Yoc_blk", "Yoo_blk", "Yooo", _
                                       "Yooc", "Ycoo", "Ycoc", "Ycco", "Yoco", "Yocc", "y1_IKr", "y2_IKr", "y3_IKr", "Ov_IKs", "C2IKs_iz", _
                                       "OcIKs_iz", "C2IKs_blk", "OcIKs_blk", "y1_IKto", "y2_IKto", "Pbspm", "E1NCX_iz", "I1NCX_iz", "I2NCX_iz", "E1NCX_blk", _
                                       "I1NCX_blk", "I2NCX_blk", "P1_6_NaK", "P7_NaK", "P8_13_NaK", "halfSL", "Fb", "Fp", "TSCa3", "TSCa3W", _
                                       "TSCa3S", "TSS", "TSW", "hw", "hp", "ATPt_cyt", "ADPt_cyt", "Pi_cyt", "PCr_cyt", "ATPt_mit", "Pi_mit", _
                                       "NADH_mit", "H_mit", "K_mit", "UQr_mit", "ctCrd_mit", "dpsi", "DT"}    '"a_IKto",
    '********** Time step **********
    Public numSweep As Double                 'counter of sweeps
    Public dt As Double = 0.00005      '  msec, time step   time step if 0.00001 msec for the Keizer Levine model of RyR

    Public sweepTime As Double                 'msec   time for each sweep

    '***************************************** save data during simulation *******************************************************
    Dim IsRecord As Boolean = False
    Dim FileName1 As String
    Dim sw1 As StreamWriter
    Dim RecordTimeStart As Double
    Dim RecordDuration As Double
    Dim RecordInterval As Double
    Dim TtoRecord As Double

    Dim DefineG As Boolean = True
    Dim adjustDt As Boolean = True                   'adjust dt for the numerical integration in Heun Method

    '******************************* Biological parameters used in Form **********************
    Dim weightkoc As Double            'net deactivation rate of RyRs
    Dim weightkco As Double            'net activation rate of RyRs

    Private Sub ButtonStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonStart.Click
        ButtonStart.Visible = False       'hide the starting command button

        ReadForm()                                   'Stimulus strength & Sweeptime max

        If DefineG Then                              'only when starting simulation
            DefineGraph()                                           'define graphs
            Clear()
            DefineG = False
        End If

        'HV.DefaultValues()
        CSVReadAndWrite.ReadCSV(CsvFileName, HV, names1)      'start from the last set of variables

        DrawAxes()                                              'clear graph and draw new graph grids

        sweepTime = MinX1                                        'reset sweepTime at the start of simulation
        Do                                       'Calculate individual sweep of the simulation
            My.Application.DoEvents()                                       'check events

            If chkHeun.Checked Then
                HV.HeunIntegration(adjustDt, sweepTime, dt) 'use Heun method of integration
            Else
                dt = 0.0005
                HV.ConvIntegration(sweepTime, dt) 'use Euler method
            End If

            weightkoc = (HV.Yoco * HV.kYocoYocc + HV.Ycco * HV.kYccoYccc + HV.Ycoo * HV.kYcooYcoc + HV.Yooo * HV.kYoooYooc) / (HV.Yoco + HV.Ycco + HV.Ycoo + HV.Yooo)
            weightkco = (HV.Yocc * HV.kYoccYoco + HV.Yccc * HV.kYcccYcco + HV.Ycoc * HV.kYcocYcoo + HV.Yooc * HV.kYoocYooo) / (HV.Yocc + HV.Yccc + HV.Ycoc + HV.Yooc)     'weighted rate of activation of RyR

            PlotResults()

            sweepTime = sweepTime + dt                                  'msec, proceed with the experimental time within one sweep
        Loop While sweepTime < MaxX1            '************************** END of one Sweep ******************************************

        WritetxtBox()                                                                  'write final values of several variables

        ButtonStart.Visible = True                                                      'display the start button

        If IsRecord = True Then
            sw1.Close()
        End If


    End Sub

    Dim Tplot As Double = 0
    Dim Tskip As Double = 0   'dt * 6 '2.1
    Dim Tint As Double = 0.2  'dt * 5     'ms interval for drawing interrupted line

    Private Sub PlotResults()
        graphVm1.PlotPoint(sweepTime, HV.Vm, BlackPen)
        graphCand2.PlotPoint(sweepTime, HV.Ca_nd00 * 1000, ChocolatePen)

        If sweepTime < 54 Then  '54 Then
            If Tplot < Tint Then
                Tplot = Tplot + dt
            Else
                graphCand2.PlotPoint(sweepTime, HV.Ca_ndLR * 1000, RedPen)
                graphCand2.PlotPoint(sweepTime, HV.Ca_nd0R * 1000, BlackPen)
                graphCand2.PlotPoint(sweepTime, HV.Ca_ndL0 * 1000, BluePen)
                Tplot = 0
            End If
        Else
            graphCand2.PlotPoint(sweepTime, HV.Ca_ndLR * 1000, RedPen)
            graphCand2.PlotPoint(sweepTime, HV.Ca_ndL0 * 1000, BluePen)
            graphCand2.PlotPoint(sweepTime, HV.Ca_nd0R * 1000, BlackPen)
        End If


        Dim pORyR As Double = HV.Yooo + HV.Ycoo + HV.Ycco + HV.Yoco
        graphpO3.PlotPoint(sweepTime, pORyR, RedPen)
        'graphpO3.PlotPoint(sweepTime, HV.pORyR * 10, RedPen)

        graphCasr4.PlotPoint(sweepTime, HV.Cafree_SRrl, BlackPen)
        graphRate5.PlotPoint(sweepTime, weightkoc, BluePen)
        graphRate5.PlotPoint(sweepTime, weightkco, RedPen)
        'graphRate5.PlotPoint(sweepTime, HV.fixedkco, GrayPen)
        graphpO3.PlotPoint(sweepTime, (HV.Yooo + HV.Yooc), ChocolatePen)

        Dim yvalue As Double = weightkco / (weightkco + weightkoc)

        'yvalue = Yocc * kYoccYoco + Yccc * kYcccYcco + Ycoc * kYcocYcoo + Yooc * kYoocYooo
        'yvalue = yvalue / (Yocc + Yccc + Ycoc + Yooc)     'weighted rate of activation of RyR
        'yvalue = yvalue / (yvalue + kRoc)                        'rate of deactivation common for all open states of RyR
        graphpO3.PlotPoint(sweepTime, yvalue, BluePen)

    End Sub

    Public Sub ReadForm()
        If rbBlinkS.Checked Then
            HV.BlinkS = True                   'The model include the blink space
            HV.ratioBS = CDbl(txtRatioBS.Text)               'ratio of gD_bs / gD_nd
        Else
            HV.BlinkS = False                   'No blink space is included in the model
        End If

        adjustDt = False

        'txtInterval.Text = CStr(MaxX1)
        HV.Samplitude = CDbl(txtStimulus.Text)        'amplitude of the stimulus current in pA

        HV.IsoMetric = False                              'isotonic contraction
        HV.isometricL = CDbl(txthalfSL.Text)          'get the halfsarcomere length from the textbox
        HV.Fext = CDbl(txtFext.Text)                        'get Fext from the textbox

        HV.sclI1I2 = 1                    'scaling factor for kI1I2 of the late scattered mode Na channel
        HV.scaleIKr = 1                'scaling factor for IKr amplitude 

        HV.clampMode = "Iclamp"                'always Current Clamp Mode in the present protocol

        HV.fNaK = 1                'scaling the Na/K pump current during digitalis experiments

    End Sub


    Private Sub DefineGraph()
        graphVm1 = New Graph2014(pbVm, MinX1, MaxX1, MinY1, MaxY1)
        graphCand2 = New Graph2014(pbCand, MinX1, MaxX1, MinY2, MaxY2)
        graphpO3 = New Graph2014(pbpO, MinX1, MaxX1, MinY3, MaxY3)
        graphCasr4 = New Graph2014(pbCasr, MinX1, MaxX1, MinY4, MaxY4)
        graphRate5 = New Graph2014(pbRate, MinX1, MaxX1, MinY5, MaxY5)
    End Sub

    Private Sub Clear()
        graphVm1.Clear()
        graphCand2.Clear()
        graphpO3.Clear()
        graphCasr4.Clear()
        graphRate5.Clear()

    End Sub


    Private Sub DrawAxes()
        graphVm1.DrawAxis(IntervalX1, IntervalY1, "", 8)
        graphCand2.DrawAxis(IntervalX1, IntervalY2, "", 8)
        graphpO3.DrawAxis(IntervalX1, IntervalY3, "", 8)
        graphCasr4.DrawAxis(IntervalX1, IntervalY4, "", 8)
        graphRate5.DrawAxis(IntervalX1, IntervalY5, "", 8)

        graphVm1.PlotLine(HV.ItestOff, MinY1, HV.ItestOff, MaxY1, BlackPen2)
        graphCand2.PlotLine(HV.ItestOff, MinY2, HV.ItestOff, MaxY2, BlackPen2)
        graphpO3.PlotLine(HV.ItestOff, MinY3, HV.ItestOff, MaxY3, BlackPen2)
        graphCasr4.PlotLine(HV.ItestOff, MinY4, HV.ItestOff, MaxY4, BlackPen2)
        graphRate5.PlotLine(HV.ItestOff, MinY5, HV.ItestOff, MaxY5, BlackPen2)

        graphVm1.PlotLine(HV.ItestOn, MinY1, HV.ItestOn, MaxY1, BlackPen2)
        graphCand2.PlotLine(HV.ItestOn, MinY2, HV.ItestOn, MaxY2, BlackPen2)
        graphpO3.PlotLine(HV.ItestOn, MinY3, HV.ItestOn, MaxY3, BlackPen2)
        graphCasr4.PlotLine(HV.ItestOn, MinY4, HV.ItestOn, MaxY4, BlackPen2)
        graphRate5.PlotLine(HV.ItestOn, MinY5, HV.ItestOn, MaxY5, BlackPen2)

    End Sub

    Private Sub WritetxtBox()
        'txtVm.Text = Format(maxValue1, "###.### V/s")    'maximum rate of rise of AP
    End Sub

    Private Sub cbReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbReset.Click

    End Sub

    Private Sub cbErase_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbErase.Click
        Clear()
    End Sub

    Private Sub ButtonEnd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonEnd.Click
        End
    End Sub

End Class