﻿Imports System.Math
Imports System.IO

Public Class HuVECIII

    'All program codes are developed in the Biosimulation laboratory, Faculty of Life Sciences, Ritsumeikan University
    'Reference; Publications
    'A human ventricular myocyte model with refined representation of excitation-contraction coupling in Biophysical Journal (2015, in press)
    'Running title;                        'CICR in human ventricular cell model
    'Authors;    'Y. Himeno†, K. Asakura†‡, C.Y. Cha†1, H. Memida†, T. Powell2, A. Amano†, and A. Noma†. 
    '†Biosimulation Project, College of Life Sciences, Ritsumeikan University,
    'Noji Higashi 1-1-1 Kusatsu-City, Shiga-prefecture, Japan
    '‡Nippon Shinyaku, Co., Ltd., 14, Nishinosho-Monguchi-cho, Kisshoin, Minami-ku, Kyoto, Japan
    '1Oxford Centre for Diabetes Endocrinology and Metabolism,
    'University of Oxford, Churchill Hospital, Oxford, OX37 LJ, UK.
    '2Department of Pharmacology, University of Oxford, Oxford OX1 3QT, UK

    'All subclasses are listed in the solution explorer.
    'CSVReadAndWrite.vb provides tools for Disk Write and Read
    'Graph2014.vb  home-made tools for Graphics in 2014

    'HuVEC.vb;    All equations of HuVEC model are described in this module class

    'Form.APClamp.vb ;        protocol of the Action potetial clamp experiment
    'Form.General.vb;           protocol of the current clamp experiments
    'Form.ICaLkinetics.vb ;   protocol to draw kinetics of LCC
    'Form.RyRtransient.vb;   protocol of the state transition of the couplon model at the onset of action potential
    'Form.scaleI1I2;               protocol for simulation of EAD  by varying the inactivation rate of INaL by using the scaling factor for kI1I2, and DAD by injecting 1~1.5 mM Ca into SRup compartment
    'Form.VmClampIV.vb;     protocol of Vm clamp experiment
    '
    'To select a 'Form' to run the protocol of interest, use 'My Project'  and select a Start up form (O) from the pull-down list in the solution explorer.

    'The CaRU of Hinch et al (2004) is used to describe the local control theory for the CICR at the dyadic junction after modifications.
    'The compartments were adjusted based on the Microdomain [Ca2+] near the Ca releasing site reported by  Karoly Acsai, Gudrun Antoons1, Leonid Livshitz, YoramRudy and Karin R. Sipido1 J Physiol 589.10 (2011) pp 2569–2583
    'The empirical equations of ion channels or transporters are mostly based on the Kyoto model, and parameters were fit to Human data.
    'This HuVEC model is scalable for the Cm and cell volume without changing its characteristics.

    ' unit     concentration; mM, time msec, volume; fL, current pA

    '********** Physical Contents **********
    Public R As Double = 8.3143                                   '  
    Public Faraday As Double = 96.4867                       '
    Public tempK As Double = 310                                '
    Public RTF As Double = R * tempK / Faraday
    Public RTF2 As Double = R * tempK / Faraday / 2

    '*************** Blink Space ***************
    Public BlinkS As Boolean = False       'include the blink space, usually this is not calculated for simplicity

    '********** External Ion Concentrations **********
    Public Ko As Double = 4.5      'mM         'Note [K+]o is different from those in rabbit, guinea pig etc.
    Public Nao As Double = 140    'mM         '
    Public Cao As Double = 1.8      'mM        '

    '********** Cell size and related Parameters **********
    Public Const Sc_Cell As Double = 1        ' scaling factor  for Cm and Cell volume. Note Sc_Cell is also applied for SR Ca dynamics and Ca diffusion. 

    Public Const Cm As Double = 192.46 * Sc_Cell                   '

    Public Const Vol_cell As Double = 120 * (37.62 * Sc_Cell) * 8.4    'um3 This volume includes all organella and contraction machinery excluding SR
    '*********** Ca compartments *************************
    Public Vol_blk As Double = 0.68 * Vol_cell                                ' diffusion space for ions 
    Public Vol_iz As Double = 0.035 * Vol_cell                                'fL  representing the nrs space defined by the Experimental work Acsai et al 
    Public Vol_jnc As Double = 0.008 * Vol_cell                              'fL  space including whole the population of LCC-RyR couples
    Public Vol_cyt As Double = Vol_jnc + Vol_iz + Vol_blk             'fL cytosolic space for ion diffusion
    Public Const Vol_SRt As Double = 0.06 * Vol_cell                     'fL  Whole SR volume 
    Public Const Vol_SRrl As Double = 0.2 * Vol_SRt                     'fL  fractinal volume of the junctional SR  revised on 150519
    Public Const Vol_SRup As Double = 0.8 * Vol_SRt                    'fL  fractional volume of the network SR  revised on 150519

    ' ****************** parameters for the mitochondria *****   Temporarily, mitochondria is removed, and [ATP] is fixed in the present version of HuVEC for simplicity
    Public Const Rcm_mit As Double = 4.35                                      '  = (Vol_cyt / Vol_mit)      'volume ratio of cytosol/mitochondria  
    Public Vol_mit As Double = Vol_cyt / Rcm_mit                             ' fL  mitochondria volume
    Public Cm_mit As Double = 0.001 * Vol_mit * Faraday               ' pF

    Dim ampRatio As Double = 110036 '* 2             ' scaling factor for the mitochondria activity; correction, multiplied by 2

    '********** Fraction of Currents jnc, iz, blk **********
    Public Const Frc_iz As Double = 0.1                              'fractional  distribution at the iz myoplasm for ion transporters except LCC and NCX
    Public Const Frc_blk As Double = 0.9                            'fractional  distribution at the iz myoplasm for ion transporters except LCC and NCX

    Public Const FrcCaL_jnc As Double = 0.75                     'fractional distribution of ICaL at the junctional myoplasm
    Public Const FrcCaL_iz As Double = 0.15                       'fractional distribution of ICaL at the iz myoplasm
    Public Const FrcCaL_blk As Double = 0.1                       'fractional distribution of ICaL at the blk myoplasm

    Public FrcNCX_jnc = 0                                                        'fractional distribution of NCX at the junctional myoplasm
    Public Const FrcNCX_iz = 0.1                                            'fractional distribution of NCX at the iz myoplasm
    Public Const FrcNCX_blk = 0.9                                          'fractional distribution of NCX at the blk myoplasm

    '*********  Ion Concentrations,  Ca space is divided into 3 compartments    *****************************
    Public ItotCa_jnc As Double
    Public ItotCa_iz As Double
    Public ItotCa_blk As Double

    Public Catot_cyt As Double           'mmole    tatal amount of Ca within the cell
    Public Catot_jnc As Double           'mM, total concentration of Ca
    Public Catot_iz As Double
    Public Catot_blk As Double

    Public Cafree_jnc As Double            'mM, free Ca in the junctional space
    Public Cafree_iz As Double            'mM, free Ca in the iz space
    Public Cafree_blk As Double            'mM, free Ca in the bulk space
    Public Carest_blk As Double           'mM, [Ca free] at the end of diastole

    Public dCatot_jncdt As Double       'mM/ms    rate of change
    Public dCatot_izdt As Double
    Public dCatot_blkdt As Double

    '*********  no compartments in Na- and K-spaces    *****************************
    Public ItotNa_cyt As Double          'pA  total current carried by Na+
    Public Nai As Double                     'mM, no compartmentation
    Public dNaidt As Double                'mM/ms  

    Public ItotK_cyt As Double          'pA  total current carried by K+
    Public Ki As Double                      'mM, no compartmentation
    Public dKidt As Double

    '*************** [Ca] within SR  ***********************
    Public Ca_SRup As Double            'mM, free Ca concentration , No Ca-buffer is assumed
    Public dCa_SRupdt As Double      'mM/ms

    Public Cafree_SRrl As Double         'mM, concentration of free Ca2+ buffered with carsequestrin
    Public Catot_SRrl As Double           'mM, concentration of total Ca
    Public dCatot_SRrldt As Double      'mM/ms

    '********** Nernst Potential  **********
    Public ECa_jnc As Double               'mV
    Public ECa_iz As Double               'mV
    Public ECa_blk As Double               'mV
    Public ENa As Double                    'mV
    Public EK As Double                      'mV

    '********** Reversal Potential defined in VL analysis  **********
    Public ErevCa_LR As Double               'mV
    Public ErevCa_L0 As Double               'mV
    Public ErevCa_iz As Double                'mV
    Public ErevCa_blk As Double               'mV
    Public ErevNa As Double                    'mV
    Public ErevK As Double                      'mV
    '********** GHK equation **********
    Public GHKCa_LR As Double             'GHK equation for Ca nanodomain when LCC and RyR are both open
    Public GHKCa_L0 As Double              'GHK equation for Ca nanodomain when only LCC is open
    Public GHKCa_iz As Double               'GHK equation for Ca in the iz space
    Public GHKCa_blk As Double             'GHK equation for Ca in the blk space
    Public GHKNa As Double
    Public GHKK As Double
    '   *****************   VL analysis of GHK components ************
    Public dGHKCa_LR As Double             'nS,  slope conductance of GHK equation for Ca   
    Public dGHKCa_L0 As Double               'nS,  slope conductance of GHK equation for Ca
    Public dGHKCa_iz As Double               'nS,  slope conductance of GHK equation for Ca
    Public dGHKCa_blk As Double             'nS, slope conductance of GHK equation for Ca
    Public dGHKNa As Double                    'nS, slope conductance of GHK equation for Na
    Public dGHKK As Double                     'nS, slope conductance of GHK equation for K

    '********** technical variables **********
    Public fixzero_INaLslowgate As Double = 1    'used to fix the slow inactivation gate of INaL
    Public fixzero_slowVariable As Double = 1      'used to fix slow variables in bifurcation analysis
    Public fixCaconc As Boolean                            'fix [Ca] in all compartments to simulate Ca chelating exp.

    Public Nvar As Double = 199           'number of time-dependent variables  
    'Public Mery(Nvar) As Double           'memory array
    Public dydt(Nvar) As Double             'memory array
    Public Carry(100) As Double             'used to bring data to Form1
    Public DA(20, 40) As Double             'for VL analysis

    '*************** Adams-Bashforth-Moulton **************************
    Public NVr As Integer = 70             'number of variables 
    Public D(9, Nvr + 1) As Double
    Dim Sum As Double = 0         'Sum is used to decide the beginning of integration
    Dim Ratio As Double                                         'ratio of (y(n+1) - y(n))/y(n)

    Public Sub HeunIntegration(ByVal adjuststepsize As Boolean, ByVal sweeptime As Double, ByRef dt As Double)
        '********************* Heun Integration method *******************************
        ' Step 1 at P1
        YtoP1_Y()                                            'copy y values to P1_y
        calculateModel(sweeptime, dt)          'calculate dydt(i) at D(2,i),  The sweeptime is necessary to calculate a given Ca transient
        copydydt()                                           'keep dydt at P1, for calculating average with dy/dt at P2
        EulerIntegration(dt)                           'P2 is obtained
        ' Step 2 at P2
        calculateModel(sweeptime + dt, dt)   'calculate dydt(i) at P2, 
        ' Step 3   numerical integration using the average dy/dt, and adjusted dt
        Averagedydt()                                      'Heun slope (corrected Euler method), using the  average dydt(i) a tP1 and dydt(i) at P2  
        P1_YtoY()                                            'recover y from P1_y at P1, and Itot_Cell , Itot_K
        If adjuststepsize = True Then adjustDT(dt) 'determine optimum dt for integration at P1

        EulerIntegration(dt)                           'P3 is obtained
        '************* end of Heun method ******************
    End Sub

    Public Sub ConvIntegration(ByVal sweeptime As Double, ByRef dt As Double)  'simply use the Euler method
        'dt = 0.01
        calculateModel(sweeptime, dt)          'calculate dydt(i) at D(2,i),  The sweeptime is necessary to calculate a given Ca transient
        EulerIntegration(dt)                           'P2 is obtained
    End Sub


    Dim P1_Vm As Double
    Dim P1_Vo As Double

    Dim P1_TnChCa As Double
    Dim P1_CaMCa As Double                  'CaM within blk space
    Dim P1_bufferSRCa As Double            'a Ca-buffer within the blk space

    Dim P1_Catot_jnc As Double
    Dim P1_Catot_iz As Double
    Dim P1_Catot_blk As Double

    Dim P1_Nai As Double
    Dim P1_Ki As Double

    Dim P1_Ca_SRup As Double
    Dim P1_Catot_SRrl As Double


    Dim P1_O_TM As Double
    Dim P1_I2_TM As Double
    Dim P1_Is_TM As Double

    Dim P1_O_LSM As Double
    Dim P1_I1_LSM As Double
    Dim P1_I2_LSM As Double
    Dim P1_Is_LSM As Double


    Dim P1_Yooo As Double
    Dim P1_Yooc As Double

    Dim P1_Ycoo As Double
    Dim P1_Ycoc As Double

    Dim P1_Ycco As Double

    Dim P1_Yoco As Double
    Dim P1_Yocc As Double


    Dim P1_Yco_iz As Double
    Dim P1_Yoc_iz As Double
    Dim P1_Yoo_iz As Double

    Dim P1_Yco_blk As Double
    Dim P1_Yoc_blk As Double
    Dim P1_Yoo_blk As Double


    Dim P1_y1_IKr As Double
    Dim P1_y2_IKr As Double
    Dim P1_y3_IKr As Double


    Dim P1_Ov_IKs As Double

    Dim P1_C2IKs_iz As Double
    Dim P1_OcIKs_iz As Double

    Dim P1_C2IKs_blk As Double
    Dim P1_OcIKs_blk As Double


    Dim P1_y1_IKto As Double
    Dim P1_y2_IKto As Double

    Dim P1_Pbspm As Double

    Dim P1_E1NCX_iz As Double
    Dim P1_I1NCX_iz As Double
    Dim P1_I2NCX_iz As Double

    Dim P1_E1NCX_blk As Double
    Dim P1_I1NCX_blk As Double
    Dim P1_I2NCX_blk As Double


    Dim P1_P1_6_NaK As Double
    Dim P1_P7_NaK As Double
    Dim P1_P8_13_NaK As Double


    Dim P1_TSCa3 As Double
    Dim P1_TSCa3W As Double
    Dim P1_TSCa3S As Double
    Dim P1_TSS As Double
    Dim P1_TSW As Double

    Dim P1_hw As Double
    Dim P1_hp As Double

    Public Sub YtoP1_Y()         'copy variable y to D9
        P1_Vm = Vm
        'P1_Vo = Vo

        P1_TnChCa = TnChCa                 'high-binding site of Tn within blk space 
        P1_CaMCa = CaMCa                  'CaM within blk space
        P1_bufferSRCa = bufferSRCa            'a Ca-buffer within the blk space

        P1_Catot_jnc = Catot_jnc
        P1_Catot_iz = Catot_iz
        P1_Catot_blk = Catot_blk

        P1_Nai = Nai
        P1_Ki = Ki

        P1_Ca_SRup = Ca_SRup
        P1_Catot_SRrl = Catot_SRrl


        P1_O_TM = O_TM
        P1_I2_TM = I2_TM
        P1_Is_TM = Is_TM

        P1_O_LSM = O_LSM
        P1_I1_LSM = I1_LSM
        P1_I2_LSM = I2_LSM
        P1_Is_LSM = Is_LSM


        P1_Yooo = Yooo
        P1_Yooc = Yooc

        P1_Ycoo = Ycoo
        P1_Ycoc = Ycoc

        P1_Ycco = Ycco

        P1_Yoco = Yoco
        P1_Yocc = Yocc


        P1_Yco_iz = Yco_iz
        P1_Yoc_iz = Yoc_iz
        P1_Yoo_iz = Yoo_iz

        P1_Yco_blk = Yco_blk
        P1_Yoc_blk = Yoc_blk
        P1_Yoo_blk = Yoo_blk


        P1_y1_IKr = y1_IKr
        P1_y2_IKr = y2_IKr
        P1_y3_IKr = y3_IKr


        P1_Ov_IKs = Ov_IKs

        P1_C2IKs_iz = C2IKs_iz
        P1_OcIKs_iz = OcIKs_iz

        P1_C2IKs_blk = C2IKs_blk
        P1_OcIKs_blk = OcIKs_blk


        P1_y1_IKto = y1_IKto
        P1_y2_IKto = y2_IKto

        P1_Pbspm = Pbspm

        P1_E1NCX_iz = E1NCX_iz
        P1_I1NCX_iz = I1NCX_iz
        P1_I2NCX_iz = I2NCX_iz

        P1_E1NCX_blk = E1NCX_blk
        P1_I1NCX_blk = I1NCX_blk
        P1_I2NCX_blk = I2NCX_blk


        P1_P1_6_NaK = P1_6_NaK
        P1_P7_NaK = P7_NaK
        P1_P8_13_NaK = P8_13_NaK


        P1_TSCa3 = TSCa3
        P1_TSCa3W = TSCa3W
        P1_TSCa3S = TSCa3S
        P1_TSS = TSS
        P1_TSW = TSW

        P1_hw = hw
        P1_hp = hp

        '*************************** Correction required for averaging slope of dy/dt related to Energy Metabolism ****************

    End Sub

    'Dim vY1 As Double
    'Dim vY2 As Double
    'Dim vY3 As Double

    Public optVm As Double = 0.12          'mV, an optimum or standard dVm per dt    
    Public upperLimitDt As Double = 0.12          'ms, upper limit of Dt       max upperlimitDt = 0.14 
    Public lowerLimitDt As Double = 0.02           'ms, lower limit of Dt

    Public Sub adjustDT(ByRef dt As Double)
        Dim temporaryDT As Double                                  'ms, temporary dt
        temporaryDT = Abs(optVm / dVmdt)                       'ms, determine a temporary dt
        dt = Int(1000 * temporaryDT) / 1000                   'ms, calculate to three decimal places
        If dt >= upperLimitDt Then dt = upperLimitDt 'ms, constrain by the upper limit of dt
        If dt <= lowerLimitDt Then dt = lowerLimitDt 'ms, constrain by the lower limit of dt
    End Sub

    Public intactMit As Boolean = True               'usually calculate Mitochondria

    Public Sub calculateModel(ByVal st As Double, ByVal DT As Double)
        'CheckV0(Vm)                                                 'avoid denominator at 0
        cytSubstrateConcentrations()                        'In the present version, all [ATP], [ADP] , [Pi], [CrP] etc are fixed.
        IonHomeoStasis()                                            'Ca buffer and Ca diffusion between jnc-iz spaces and iz-blk space
        MembraneCurrent(Vm)                                  'calculate the ionic currents
        SRCadynamics()                                               'Ca diffusion within SR, Ca flux via CaRU, Ca uptake by SERCA
        MembranePotential(st, DT)                             'calculate the membrane fluxes
        Contraction(Cafree_blk)                                   'Negroni & Lascano 2008
        dIonConcdt()                                                     'rate of change in ion concentrations
        If intactMit = True Then EnergyMetabolism() 'Energy balance  skipped in the present version

        '*************** optional calculation of  total Ca within the cell ********************************
        Catot_cyt = 0.68 * (Cafree_blk + TnChCa + CaMCa + bufferSRCa + (TSCa3 + TSCa3W + TSCa3S) * 3 / 1000)        'amount of Ca within blk-space
        Catot_cyt = Catot_cyt + (Ca_SRup * 0.8 + Catot_SRrl * 0.2) * 0.06                                                                  'amount of Ca within SR
        Catot_cyt = Catot_cyt + (Cafree_iz + Lb_iz + Hb_iz) * 0.035 + (Cafree_jnc + Lb_jnc + Hb_jnc) * 0.008            'amount of Ca within iz, and jnc spaces


    End Sub


    Public Sub ModelCell(ByVal sweeptime As Double, ByVal dt As Double)
        'CheckV0(Vm)                                                 'avoid denominator at 0
        cytSubstrateConcentrations()                        'In the present version, all [ATP], [ADP] , [Pi], [CrP] etc are fixed.
        IonHomeoStasis()                                            'Ca buffer and Ca diffusion between jnc-iz spaces and iz-blk space
        MembraneCurrent(Vm)                                  'calculate the ionic currents
        SRCadynamics()                                               'Ca diffusion within SR, Ca flux via CaRU, Ca uptake by SERCA
        MembranePotential(sweeptime, dt)                    'calculate the membrane fluxes
        Contraction(Cafree_blk)                                     'Negroni & Lascano 2008
        dIonConcdt()                                                            'rate of change in ion concentrations
        EnergyMetabolism()                                               'Energy balance  skipped in the present version
        EulerIntegration(dt)                                                'integrate ODE using Euler Method,  renew the value in y(x)
    End Sub

    Public Sub EnergyMetabolism()
        mitConcentrations()

        mitRate()
        cytRate()
        dydt_mit()
    End Sub


    Public Sub EulerIntegration(ByVal dt As Double)                                 'Euler method

        Vm = Vm + dVmdt * dt
        If Vm > -0.0000001 And Vm < 0.0000001 Then Vm = 0.0000001

        TnChCa = TnChCa + dTnChCadt * dt                'high-binding site of Tn within blk space 
        CaMCa = CaMCa + dCaMCadt * dt                     'CaM within blk space
        bufferSRCa = bufferSRCa + dSRCadt * dt           'a Ca-buffer within the blk space

        Nai = Nai + dNaidt * dt * fixzero_slowVariable
        Ki = Ki + dKidt * dt * fixzero_slowVariable

        Catot_jnc = Catot_jnc + dCatot_jncdt * dt            'total Ca within jnc space
        Catot_iz = Catot_iz + dCatot_izdt * dt                  'total Ca within iz space
        Catot_blk = Catot_blk + dCatot_blkdt * dt            'total Ca within blk space

        Ca_SRup = Ca_SRup + dCa_SRupdt * dt              'free Ca within SRup
        Catot_SRrl = Catot_SRrl + dCatot_SRrldt * dt       'free Ca within SRrl

        '****************** INa model *******************
        O_TM = O_TM + dO_TMdt * dt                         'gating of INa_TM
        I2_TM = I2_TM + dI2_TMdt * dt                       'gating of INa_TM
        Is_TM = Is_TM + dIs_TMdt * dt                         'gating of INa_TM

        O_LSM = O_LSM + dO_LSMdt * dt                                                      'gating of INa_LM
        I1_LSM = I1_LSM + dI1_LSMdt * dt                                                    'gating of INa_LM
        I2_LSM = I2_LSM + dI2_LSMdt * dt * fixzero_INaLslowgate             'gating of INa_LM
        Is_LSM = Is_LSM + dIs_LSMdt * dt * fixzero_INaLslowgate              'gating of INa_LM

        '**************  IRyR -Hinch model,   Full number of 8 states model    dyadic space ******************************
        Yooo = Yooo + dYooodt * dt
        Yooc = Yooc + dYoocdt * dt

        Ycoo = Ycoo + dYcoodt * dt
        Ycoc = Ycoc + dYcocdt * dt

        Ycco = Ycco + dYccodt * dt

        Yoco = Yoco + dYocodt * dt
        Yocc = Yocc + dYoccdt * dt

        '*******************************  4 state model; ICaL state transition in the iz and blk spaces *************
        Yco_iz = Yco_iz + dYco_izdt * dt
        Yoc_iz = Yoc_iz + dYoc_izdt * dt
        Yoo_iz = Yoo_iz + dYoo_izdt * dt

        Yco_blk = Yco_blk + dYco_blkdt * dt
        Yoc_blk = Yoc_blk + dYoc_blkdt * dt
        Yoo_blk = Yoo_blk + dYoo_blkdt * dt

        '******************************* IKr  three gates     'ORd model**********************
        y1_IKr = y1_IKr + dy1_IKrdt * dt
        y2_IKr = y2_IKr + dy2_IKrdt * dt
        y3_IKr = y3_IKr + dy3_IKrdt * dt

        '****************** IKs   -Kyoto model *************************************************
        Ov_IKs = Ov_IKs + dOv_IKsdt * dt                 'voltage-gate

        C2IKs_iz = C2IKs_iz + dC2IKs_izdt * dt       'nrs compartment, Calcium gate with C1 - C2 - Oc states
        OcIKs_iz = OcIKs_iz + dOcIKs_izdt * dt         'nrs compartment, Calcium gate with C1 - C2 - Oc states

        C2IKs_blk = C2IKs_blk + dC2IKs_blkdt * dt        'blk compartment, Calcium gate with C1 - C2 - Oc states
        OcIKs_blk = OcIKs_blk + dOcIKs_blkdt * dt           'blk compartment, Calcium gate with C1 - C2 - Oc states

        '****************** IKto (Endo) Kyoto model *************************************************
        y1_IKto = y1_IKto + dy1_IKtodt * dt    'fast inactivation parameter
        y2_IKto = y2_IKto + dy2_IKtodt * dt    'slow inactivation parameter

        '****************** IK1   Ishihara model ********************************************
        Pbspm = Pbspm + dPbspmdt * dt                         'spermin gating of IK1, blocked state

        '****************** NCX  *************************************************
        E1NCX_iz = E1NCX_iz + dE1NCX_izdt * dt
        I1NCX_iz = I1NCX_iz + dI1NCX_izdt * dt * fixzero_slowVariable
        I2NCX_iz = I2NCX_iz + dI2NCX_izdt * dt

        E1NCX_blk = E1NCX_blk + dE1NCX_blkdt * dt
        I1NCX_blk = I1NCX_blk + dI1NCX_blkdt * dt * fixzero_slowVariable
        I2NCX_blk = I2NCX_blk + dI2NCX_blkdt * dt

        '****************** NaK pump  *************************************************
        P1_6_NaK = P1_6_NaK + dP1_6_NaKdt * dt
        P7_NaK = P7_NaK + dP7_NaKdt * dt
        P8_13_NaK = P8_13_NaK + dP8_13_NaKdt * dt

        '*****************************************  Contraction ************************************************
        halfSL = halfSL
        Fb = Fb
        Fp = Fp

        TSCa3 = TSCa3 + dTSCa3dt * dt                     'state transition in the TS   (contraction)
        TSCa3W = TSCa3W + dTSCa3Wdt * dt                    'state transition in the TS   (contraction)
        TSCa3S = TSCa3S + dTSCa3Sdt * dt                    'state transition in the TS   (contraction)
        TSS = TSS + dTSSdt * dt                    'state transition in the TS   (contraction)
        TSW = TSW + dTSWdt * dt                    'state transition in the TS   (contraction)

        hw = hw + dhwdt * dt                   'change in hw and hs
        hp = hp + dhpdt * dt                     'change in hw and hs

        '*****************************************  Mitochondria  ************************************************
        ATPt_cyt = ATPt_cyt + dATPt_cytdt * dt                         '* fixzero_slowVariable
        ADPt_cyt = ADPt_cyt + dADPt_cytdt * dt                        '* fixzero_slowVariable
        Pi_cyt = Pi_cyt + dPi_cytdt * dt                                        '* fixzero_slowVariable
        PCr_cyt = PCr_cyt + dPCr_cytdt * dt                                '* fixzero_slowVariable
        'H_cyt = H_cyt + dH_cytdt * dt                                          '* fixzero0

        ATPt_mit = ATPt_mit + dATPt_mitdt * dt                        ' * fixzero_slowVariable
        Pi_mit = Pi_mit + dPi_mitdt * dt                                       '* fixzero_slowVariable
        NADH_mit = NADH_mit + dNADH_mitdt * dt                 '* fixzero_slowVariable
        H_mit = H_mit + dH_mitdt * dt                                          '* fixzero_slowVariable
        K_mit = K_mit + dK_mitdt * dt                                          '* fixzero_slowVariable
        UQr_mit = UQr_mit + dUQr_mitdt * dt                              '* fixzero_slowVariable
        ctCrd_mit = ctCrd_mit + dctCrd_mitdt * dt                         '* fixzero_slowVariable
        'O2_mit = O2_mit + dO2_mitdt                                            '* dt * fixzero0
        dpsi = dpsi + ddpsidt * dt                                                      '* fixzero_slowVariable

    End Sub


    Public P1_dVmdt As Double
    Dim P1_dCatot_jncdt As Double
    Dim P1_dCatot_izdt As Double
    Dim P1_dCatot_blkdt As Double

    Dim P1_dNaidt As Double
    Dim P1_dKidt As Double

    Dim P1_dCa_SRupdt As Double
    Dim P1_dCatot_SRrldt As Double

    Dim P1_dO_TMdt As Double
    Dim P1_dI2_TMdt As Double
    Dim P1_dIs_TMdt As Double

    Dim P1_dO_LSMdt As Double
    Dim P1_dI1_LSMdt As Double
    Dim P1_dI2_LSMdt As Double
    Dim P1_dIs_LSMdt As Double


    Dim P1_dYooodt As Double
    Dim P1_dYoocdt As Double

    Dim P1_dYcoodt As Double
    Dim P1_dYcocdt As Double

    Dim P1_dYccodt As Double

    Dim P1_dYocodt As Double
    Dim P1_dYoccdt As Double


    Dim P1_dYco_izdt As Double
    Dim P1_dYoc_izdt As Double
    Dim P1_dYoo_izdt As Double

    Dim P1_dYco_blkdt As Double
    Dim P1_dYoc_blkdt As Double
    Dim P1_dYoo_blkdt As Double


    Dim P1_dy1_IKrdt As Double
    Dim P1_dy2_IKrdt As Double
    Dim P1_dy3_IKrdt As Double


    Dim P1_dOv_IKsdt As Double

    Dim P1_dC2IKs_izdt As Double
    Dim P1_dOcIKs_izdt As Double

    Dim P1_dC2IKs_blkdt As Double
    Dim P1_dOcIKs_blkdt As Double


    Dim P1_dy1_IKtodt As Double
    Dim P1_dy2_IKtodt As Double

    Dim P1_dPbspmdt As Double

    Dim P1_dE1NCX_izdt As Double
    Dim P1_dI1NCX_izdt As Double
    Dim P1_dI2NCX_izdt As Double

    Dim P1_dE1NCX_blkdt As Double
    Dim P1_dI1NCX_blkdt As Double
    Dim P1_dI2NCX_blkdt As Double

    Dim P1_dP1_6_NaKdt As Double
    Dim P1_dP7_NaKdt As Double
    Dim P1_dP8_13_NaKdt As Double


    Dim P1_dTSCa3dt As Double
    Dim P1_dTSCa3Wdt As Double
    Dim P1_dTSCa3Sdt As Double
    Dim P1_dTSSdt As Double
    Dim P1_dTSWdt As Double

    Dim P1_dhwdt As Double
    Dim P1_dhpdt As Double

    Dim P1_dTnChCadt As Double                 'high-binding site of Tn within blk space 
    Dim P1_dCaMCadt As Double                      'CaM within blk space
    Dim P1_dSRCadt As Double                     'a Ca-buffer within the blk space

    Public P1_Itot_Cell As Double
    Public P1_Itot_K As Double

    Public Sub copydydt()         'copy dy/dt to D9
        P1_dVmdt = dVmdt
        'old_dVodt = dVodt

        P1_dTnChCadt = dTnChCadt                 'high-binding site of Tn within blk space 
        P1_dCaMCadt = dCaMCadt                      'CaM within blk space
        P1_dSRCadt = dSRCadt                     'a Ca-buffer within the blk space

        P1_dCatot_jncdt = dCatot_jncdt
        P1_dCatot_izdt = dCatot_izdt
        P1_dCatot_blkdt = dCatot_blkdt

        P1_dNaidt = dNaidt
        P1_dKidt = dKidt

        P1_dCa_SRupdt = dCa_SRupdt
        P1_dCatot_SRrldt = dCatot_SRrldt


        P1_dO_TMdt = dO_TMdt
        P1_dI2_TMdt = dI2_TMdt
        P1_dIs_TMdt = dIs_TMdt

        P1_dO_LSMdt = dO_LSMdt
        P1_dI1_LSMdt = dI1_LSMdt
        P1_dI2_LSMdt = dI2_LSMdt
        P1_dIs_LSMdt = dIs_LSMdt


        P1_dYooodt = dYooodt
        P1_dYoocdt = dYoocdt

        P1_dYcoodt = dYcoodt
        P1_dYcocdt = dYcocdt

        P1_dYccodt = dYccodt

        P1_dYocodt = dYocodt
        P1_dYoccdt = dYoccdt


        P1_dYco_izdt = dYco_izdt
        P1_dYoc_izdt = dYoc_izdt
        P1_dYoo_izdt = dYoo_izdt

        P1_dYco_blkdt = dYco_blkdt
        P1_dYoc_blkdt = dYoc_blkdt
        P1_dYoo_blkdt = dYoo_blkdt


        P1_dy1_IKrdt = dy1_IKrdt
        P1_dy2_IKrdt = dy2_IKrdt
        P1_dy3_IKrdt = dy3_IKrdt


        P1_dOv_IKsdt = dOv_IKsdt

        P1_dC2IKs_izdt = dC2IKs_izdt
        P1_dOcIKs_izdt = dOcIKs_izdt

        P1_dC2IKs_blkdt = dC2IKs_blkdt
        P1_dOcIKs_blkdt = dOcIKs_blkdt


        P1_dy1_IKtodt = dy1_IKtodt
        P1_dy2_IKtodt = dy2_IKtodt

        P1_dPbspmdt = dPbspmdt


        P1_dE1NCX_izdt = dE1NCX_izdt
        P1_dI1NCX_izdt = dI1NCX_izdt
        P1_dI2NCX_izdt = dI2NCX_izdt

        P1_dE1NCX_blkdt = dE1NCX_blkdt
        P1_dI1NCX_blkdt = dI1NCX_blkdt
        P1_dI2NCX_blkdt = dI2NCX_blkdt

        P1_dP1_6_NaKdt = dP1_6_NaKdt
        P1_dP7_NaKdt = dP7_NaKdt
        P1_dP8_13_NaKdt = dP8_13_NaKdt

        P1_dTSCa3dt = dTSCa3dt
        P1_dTSCa3Wdt = dTSCa3Wdt
        P1_dTSCa3Sdt = dTSCa3Sdt
        P1_dTSSdt = dTSSdt
        P1_dTSWdt = dTSWdt

        P1_dhwdt = dhwdt
        P1_dhpdt = dhpdt

        P1_Itot_Cell = Itot_cell              'copy Itot_Cell for calculating P4
        P1_Itot_K = Itot_K                     'copy Itot_K for calculating P4
    End Sub


    Public Sub Averagedydt()         'calculate the average
        dVmdt = (P1_dVmdt + dVmdt) / 2
        'dVodt = (old_dVodt + dVodt) / 2
        dTnChCadt = (P1_dTnChCadt + dTnChCadt) / 2                'high-binding site of Tn within blk space 
        dCaMCadt = (P1_dCaMCadt + dCaMCadt) / 2                      'CaM within blk space
        dSRCadt = (P1_dSRCadt + dSRCadt) / 2                     'a Ca-buffer within the blk space

        dCatot_jncdt = (P1_dCatot_jncdt + dCatot_jncdt) / 2
        dCatot_izdt = (P1_dCatot_izdt + dCatot_izdt) / 2
        dCatot_blkdt = (P1_dCatot_blkdt + dCatot_blkdt) / 2

        dCa_SRupdt = (P1_dCa_SRupdt + dCa_SRupdt) / 2
        dCatot_SRrldt = (P1_dCatot_SRrldt + dCatot_SRrldt) / 2

        dNaidt = (P1_dNaidt + dNaidt) / 2
        dKidt = (P1_dKidt + dKidt) / 2

        dO_TMdt = (P1_dO_TMdt + dO_TMdt) / 2
        dI2_TMdt = (P1_dI2_TMdt + dI2_TMdt) / 2
        dIs_TMdt = (P1_dIs_TMdt + dIs_TMdt) / 2

        dO_LSMdt = (P1_dO_LSMdt + dO_LSMdt) / 2
        dI1_LSMdt = (P1_dI1_LSMdt + dI1_LSMdt) / 2
        dI2_LSMdt = (P1_dI2_LSMdt + dI2_LSMdt) / 2
        dIs_LSMdt = (P1_dIs_LSMdt + dIs_LSMdt) / 2


        dYooodt = (P1_dYooodt + dYooodt) / 2
        dYoocdt = (P1_dYoocdt + dYoocdt) / 2

        dYcoodt = (P1_dYcoodt + dYcoodt) / 2
        dYcocdt = (P1_dYcocdt + dYcocdt) / 2

        dYccodt = (P1_dYccodt + dYccodt) / 2

        dYocodt = (P1_dYocodt + dYocodt) / 2
        dYoccdt = (P1_dYoccdt + dYoccdt) / 2


        dYco_izdt = (P1_dYco_izdt + dYco_izdt) / 2
        dYoc_izdt = (P1_dYoc_izdt + dYoc_izdt) / 2
        dYoo_izdt = (P1_dYoo_izdt + dYoo_izdt) / 2

        dYco_blkdt = (P1_dYco_blkdt + dYco_blkdt) / 2
        dYoc_blkdt = (P1_dYoc_blkdt + dYoc_blkdt) / 2
        dYoo_blkdt = (P1_dYoo_blkdt + dYoo_blkdt) / 2


        dy1_IKrdt = (P1_dy1_IKrdt + dy1_IKrdt) / 2
        dy2_IKrdt = (P1_dy2_IKrdt + dy2_IKrdt) / 2
        dy3_IKrdt = (P1_dy3_IKrdt + dy3_IKrdt) / 2


        dOv_IKsdt = (P1_dOv_IKsdt + dOv_IKsdt) / 2

        dC2IKs_izdt = (P1_dC2IKs_izdt + dC2IKs_izdt) / 2
        dOcIKs_izdt = (P1_dOcIKs_izdt + dOcIKs_izdt) / 2

        dC2IKs_blkdt = (P1_dC2IKs_blkdt + dC2IKs_blkdt) / 2
        dOcIKs_blkdt = (P1_dOcIKs_blkdt + dOcIKs_blkdt) / 2


        dy1_IKtodt = (P1_dy1_IKtodt + dy1_IKtodt) / 2
        dy2_IKtodt = (P1_dy2_IKtodt + dy2_IKtodt) / 2

        dPbspmdt = (P1_dPbspmdt + dPbspmdt) / 2


        dE1NCX_izdt = (P1_dE1NCX_izdt + dE1NCX_izdt) / 2
        dI1NCX_izdt = (P1_dI1NCX_izdt + dI1NCX_izdt) / 2
        dI2NCX_izdt = (P1_dI2NCX_izdt + dI2NCX_izdt) / 2

        dE1NCX_blkdt = (P1_dE1NCX_blkdt + dE1NCX_blkdt) / 2
        dI1NCX_blkdt = (P1_dI1NCX_blkdt + dI1NCX_blkdt) / 2
        dI2NCX_blkdt = (P1_dI2NCX_blkdt + dI2NCX_blkdt) / 2

        dP1_6_NaKdt = (P1_dP1_6_NaKdt + dP1_6_NaKdt) / 2
        dP7_NaKdt = (P1_dP7_NaKdt + dP7_NaKdt) / 2
        dP8_13_NaKdt = (P1_dP8_13_NaKdt + dP8_13_NaKdt) / 2

        dTSCa3dt = (P1_dTSCa3dt + dTSCa3dt) / 2
        dTSCa3Wdt = (P1_dTSCa3Wdt + dTSCa3Wdt) / 2
        dTSCa3Sdt = (P1_dTSCa3Sdt + dTSCa3Sdt) / 2
        dTSSdt = (P1_dTSSdt + dTSSdt) / 2
        dTSWdt = (P1_dTSWdt + dTSWdt) / 2

        dhwdt = (P1_dhwdt + dhwdt) / 2
        dhpdt = (P1_dhpdt + dhpdt) / 2


    End Sub


    Public Sub P1_YtoY()         'copy variable y to D9
        Vm = P1_Vm

        TnChCa = P1_TnChCa                  'high-binding site of Tn within blk space 
        CaMCa = P1_CaMCa                  'CaM within blk space
        bufferSRCa = P1_bufferSRCa            'a Ca-buffer within the blk space

        Catot_jnc = P1_Catot_jnc
        Catot_iz = P1_Catot_iz
        Catot_blk = P1_Catot_blk

        Nai = P1_Nai
        Ki = P1_Ki

        Ca_SRup = P1_Ca_SRup
        Catot_SRrl = P1_Catot_SRrl


        O_TM = P1_O_TM
        I2_TM = P1_I2_TM
        Is_TM = P1_Is_TM

        O_LSM = P1_O_LSM
        I1_LSM = P1_I1_LSM
        I2_LSM = P1_I2_LSM
        Is_LSM = P1_Is_LSM


        Yooo = P1_Yooo
        Yooc = P1_Yooc

        Ycoo = P1_Ycoo
        Ycoc = P1_Ycoc

        Ycco = P1_Ycco

        Yoco = P1_Yoco
        Yocc = P1_Yocc


        Yco_iz = P1_Yco_iz
        Yoc_iz = P1_Yoc_iz
        Yoo_iz = P1_Yoo_iz

        Yco_blk = P1_Yco_blk
        Yoc_blk = P1_Yoc_blk
        Yoo_blk = P1_Yoo_blk


        y1_IKr = P1_y1_IKr
        y2_IKr = P1_y2_IKr
        y3_IKr = P1_y3_IKr


        Ov_IKs = P1_Ov_IKs

        C2IKs_iz = P1_C2IKs_iz
        OcIKs_iz = P1_OcIKs_iz

        C2IKs_blk = P1_C2IKs_blk
        OcIKs_blk = P1_OcIKs_blk


        y1_IKto = P1_y1_IKto
        y2_IKto = P1_y2_IKto

        Pbspm = P1_Pbspm

        E1NCX_iz = P1_E1NCX_iz
        I1NCX_iz = P1_I1NCX_iz
        I2NCX_iz = P1_I2NCX_iz

        E1NCX_blk = P1_E1NCX_blk
        I1NCX_blk = P1_I1NCX_blk
        I2NCX_blk = P1_I2NCX_blk


        P1_6_NaK = P1_P1_6_NaK
        P7_NaK = P1_P7_NaK
        P8_13_NaK = P1_P8_13_NaK


        TSCa3 = P1_TSCa3
        TSCa3W = P1_TSCa3W
        TSCa3S = P1_TSCa3S
        TSS = P1_TSS
        TSW = P1_TSW

        hw = P1_hw
        hp = P1_hp

    End Sub

    '********** Injection current **********
    Public clampMode As String             'decide if V clamp or I clamp
    Public Icompensation As Double         'current applied from external device, assumed to be carried by K+

    '***** current clmap *****
    Public Samplitude As Double               'pA     current amplitude 
    Public ItestOn As Double = 50             'ms,  start time of external stimulation
    Public ItestOff As Double = 53             'ms,  end time of external stimulation
    Public SInterval As Double                   'ms,   stimulus interval for a paired pulse application
    Public gap As Double = 0                       'ms,

    '***** voltage clmap *****
    Public Vhold As Double                          'mV, holding potential
    Public Vtest As Double                           'mV, test voltage
    Public testV As Double                           'mV, test voltage
    Public VtestInterval As Double = 4        'mV    interval of voltage clamp test pulse for determining the I-V relationship
    Public VtestOn As Double                      'ms, start of the voltage pulse
    Public VtestOff As Double                      'ms,   end of the voltage pulse

    Private Sub Iclampsub(ByRef t As Double)      'current clamp experiment
        Icompensation = 0
        If t >= ItestOn And t < ItestOff Then
            Icompensation = Samplitude
        Else
            Icompensation = 0
        End If

    End Sub

    Private Sub Vclampsub(ByRef t As Double, ByVal vtest As Double, ByVal dt As Double)   'voltage clamp experiments
        Dim fbgain As Double = 1 / (2 * dt)                       'A 'feedback gain' is determined by the (x*dt) x=2 is best so far examined
        If t >= VtestOn And t < VtestOff Then
            Icompensation = -(vtest - Vm) * fbgain
        Else
            Icompensation = -(Vhold - Vm) * fbgain
        End If
    End Sub

    Public Sub MembraneCurrent(ByVal v As Double)
        NernstGHK(v)

        INarate(v)
        INasub(v, GHKNa, GHKK, INaT_Na_cyt, INaT_K_cyt, INaL_Na_cyt, INaL_K_cyt)

        INaT = INaT_Na_cyt + INaT_K_cyt
        INaL = INaL_Na_cyt + INaL_K_cyt
        INa = INaT + INaL

        ICaL4stateGating(v, Cafree_iz, Yco_iz, Yoo_iz, Yoc_iz, dYco_izdt, dYoo_izdt, dYoc_izdt)        'iz space
        ICaL4stateAmplitude(FrcCaL_iz, Yoo_iz, GHKCa_iz, GHKNa, GHKK, ICaLCa_iz, ICaLNa_iz, ICaLK_iz)

        ICaL4stateGating(v, Cafree_blk, Yco_blk, Yoo_blk, Yoc_blk, dYco_blkdt, dYoo_blkdt, dYoc_blkdt)    'blk space
        ICaL4stateAmplitude(FrcCaL_blk, Yoo_blk, GHKCa_blk, GHKNa, GHKK, ICaLCa_blk, ICaLNa_blk, ICaLK_blk)

        ICaL = ICaLCa_LR + ICaLCa_L0 + ICaLNa_jnc + ICaLK_jnc + ICaLCa_iz + ICaLNa_iz + ICaLK_iz + ICaLCa_blk + ICaLNa_blk + ICaLK_blk
        ICaL_jnc = ICaLCa_LR + ICaLCa_L0 + ICaLNa_jnc + ICaLK_jnc
        ICaL_iz = ICaLCa_iz + ICaLNa_iz + ICaLK_iz
        ICaL_blk = ICaLCa_blk + ICaLNa_blk + ICaLK_blk

        ICabsub(Frc_iz, GHKCa_iz, ICab_iz)
        ICabsub(Frc_blk, GHKCa_blk, ICab_blk)
        ICab = ICab_iz + ICab_blk

        IK1sub(v, EK, Pbspm, dPbspmdt)                                   'IK1
        IK1 = IK1_K_cyt

        IKrsub(v)
        IKr = IKr_K_cyt

        VgateIKs(v, Ov_IKs, dOv_IKsdt) ''IKs
        IKsKyotosub(Cafree_iz, Frc_iz, IKs_K_iz, IKs_Na_iz, Ov_IKs, C2IKs_iz, OcIKs_iz, dOcIKs_izdt, dC2IKs_izdt, pO_IKs_iz)
        IKsKyotosub(Cafree_blk, Frc_blk, IKs_K_blk, IKs_Na_blk, Ov_IKs, C2IKs_blk, OcIKs_blk, dOcIKs_blkdt, dC2IKs_blkdt, pO_IKs_blk)

        IKs = IKs_K_iz + IKs_K_blk + IKs_Na_iz + IKs_Na_blk
        IKs_iz = IKs_K_iz + IKs_Na_iz
        IKs_blk = IKs_K_blk + IKs_Na_blk

        '*************************  IKto_endo *************** 
        IKtoEndosub(v, IKto_K_cyt, IKto_Na_cyt, y1_IKto, y2_IKto, dy1_IKtodt, dy2_IKtodt)
        IKto = IKto_K_cyt + IKto_Na_cyt
        'IKtoEpisub(Vm, IKto_K_cyt, IKto_Na_cyt, y1_IKto, y2_IKto, dy1_IKtodt, dy2_IKtodt)
        'IKto = IKto_Na_cyt + IKto_K_cyt

        '***********************************************************************
        IKplsub(v)                                    'IK plateau   Kyoto model
        IKpl = IKpl_K_cyt

        IbNSCsub()
        IbNSC = IbNSC_K_cyt + IbNSC_Na_cyt

        ILCasub(Cafree_iz, Frc_iz, ILCa_Na_iz, ILCa_K_iz, pO_LCa_iz)
        ILCasub(Cafree_blk, Frc_blk, ILCa_Na_blk, ILCa_K_blk, pO_LCa_blk)
        ILCa = ILCa_Na_iz + ILCa_K_iz + ILCa_Na_blk + ILCa_K_blk
        ILCa_iz = ILCa_Na_iz + ILCa_K_iz
        ILCa_blk = ILCa_Na_blk + ILCa_K_blk

        IKATPsub(v)
        IKATP = IKATPK_cyt

        INaKsub(1, v, Nai, Ki, MgATP_cyt, MgADP_cyt, INaK, INaK_Na_cyt, INaK_K_cyt, dATP_NaK, P1_6_NaK, P7_NaK, P8_13_NaK, dP1_6_NaKdt, dP7_NaKdt, dP8_13_NaKdt)

        INCXsub(FrcNCX_iz, v, Nai, Cafree_iz, INCX_iz, INCXNa_iz, INCXCa_iz, E1NCX_iz, I1NCX_iz, I2NCX_iz, dE1NCX_izdt, dI1NCX_izdt, dI2NCX_izdt)
        INCXsub(FrcNCX_blk, v, Nai, Cafree_blk, INCX_blk, INCXNa_blk, INCXCa_blk, E1NCX_blk, I1NCX_blk, I2NCX_blk, dE1NCX_blkdt, dI1NCX_blkdt, dI2NCX_blkdt)
        INCX = INCXNa_iz + INCXCa_iz + INCXNa_blk + INCXCa_blk    '+  INCXNa_jnc + INCXCa_jnc 

        IPMCAsub(Frc_iz, Cafree_iz, IPMCA_Ca_iz, dATP_PMCA_iz)                              'PMCA
        IPMCAsub(Frc_blk, Cafree_blk, IPMCA_Ca_blk, dATP_PMCA_blk)
        IPMCA = IPMCA_Ca_iz + IPMCA_Ca_blk

    End Sub

    '********** Membrane potential **************************************************************************************************************
    Public Vm As Double             'mV, membrane potential
    Public Vrest As Double          'mV, resting membrane potential
    Public Vmaxpl As Double      'mV, used to measure the maximum plateau  potential (10 ms after the offset of stimulus current pulse)

    'Public Vcom As Double          'mV, command potential for the voltage clamp experiments

    Public dVmdt As Double           'mV/ms ,   d(Vm)/dt
    Public Itot_Ca As Double          'total current of Calcium ion
    Public Itot_Na As Double          'total current of Sodium ion
    Public Itot_K As Double             '=Ito + IKr + IKs +IK1 - 2*INaK + ICaK + IKpl
    Public Itot_cell As Double          'total current  = Itot_Ca + Itot_Na + Itot_K

    Public Sig_INa As Double = 0             'Sum of INa * dt
    Public Sig_INCXNa As Double = 0     'Sum of INCX_Na * dt
    Public Sig_allNa As Double = 0          'Sum of all Na-mediated current * dt
    Public IsMClamp As Boolean

    Public Sub MembranePotential(ByVal sweeptime As Double, ByVal dt As Double)
        'Compositions of each total ionic current of Ca, Na, or K   
        ItotCa_jnc = ICaLCa_LR + ICaLCa_L0                                                            '
        ItotCa_iz = ICaLCa_iz + IPMCA_Ca_iz + INCXCa_iz + ICab_iz                 '
        ItotCa_blk = ICaLCa_blk + IPMCA_Ca_blk + INCXCa_blk + ICab_blk    '

        ItotNa_cyt = (ICaLNa_jnc + ICaLNa_iz + ICaLNa_blk) + (INCXNa_iz + INCXNa_blk) + (IKs_Na_iz + IKs_Na_blk) _
            + INaT_Na_cyt + INaL_Na_cyt + INaK_Na_cyt + IKto_Na_cyt + IbNSC_Na_cyt + ILCa_Na_iz + ILCa_Na_blk                                        '

        ItotK_cyt = (ICaLK_jnc + ICaLK_iz + ICaLK_blk) + INaT_K_cyt + INaL_K_cyt + IK1_K_cyt + IKr_K_cyt + (IKs_K_iz + IKs_K_blk) _
            + IKto_K_cyt + IKpl_K_cyt + INaK_K_cyt + IKATPK_cyt + IbNSC_K_cyt + ILCa_K_iz + ILCa_K_blk                                                         '

        '******************** Integrate currents **************************
        Sig_INa = Sig_INa + INaT_Na_cyt + INaL_Na_cyt
        Sig_INCXNa = Sig_INCXNa + (ICaLCa_LR + ICaLCa_L0 + ICaLCa_iz + ICaLCa_blk) * 3 / 2      'convert Ca flux to Na flux
        Sig_allNa = Sig_allNa + (ICaLCa_LR + ICaLCa_L0 + ICaLCa_iz + ICaLCa_blk) * 3 / 2 + INaT_Na_cyt + INaL_Na_cyt _
        + (IKs_Na_iz + IKs_Na_blk) + IKto_Na_cyt + IbNSC_Na_cyt + (ILCa_Na_iz + ILCa_Na_blk) + ICab_blk * 3 / 2

        '*******************  In case of freezing the membrane ion fluxes ***********************************************
        If IsMClamp = True Then
            ItotCa_jnc = 0          'freeze membrane Ca flux
            ItotCa_iz = 0            'freeze membrane Ca flux
            ItotCa_blk = 0           'freeze membrane Ca flux
            ItotNa_cyt = 0           'freeze membrane Na flux
            ItotK_cyt = 0             'freeze membrane K flux
            Samplitude = 0          'no stimulation ,   spontaneous action potential
        End If
        '*************************************************************************************************************
        Itot_Ca = ItotCa_jnc + ItotCa_iz + ItotCa_blk                  '
        Itot_Na = ItotNa_cyt
        Itot_K = ItotK_cyt

        Itot_cell = Itot_Ca + Itot_Na + Itot_K                     '

        If clampMode = "Iclamp" Then
            Iclampsub(sweeptime)                'calculate the stimulus current applied between  times onset and  offset
        ElseIf clampMode = "Vclamp" Then
            Vclampsub(sweeptime, Vtest, dt)    'voltage clamp protocol
        End If

        dVmdt = -Itot_cell - Icompensation                              'mV/ms

    End Sub

    Public Sub dIonConcdt()

        If fixCaconc = "true" Then Jrel_SR = 0 'assuming that SR Ca release is blocked because SR Ca is depleted when Ca-chelating agent is applied into the cell.

        dCatot_jncdt = (-ItotCa_jnc * Cm / (2 * Faraday) + Jrel_SR - JCa_jnciz) / Vol_jnc
        dCatot_izdt = (-ItotCa_iz * Cm / (2 * Faraday) + JCa_jnciz - JCa_izblk) / Vol_iz
        dCatot_blkdt = (-ItotCa_blk * Cm / (2 * Faraday) - JSERCA + JCa_izblk) / Vol_blk

        dNaidt = -Itot_Na * Cm / (Vol_cyt * Faraday)

        dKidt = -(Itot_K + Icompensation) * Cm / (Vol_cyt * Faraday)

        dCa_SRupdt = (JSERCA - Jtrans_SR) / Vol_SRup
        dCatot_SRrldt = (Jtrans_SR - Jrel_SR) / Vol_SRrl

    End Sub

    Public Sub SRCadynamics()
        '***********' unit : amole / ms  *********************************************************
        TransferCaSR(Ca_SRup, Cafree_SRrl, Jtrans_SR)
        CaRU(Vm, FrcCaL_jnc, GHKNa, GHKK, ICaLCa_LR, ICaLCa_L0, ICaLNa_jnc, ICaLK_jnc, Jrel_SR)
        SERCATC(Cafree_blk, Ca_SRup, JSERCA, dATP_SERCA)       'Tran Crampin model
    End Sub

    '********** SR fluxe: RyR-kinetics, SR Ca pump, SR Ca leak **********
    Public Jrel_SR As Double
    Public JSERCA As Double
    Public JSERCA_TC As Double            'Tran Crampin model is used
    '    Public Jleak_SR As Double


    Public Ptrans As Double = 4.8037 * Sc_Cell
    Public Jtrans_SR As Double                 'Transfer of Ca from uptake site to release site
    Public freezeSR As Double = 1

    Private Sub TransferCaSR(ByVal Ca_SRup As Double, ByVal Ca_SRrl As Double, ByRef Jtrans_SR As Double)
        Jtrans_SR = Ptrans * (Ca_SRup - Ca_SRrl) * freezeSR
    End Sub

    '******* modified version of Tran K., Smith N.P., DS Loiselle & EJ Crampin, 2009  Biophysical Journal 96:2029-2042 ************
    Public maxISERCA As Double = 106444.8 * Sc_Cell

    Public dATP_SERCA As Double
    Public dATP_SERCA_TC As Double
    Dim Hill As Double = 1.7                           '
    Dim Kd_Cai As Double = 0.0027            '
    Dim Kd_Casr As Double = 1.378                '

    Public Sub SERCATC(ByVal Cai As Double, ByVal Casr As Double, ByRef JSERCA As Double, ByRef dATP_SERCA As Double)

        Dim a1 As Double = 25900 * MgATP_cyt                                              '
        Dim a2 As Double = 2540 / (1 + Math.Pow(Kd_Cai / Cai, Hill))         '
        Dim a3 As Double = 5.35 / (1 + Math.Pow(Casr / Kd_Casr, Hill))            '

        Dim b1 As Double = 0.1972 / (1 + Math.Pow(Cai / Kd_Cai, Hill))                 '
        Dim b2 As Double = 25435 * MgADP_cyt / (1 + Math.Pow(Kd_Casr / Casr, Hill))    '
        Dim b3 As Double = 149 * Pifree_cyt

        Dim denomi As Double = a2 * a3 + b1 * a3 + b1 * b2 + a1 * a3 + b2 * a1 + b2 * b3 + a1 * a2 + b3 * b1 + b3 * a2     '
        Dim numer As Double = a1 * a2 * a3 - b1 * b2 * b3                                                                                                               '
        Dim Vcyc As Double = numer / denomi * 6.86                                                                                                                        '
        Dim ampFactor As Double = 1                           'scaling factor of Vmax in NCX
        JSERCA = (ampFactor * maxISERCA * Vcyc) / (2 * Faraday) * freezeSR / 1000                           '
        dATP_SERCA = JSERCA / 2                       'stoichiometry = 2Ca / 1ATP      amole / ms

    End Sub

    Public Sub NernstGHK(ByVal v As Double)

        GHKNernst(ECa_jnc, ErevCa_LR, GHKCa_LR, dGHKCa_LR, v, Ca_ndLR, Cao, 2)
        GHKNernst(ECa_jnc, ErevCa_L0, GHKCa_L0, dGHKCa_L0, v, Ca_ndL0, Cao, 2)
        GHKNernst(ECa_iz, ErevCa_iz, GHKCa_iz, dGHKCa_iz, v, Cafree_iz, Cao, 2)
        GHKNernst(ECa_blk, ErevCa_blk, GHKCa_blk, dGHKCa_blk, v, Cafree_blk, Cao, 2)
        GHKNernst(ENa, ErevNa, GHKNa, dGHKNa, v, Nai, Nao, 1)
        GHKNernst(EK, ErevK, GHKK, dGHKK, v, Ki, Ko, 1)
    End Sub

    Public Sub CheckV0(ByRef v As Double)
        'to avoid 0 error, 0 is replaced
        'If v > -0.0001 And v <= 0 Then
        '    v = -0.0001
        'ElseIf v < 0.0001 And v > 0 Then
        '    v = 0.0001
        'End If
    End Sub

    Public Sub GHKNernst(ByRef ES As Double, ByRef Erev As Double, ByRef GHKS As Double, ByRef dGHKS As Double, ByVal v As Double, ByVal Si As Double, ByVal Se As Double, ByVal z As Double)
        ES = RTF / z * Log(Se / Si)

        If v > -0.00001 And v <= 0 Then
            v = -0.00001
        ElseIf v < 0.00001 And v > 0 Then
            v = 0.00001
        End If
        Dim B As Double = z * v / RTF
        Dim A As Double = Exp(-B)

        GHKS = B * (Si - Se * A) / (1 - A)      'GHK equation removed for P * zF
        dGHKS = ((Si - Se * A) / (1 - A) + (Se - Si) * B * A / (1 - A) ^ 2) / (RTF / z)
        Erev = v - GHKS / dGHKS
    End Sub

    'Public Sub NernstGHKOLD(ByVal v As Double, ByRef GHKNa As Double, ByRef GHKK As Double, ByRef GHKCa_jnc As Double, ByRef GHKCa_iz As Double, ByRef GHKCa_blk As Double, ByRef ENa As Double, ByRef EK As Double, ByRef ECa_jnc As Double, ByRef ECa_iz As Double, ByRef ECa_blk As Double)
    '    'Nernst equilibrium potentials
    '    ECa_jnc = RTF2 * Log(Cao / Ca_jnc)  'Note that LCC uses Ca_jnc, not Ca_nd
    '    ECa_iz = RTF2 * Log(Cao / Ca_iz)
    '    ECa_blk = RTF2 * Log(Cao / Ca_blk)
    '    ENa = RTF * Log(Nao / Nai)
    '    EK = RTF * Log(Ko / Ki)

    '    Dim A As Double = Exp(-v / RTF2)
    '    Dim B As Double = Exp(-v / RTF)
    '    Dim C As Double = v / RTF
    '    Dim D As Double = v / RTF2

    '    If v > -0.00001 And v < 0.00001 Then                    '0 cannot be used.
    '        GHKCa_jnc = Ca_jnc - Cao  'Note that LCC uses Ca_jnc, not Ca_nd
    '        GHKCa_iz = Ca_iz - Cao
    '        GHKCa_blk = Ca_blk - Cao
    '        GHKNa = Nai - Nao
    '        GHKK = Ki - Ko
    '    Else
    '        GHKCa_jnc = C * 2 * (Ca_jnc - Cao * A) / (1 - A)      'GHK equation removed for P * zF
    '        GHKCa_iz = C * 2 * (Ca_iz - Cao * A) / (1 - A)
    '        GHKCa_blk = C * 2 * (Ca_blk - Cao * A) / (1 - A)
    '        GHKNa = C * (Nai - Nao * B) / (1 - B)
    '        GHKK = C * (Ki - Ko * B) / (1 - B)
    '    End If

    '    '************ tangential slope applied to the GHK curve ************************
    '    dGHKCa_jnc = (((Ca_jnc - Cao * A) / (1 - A) + (Cao - Ca_jnc) * D * A / (1 - A) ^ 2) / RTF2)    'tangential slope = differentiated GHK eq
    '    dGHKCa_iz = (((Ca_iz - Cao * A) / (1 - A) + (Cao - Ca_iz) * D * A / (1 - A) ^ 2) / RTF2)          'tangential slope = differentiated GHK eq
    '    dGHKCa_blk = (((Ca_blk - Cao * A) / (1 - A) + (Cao - Ca_blk) * D * A / (1 - A) ^ 2) / RTF2)    'tangential slope = differentiated GHK eq
    '    dGHKNa = (((Nai - Nao * B) / (1 - B) + (Nao - Nai) * C * B / (1 - B) ^ 2) / RTF)          'tangential slope of Na GHK eq
    '    dGHKK = (((Ki - Ko * B) / (1 - B) + (Ko - Ki) * C * B / (1 - B) ^ 2) / RTF)                    'tangential slope of K GHK eq

    '    ErevCa_jnc = v - GHKCa_jnc / dGHKCa_jnc      'Erev of the tangential slope
    '    ErevCa_iz = v - GHKCa_iz / dGHKCa_iz            'Erev of the tangential slope
    '    ErevCa_blk = v - GHKCa_blk / dGHKCa_blk      'Erev of the tangential slope
    '    ErevNa = v - GHKNa / dGHKNa                         'Erev of the tangential slope
    '    ErevK = v - GHKK / dGHKK                               'Erev of the tangential slope

    'End Sub



    Public Sub IonHomeoStasis()              'Ion concentrations
        CaBuffblk()
        CaBuffiz()
        CaBuffjnc()
        CaBuffSRrl()
        BoundaryDiffusion()
    End Sub

    '**********************  Myoplasmic Calcium Buffers , based on Grandi, Pasqualini & Bers 2010, J Mol Cell Cardiol 48:112********************************************
    Public KdMsnCa As Double             'Msn dissociation constant for Ca
    Public KdMsnMg As Double            'Msn dissociation constnat for Mg
    Public Msnfree As Double               'free Msn
    Public MsnCa As Double
    Public MsnMg As Double
    Public dMsnCadt As Double
    Public dMsnMgdt As Double

    Public JCaB_blk As Double

    Public Lb_jnc As Double
    Public Hb_jnc As Double
    Public dLb_jncdt As Double
    Public dHb_jncdt As Double

    Public JCaB_jnc As Double       '=SLLj + SLHj

    Public Lb_iz As Double                'bound form
    Public Hb_iz As Double                 'bound form
    Public dLb_izdt As Double                'bound form
    Public dHb_izdt As Double                 'bound form

    Public JCaB_iz As Double         '=SLLsl + SLHsl

    Public H_iz As Double                  'free form
    Public dHbjncdt As Double
    Public dHbizdt As Double

    '********** Buffer  parameters **********
    Const BtotCaM As Double = 0.024                 'mM
    Public CaMfree As Double                             'free CaM
    Public CaMCa As Double                               'Ca bound form
    Public dCaMCadt As Double                         'dy/dt
    Const KonCaM As Double = 34                       'ms^-1mM^-1    on rate
    Const KoffCaM As Double = 0.238                 'ms^-1                off rate
    Public KdCaM As Double                               'CaM  dissociation constant for Ca

    Const BtotTnCh As Double = 0.12                     'mM
    Public TnChfree As Double                               'free TnCh
    Public TnChCa As Double
    Public TnChMg As Double
    Public dTnChCadt As Double
    Public dTnChMgdt As Double
    Const KoffTnChCa As Double = 0.000032      'ms^-1
    Const KonTnChCa As Double = 2.37              'ms^-1mM^-1
    Public KdTnChCa As Double                         'TnCh dissociation constant for Ca
    Public KdTnChMg As Double                        'TnCh dissociation constant for Mg

    Const BtotSR As Double = 0.0171                  '19 * 0.0009           'mM
    Public SRfree As Double                                 'free SR
    Public bufferSRCa As Double
    Public dSRCadt As Double
    Const KoffSR As Double = 0.06                       'ms^-1
    Const KonSR As Double = 100                        'ms^-1mM^-1
    Public KdSRCa As Double                   'unspecified SR Ca binding, dissociation constant for Ca

    Const Btot_Csqn As Double = 3
    Const KoffCsqn As Double = 65                      'ms^-1
    Const KonCsqn As Double = 100                    'ms^-1mM^-1

    Private Sub CaBuffblk()

        Cafree_blk = Catot_blk - (CaMCa + TnChCa + bufferSRCa + 3 * (TSCa3 + TSCa3W + TSCa3S) / 1000)  '

        dCaMCadt = KonCaM * Cafree_blk * (BtotCaM - CaMCa) - KoffCaM * CaMCa                               '
        dTnChCadt = KonTnChCa * Cafree_blk * (BtotTnCh - TnChCa) - KoffTnChCa * TnChCa           '
        dSRCadt = KonSR * Cafree_blk * (BtotSR - bufferSRCa) - KoffSR * bufferSRCa                             '

        If fixCaconc = True Then                  'in case of using enough amount of any artificail Ca buffer
            Cafree_blk = 0.00001                    'arbitrary value, but good enough
        End If
    End Sub

    Const BtotL_jnc As Double = 1.1095            ' '0.00046 * Vol_blk / Vol_jnc      'mM   Note, the on- and off-rates are the same as in sl
    Const BtotH_jnc As Double = 0.398              '  '0.000165 * Vol_blk / Vol_jnc    'mM
    Dim Lf_jnc As Double
    Dim Hf_jnc As Double

    Private Sub CaBuffjnc()                   'This is used to calculate the instantaneous equilibrium of binding reactions.

        'Cafree_jnc = Catot_jnc - (Lb_jnc + Hb_jnc)        '

        Dim N As Double = 0
        Do
            Cafree_jnc = Catot_jnc / (1 + Lf_jnc / KdL_iz + Hf_jnc / KdH_iz)      '

            Lf_jnc = BtotL_jnc / (1 + (Cafree_jnc / KdL_iz))                                    '
            Hf_jnc = BtotH_jnc / (1 + (Cafree_jnc / KdH_iz))                                  '
            N = N + 1
        Loop While N < 10                                                              '.

        Lb_jnc = BtotL_jnc - Lf_jnc                    '
        Hb_jnc = BtotH_jnc - Hf_jnc                  '

        If fixCaconc = True Then                                'in case of using any artificial Ca-buffer
            'Cafree_jnc = 0.00001
        End If

    End Sub

    Const BtotL_iz As Double = 0.6078          '0.0374 * Vol_blk / Vol_iz        'mM
    Const koffL_iz As Double = 1.3                            'ms^-1    sarcolemmal Ca buffer with a low affinity
    Const KonL_iz As Double = 100                             'ms^-1mM^-1   sarcolemmal Ca buffer with a low affinity
    Const BtotH_iz As Double = 0.2178          '0.0134 * Vol_blk / Vol_iz       'mM
    Const KoffH_iz As Double = 0.03                           'ms^-1
    Const KonH_iz As Double = 100                             'ms^-1mM^-1

    Dim KdL_iz As Double = koffL_iz / KonL_iz                   'dissociation constant of the low affinity buffer in the iz/jnc space
    Dim KdH_iz As Double = KoffH_iz / KonH_iz                'dissociation constant of the high affinity buffer in the iz/jnc space
    Dim Lf_iz As Double
    Dim Hf_iz As Double

    Private Sub CaBuffiz()                  'This is used to calculate the instantaneous equilibrium of binding reactions.

        'Cafree_iz = Catot_iz - (Lb_iz + Hb_iz)           '
        'Lf_iz = BtotL_iz - Lb_iz                                    '
        'Hf_iz = BtotH_iz - Hb_iz                                  '

        Dim N As Long = 0                                                                              'counter of the Do Loop
        Do
            Cafree_iz = Catot_iz / (1 + Lf_iz / KdL_iz + Hf_iz / KdH_iz)      '

            Lf_iz = BtotL_iz / (1 + (Cafree_iz / KdL_iz))                            '
            Hf_iz = BtotH_iz / (1 + (Cafree_iz / KdH_iz))                          '
            N = N + 1
        Loop While N < 10                                                              '10 cycles seems to be good enough.

        Lb_iz = BtotL_iz - Lf_iz                    '
        Hb_iz = BtotH_iz - Hf_iz                  '

        If fixCaconc = True Then                  'in case of using the artificial Ca buffer
            'Cafree_iz = 0.00001
        End If


    End Sub

    Private Sub CaBuffSRrl()

        Dim KdCsqnCa As Double = KoffCsqn / KonCsqn
        Dim aa As Double = 1
        Dim bb As Double = Btot_Csqn - Catot_SRrl + KdCsqnCa                 '
        Dim cc As Double = -KdCsqnCa * Catot_SRrl                                     '
        Cafree_SRrl = (-bb + Sqrt(bb * bb - 4 * aa * cc)) / (2 * aa)                 '

    End Sub

    '********** Diffusion between spaces **********
    Public Const GCa_jnciz As Double = 3395.88 * Sc_Cell                 '3215.8 * 0.5 * Sc_Vol    '459.4 * 20 * 0.7 * 0.5 '824.13054227789689 * ratioCm * ratioVolume    '1 / 0.0012134
    Public Const GCa_izblk As Double = 3507.78 * Sc_Cell                  '2076.1 * 0.8 * Sc_Vol     '3724.25607984805 * ratioCm * ratioVolume                                       '1 / 0.00026851

    Public JCa_jnciz As Double
    Public JCa_izblk As Double

    Private Sub BoundaryDiffusion()
        JCa_jnciz = GCa_jnciz * (Cafree_jnc - Cafree_iz)                         '
        JCa_izblk = GCa_izblk * (Cafree_iz - Cafree_blk)                        '
    End Sub

    '''***************************************' INa   *****************************************
    Public INa As Double                          'pA, current amplitude at the time of observation
    Public sclI1I2 As Double = 1                                             'scaling factor for the slow inactivation rate of INaL
    '********* inward current amplitudes **************  current density **************

    Public INaT As Double                         'pA, current amplitude
    Public INaL As Double                         'pA, current amplitude
    Public LSMratio As Double = 0.175 * 0.75        '0.9  0.75

    Public INaT_Na_cyt As Double             'pA, amplitude of Na component
    Public INaT_K_cyt As Double               'pA, amplitude of K component
    Public INaL_Na_cyt As Double             'pA, amplitude of Na component
    Public INaL_K_cyt As Double               'pA, amplitude of K component


    Public fC_INa As Double

    Public pO_INa_TM As Double
    Public pO_INa_LSM As Double
    Public C_TM As Double             'transient mode of INa       pC_TM
    Public O_TM As Double             'transient mode of INa       pO_TM
    Public I2_TM As Double             'transient mode of INa       pI2_TM
    Public Is_TM As Double             'transient mode of INa       pIs_TM

    Public dO_TMdt As Double             'transient mode of INa   dpO_TMdt
    Public dI2_TMdt As Double             'transient mode of INa   dpI2_TMdt
    Public dIs_TMdt As Double             'transient mode of INa   dpIs_TMdt

    Public C_LSM As Double             'LSM mode of INa
    Public O_LSM As Double             'LSM mode of INa
    Public I1_LSM As Double             'LSM mode of INa
    Public I2_LSM As Double             'LSM mode of INa
    Public Is_LSM As Double             'LSM mode of INa
    '   Public Iinact_LSM As Double        'Is_LSM  used in calculating the steady state I-V

    Public fixedIs_LSM As Double

    Public dO_LSMdt As Double
    Public dI1_LSMdt As Double
    Public dI2_LSMdt As Double
    Public dIs_LSMdt As Double

    Public kC2O_INa, kOC_INa, kOI2_TM, kI2O_TM, kC2I2_TM, kI2C_TM, kIsb_TM, kIsf_TM As Double
    Public kI1I2, kOI1_LSM, kI1O_LSM, kI1C_LSM, kC2I1_LSM, kC2I2_LSM, kI2C_LSM, kIsb_LSM, kIsf_LSM As Double

    Private Sub INarate(ByVal v As Double)
        '************************************ pure Voltage-dependent gating ******************************************************* 

        kC2O_INa = 1.0 / (0.0025 * Exp(-v / 8.0) + 0.15 * Exp(-v / 100.0))           '
        kOC_INa = 1.0 / (30.0 * Exp(v / 12.0) + 0.53 * Exp(v / 50.0))                    '

        kOI2_TM = 1.0 / (0.0433 * Exp(-v / 27) + 0.34 * Exp(-v / 2000.0))           '
        kI2O_TM = 0.0001312                                                                                  '

        kC2I2_TM = 0.5 / (1.0 + kI2O_TM * kOC_INa / kOI2_TM / kC2O_INa)           '
        kI2C_TM = 0.5 - kC2I2_TM                                                                                      '

        kIsb_TM = 1.0 / (300000 * Exp(v / 10) + 50000.0 * Exp(v / 16.0))             '     
        kIsf_TM = 1.0 / (0.016 * Exp(-v / 9.9) + 8 * Exp(-v / 45.0))                          '

        '*******************  Calculate state transitions of TM ***************************************************************
        C_TM = 1.0 - Is_TM - O_TM - I2_TM                          '' 

        fC_INa = 1 / (1 + Exp(-(v + 48) / 7))                               '

        dO_TMdt = I2_TM * kI2O_TM + fC_INa * C_TM * kC2O_INa - O_TM * (kOC_INa + kOI2_TM)                                                             '
        dI2_TMdt = fC_INa * C_TM * kC2I2_TM + O_TM * kOI2_TM + Is_TM * kIsb_TM - I2_TM * (kI2C_TM + kI2O_TM + kIsf_TM)    '
        dIs_TMdt = I2_TM * kIsf_TM + C_TM * kIsf_TM - Is_TM * 2 * kIsb_TM                                                                                                     '

        '****************  determine the gating of LSM *********** using common C1-C2-O gating with the transient mode ****************************
        kI1I2 = 0.00534 * sclI1I2 * fixzero_INaLslowgate                                   '

        kOI1_LSM = kOI2_TM                                                                                '
        kI1O_LSM = 0.01                                                                                         '
        kI1C_LSM = kI2C_TM                                                                               '
        kC2I1_LSM = kC2I2_TM                                                                          '

        kC2I2_LSM = kC2I2_TM * fixzero_INaLslowgate                 '
        kI2C_LSM = kI2C_TM * fixzero_INaLslowgate                      '

        kIsb_LSM = 1.0 / (300000 * Exp(v / 10) + 50000.0 * Exp(v / 16.0)) * fixzero_INaLslowgate   ' set #1    
        kIsf_LSM = 1.0 / (0.016 * Exp(-v / 9.9) + 8 * Exp(-v / 45)) * fixzero_INaLslowgate

        '****************  Calculate state transitions of LSM ***************************************
        C_LSM = 1.0 - Is_LSM - O_LSM - I1_LSM - I2_LSM

        dO_LSMdt = I1_LSM * kI1O_LSM + C_LSM * fC_INa * kC2O_INa - O_LSM * (kOC_INa + kOI1_LSM)                                        '
        dI1_LSMdt = O_LSM * kOI1_LSM + C_LSM * fC_INa * kC2I1_LSM - I1_LSM * (kI1O_LSM + kI1C_LSM + kI1I2)                   '
        dI2_LSMdt = C_LSM * fC_INa * kC2I2_LSM + I1_LSM * kI1I2 + Is_LSM * kIsb_LSM - I2_LSM * (kI2C_LSM + kIsf_LSM)    '
        dIs_LSMdt = I2_LSM * kIsf_LSM + C_LSM * kIsf_LSM - Is_LSM * 2 * kIsb_LSM                                                                              '

    End Sub

    Public PNa As Double = 6.756 * 1.2                    '
    Public relativePK_INa As Double = 0.1

    Private Sub INasub(ByVal v As Double, ByVal GHKNa As Double, ByVal GHKK As Double, ByRef INa_TM_Na_cyt As Double, ByRef INa_TM_K_cyt As Double, ByRef INa_LSM_Na_cyt As Double, ByRef INa_LSM_K_cyt As Double)
        ' ************************ Current amplitude of INa_total based on the previous gating condition *************************************
        INa_TM_Na_cyt = PNa * (1 - LSMratio) * GHKNa * O_TM                                          'current component carried by Na+
        INa_TM_K_cyt = PNa * (1 - LSMratio) * relativePK_INa * GHKK * O_TM              'current component carried by K+
        INa_LSM_Na_cyt = PNa * LSMratio * GHKNa * O_LSM                                             'current component carried by Na+
        INa_LSM_K_cyt = PNa * LSMratio * relativePK_INa * GHKK * O_LSM                 'current component carried by K+

    End Sub

    '****  8 States of the CaRU (voltage-gate LCC, Ca-gate LCC, RyR gate) ************
    Public Yooo As Double
    Public Yooc As Double
    'Public Yooi As Double

    Public Yoco As Double
    Public Yocc As Double
    'Public Yoci As Double

    Public Ycoo As Double
    Public Ycoc As Double
    'Public Ycoi As Double

    Public Ycco As Double
    Public Yccc As Double
    'Public Ycci As Double

    '****************** dy/dt for 8 states
    Public dYooodt As Double
    Public dYoocdt As Double
    'Public dYooidt As Double

    Public dYocodt As Double
    Public dYoccdt As Double
    'Public dYocidt As Double

    Public dYcoodt As Double
    Public dYcocdt As Double
    'Public dYcoidt As Double

    Public dYccodt As Double
    Public dYcccdt As Double
    'Public dYccidt As Double

    '***************  Transiton rate
    Public kYoooYooc As Double      'RyR gate 
    Public kYoocYooo As Double

    Public kYocoYocc As Double
    Public kYoccYoco As Double

    Public kYcooYcoc As Double
    Public kYcocYcoo As Double

    Public kYccoYccc As Double
    Public kYcccYcco As Double

    Public kYoooYcoo As Double      'ICaL gate (Voltage)
    Public kYcooYooo As Double
    Public kYoocYcoc As Double
    Public kYcocYooc As Double

    Public kYocoYcco As Double
    Public kYccoYoco As Double
    Public kYoccYccc As Double
    Public kYcccYocc As Double

    Public kYoooYoco As Double      'ICaL gate (Ca)
    Public kYocoYooo As Double
    Public kYoocYocc As Double
    Public kYoccYooc As Double

    Public kYcooYcco As Double
    Public kYccoYcoo As Double
    Public kYcocYccc As Double
    Public kYcccYcoc As Double

    Public Ca_ndLR As Double                  'mM, [Ca]nd when both R and L are open
    Public Ca_nd0R As Double                   'mM, [Ca]nd when only R is open
    Public Ca_ndL0 As Double                    'mM, [Ca]nd when only L is open
    Public Ca_nd00 As Double                    'mM, [Ca]nd when both R and L are closed
    Public minCa_ndLR As Double            'peak search for  minimum [Ca]nd when both R and L are open

    Public kRoc As Double        'rate for RyR 
    Public kRco As Double
    Dim kvco As Double          'rate for activation for LCC
    Dim kvoc As Double          'rate for deactivation for LCC

    Public JLCC As Double = 0.000913             'um3ms-1 conductance of a unit  LCC
    Public JRyR As Double = 0.02                       ' um3ms-1 conductance of a unit RyR  in a single CaRU

    Public gD_nd As Double = 0.065
    Public gD_free As Double = 0.065               ' um3ms-1 Diffusion const between channel mouth and  iz-space or blk_space
    Public po_LCC As Double                               'open probability of LCC
    Public po_RyR As Double                               'open probability of RYR
    Public Cabs_0R As Double
    Public Cabs_LR As Double

    Public PRyR As Double = 5191 * Sc_Cell       '
    Public po_baseRyR As Double = 0.000075            'a lower limit of the open probability of RyR 
    Public ratioNa As Double = 0.0000185                  'permeability ration of pNa/pCa via ICaL channel
    Public ratioK As Double = 0.000367                       'permeability ration of pK/pCa via ICaL channel

    '


    Public Sub CaRU(ByVal v As Double, ByVal Fraction As Double, ByVal ghkNa As Double, ByVal ghkK As Double, ByRef ICaLCa_LR As Double, ByRef ICaLCa_L0 As Double, ByRef ICaLNa As Double, ByRef ICaLK As Double, ByRef Jrel_SR As Double)

        po_LCC = Yooo + Yooc                                    'open probability of LCC
        po_RyR = Yooo + Ycoo + Ycco + Yoco             'open probability of RyR

        ''''****  prepare Ca_nd for the 4 different combinations of RyR and LCC   *******************
        CaNanoDomain(v)                       'determine [Ca]nd at various combinations with and without the blink space

        ' ******  Case of LCC is open ********
        RyRkineticsStern2013(Ca_ndL0, Ca_ndLR, kRoc, kRco)     'LCC is open
        kYoooYooc = kRoc                'trigger step closing rate
        kYoocYooo = kRco                'trigger step opening rate

        ' ******  Case of LCC is closed ********
        RyRkineticsStern2013(Ca_nd00, Ca_nd0R, kRoc, kRco)       'LCC is closed
        kYcooYcoc = kRoc               'trigger step closing rate     Ycoo
        kYcocYcoo = kRco               'trigger step opening rate

        kYccoYccc = kRoc               'trigger step closing rate   Ycco
        kYcccYcco = kRco               'trigger step opening rate

        kYocoYocc = kRoc               'trigger step closing rate   Yoco
        kYoccYoco = kRco

        '*********** the Vm gate of LCC ***********
        LCCVm(v, kvco, kvoc)

        kYcooYooo = kvco                'LCC activation Ycoo
        kYoooYcoo = kvoc
        kYcocYooc = kvco                 'LCC activation from Ycoc
        kYoocYcoc = kvoc

        kYccoYoco = kvco               'LCC activation from Ycco
        kYocoYcco = kvoc
        kYcccYocc = kvco               'LCC activation from Yccc
        kYoccYccc = kvoc

        '************   Ca gate of LCC ********************************

        Dim kcaco As Double
        Dim kcaoc As Double

        LCCCa(v, Ca_ndLR, kcaco, kcaoc)                            '[Ca]nd when both RyR and LCC are open 
        kYoooYoco = kcaoc           'inactivation step of Ca-gate of Yooo
        kYocoYooo = kcaco          'removal of inactivation 

        LCCCa(v, Ca_ndL0, kcaco, kcaoc)                            '[Ca]nd when  LCC is open
        kYoocYocc = kcaoc            'inactivation step of Ca-gate of Yooc
        kYoccYooc = kcaco            'removal of inactivation

        LCCCa(v, Ca_nd0R, kcaco, kcaoc)                           '[Ca]nd when  RyR is open
        kYcooYcco = kcaoc             'inactivation step of Ca-gate of Ycoo
        kYccoYcoo = kcaco             'removal of inactivation

        LCCCa(v, Ca_nd00, kcaco, kcaoc)                           '[Ca]nd when both RyR and LCC are closed
        kYcocYccc = kcaoc            'inactivation step of Ca-gate of Ycoc
        kYcccYcoc = kcaco            'removal of inactivation


        RyRdydt()               '********** prepare dy/dt for each transition step ***

        ''''************  whole cell  ICaL_jnc via RyR ***************************************************
        perNa_ICaL = ratioNa * PCaL                                                                   '
        perK_ICaL = ratioK * PCaL                                                                       '
        ICaLCa_LR = Fraction * PCaL * GHKCa_LR * Yooo * ATPfactor * scaleICaL             '  
        ICaLCa_L0 = Fraction * PCaL * GHKCa_L0 * Yooc * ATPfactor * scaleICaL             '  

        ICaLNa = Fraction * perNa_ICaL * ghkNa * po_LCC * ATPfactor * scaleICaL             '
        ICaLK = Fraction * perK_ICaL * ghkK * po_LCC * ATPfactor * scaleICaL                      '

        '***************  Flux through RyR , Jrel_SR  with or without the assumption of blink space  *********************
        Dim pot_RyR As Double = po_RyR + po_baseRyR                                  '

        If BlinkS = False Then                   '**************** No blink space *****************
            Jrel_SR = PRyR * pot_RyR * (Cafree_SRrl - Cafree_jnc) * freezeSR     '  

        Else                                                 '**************** With blink space *****************
            Dim fbs As Double = JRyR / gD_bs

            If v > -0.00001 And v <= 0 Then
                v = -0.00001
            ElseIf v < 0.00001 And v > 0 Then
                v = 0.00001
            End If
            Dim expF As Double = Math.Exp(-v / RTF2)


            fR = JRyR / gD_nd
            fL = JLCC / gD_nd
            Dim po_RyR_LR = Yooo                                                    'pO of couplon (RyRs)
            Dim po_RyR_0R = Ycoo + Ycco + Yoco                             'pO of couplon (RyRs)

            '******Note !          Ca_SRrel is calculated for the blink space ***********
            Cabs_0R = ((Ca_nd00 / (1 + fR)) + (Cafree_SRrl / fbs)) / (1 + (1 / fbs) - (fR / (1 + fR)))
            Cabs_LR = ((Ca_nd00 + fL * Cao * expF * (v / RTF2) / (1 - expF)) / (1 + fR + fL * (v / RTF2) / (1 - expF)) + Cafree_SRrl / fbs) / (1 + 1 / fbs - fR / (1 + fR + fL * (v / RTF2) / (1 - expF)))

            Dim Jrel_SR_LR = PRyR * po_RyR_LR * (Cabs_LR - Ca_ndLR) * freezeSR
            Dim Jrel_SR_0R = PRyR * po_RyR_0R * (Cabs_0R - Ca_nd0R) * freezeSR
            Dim Jrel_SR_base = PRyR * po_baseRyR * (Cafree_SRrl - Cafree_jnc) * freezeSR
            Jrel_SR = Jrel_SR_LR + Jrel_SR_0R + Jrel_SR_base
        End If

    End Sub

    Public Sub CaNanoDomain(ByVal v As Double)       'Calculate [Ca]nd with or without the blink space
        If v > -0.00001 And v <= 0 Then
            v = -0.00001
        ElseIf v < 0.00001 And v > 0 Then
            v = 0.00001
        End If

        Dim expF As Double = Math.Exp(-v / RTF2)

        If scaleICaL = 0 Then
            JLCC = 0
        Else
            JLCC = 0.000913
        End If

        If BlinkS = False Then      '**************** No blink space *****************
            Ca_ndLR = (Cafree_jnc + Cafree_SRrl * JRyR / gD_nd + expF * Cao * JLCC / gD_nd * (v / RTF2) / (1 - expF)) / (1 + JRyR / gD_nd + JLCC / gD_nd * v / RTF2 / (1 - expF))
            Ca_nd0R = (Cafree_jnc + Cafree_SRrl * JRyR / gD_nd) / (1 + JRyR / gD_nd)
            Ca_ndL0 = (Cafree_jnc + expF * Cao * JLCC / gD_nd * (v / RTF2) / (1 - expF)) / (1 + JLCC / gD_nd * v / RTF2 / (1 - expF))               ' Ca_nd with L open 
            Ca_nd00 = Cafree_jnc                                                                                   'Ca_nd without R and L open
        Else                                    '***************** Blink space ********************
            gD_bs = gD_nd * ratioBS                    'a new gD_bs is calculated
            fR = JRyR / gD_nd
            fL = JLCC / gD_nd
            fbs = JRyR / gD_bs

            Ca_ndLR = (Cafree_jnc + Cabs_LR * fR + expF * Cao * fL * (v / RTF2) / (1 - expF)) / (1 + fR + fL * v / RTF2 / (1 - expF))
            Ca_nd0R = (Cafree_jnc + Cabs_0R * fR) / (1 + fR)
            Ca_ndL0 = (Cafree_jnc + expF * Cao * fL * (v / RTF2) / (1 - expF)) / (1 + fL * v / RTF2 / (1 - expF))               ' Ca_nd with L open 
            Ca_nd00 = Cafree_jnc                                                                                   'Ca_nd without R and L open

        End If

        If fixCaconc = True Then      'no Ca release from SR
            Ca_ndLR = 0.00001
            Ca_nd0R = 0.00001
            Ca_nd00 = 0.00001     'Ca_nd00
        End If

    End Sub

    Dim gD_bs As Double                      'diffusion conductivity of Ca between the blink space and SRrl
    Dim fbs As Double
    Dim fR As Double
    Dim fL As Double
    Public ratioBS As Double = 1


    '****************** Stern et al. 2013 *********************
    Public ft As Double                   'fraction of open RyR in the triggering step
    Dim fn As Double = 7               '7, representing the nearest neighbor influence of Ca released from one RyR during the regenerative phase
    Public Km_RyR As Double

    Dim RyRkco As Double                         'kco of a single RyR
    Dim RyRkoc As Double                          'koc of a single RyR
    Public fixedkco As Double
    Dim Qten As Double = 3                         ' adjust for temperature assuming that recordings of the single RyR channel have been conducted at the room temp. 
    Dim sloc0 As Double = 0.1                      'original = 0.1 mM      biasing factor defined in Stern et al 2013
    Dim hry As Double = 2.7                         '2.65 (3 introduced on March 27) original Stern et al 2013
    Dim kom_RyR As Double = 0.5664         '3.0    closing rate     modified from Stern et al 2013
    Dim kon_RyR As Double = 0.4              '0.4  ms-1 mM-1
    Dim khalf_RyR As Double = 0.025        '0.025mM
    Dim pC As Double                                'steady-state probability of closed state used to convert RyR_koc to couplon_kroc 
    Dim Nc As Double = 10                        'number of RyRs in a couplon used to convert RyR_koc to couplon_kroc
    Dim Akrco As Double                           'temporaly couplon activation rate, 
    Dim Ckcop As Double
    Dim ftpC As Double

    Public Sub RyRkineticsStern2013(ByVal caT As Double, ByVal caA As Double, ByRef kroc As Double, ByRef krco As Double)
        '**** triggering  step with caT ********************
        RyRkco = Qten * kon_RyR / (1 + Math.Pow(khalf_RyR / caT, hry))         '/ms, opening rate of a single RyR channel 
        RyRkoc = Qten * kom_RyR                                                                        '/ms, closing rate of a single RyR channel                                                                             ' 
        ft = RyRkco / (RyRkco + RyRkoc)                                                       'assuming an instantaneous equilibrium 

        '**** RyR-RyR cooperative step with caA ***, positive feedback step ********************
        Akrco = Qten * kon_RyR * (sloc0 + Cafree_SRrl) / (1 + Math.Pow(khalf_RyR / caA, hry))         ' opening rate of couplon  dependent on [Ca]SRrl  with caT
        krco = fn * ft * Akrco                              '7,  overall opening rate of a couplon with caA

        Ckcop = Qten * kon_RyR / (1 + Math.Pow(khalf_RyR / Ca_nd00, hry))         '/ms, opening rate of a single RyR channel 
        ftpC = Ckcop / (Ckcop + RyRkoc)
        pC = RyRkoc / (ftpC * Akrco + RyRkoc)                       'steady-state probability of closed state
        kroc = RyRkoc * pC ^ ((Nc - 1) * 0.74)                         'closing rate of couplon

        fixedkco = fn * ft * Qten * kon_RyR * (0.7337) / (1 + Math.Pow(khalf_RyR / caA, hry))         'an optional situation of assuming kco remain constant.  opening rate of couplon  dependent on [Ca]SRrl  with caT

    End Sub

    Private Sub RyRdydt()                   'State transition of CaRU
        Yccc = 1 - (Yooo + Yooc + Ycoo + Ycoc + Ycco + Yoco + Yocc)

        dYooodt = kYoocYooo * Yooc + kYcooYooo * Ycoo + kYocoYooo * Yoco - (kYoooYooc + kYoooYcoo + kYoooYoco) * Yooo
        dYoocdt = kYcocYooc * Ycoc + kYoooYooc * Yooo + kYoccYooc * Yocc - (kYoocYcoc + kYoocYooo + kYoocYocc) * Yooc

        dYcoodt = kYcocYcoo * Ycoc + kYoooYcoo * Yooo + kYccoYcoo * Ycco - (kYcooYcoc + kYcooYooo + kYcooYcco) * Ycoo
        dYcocdt = kYcooYcoc * Ycoo + kYoocYcoc * Yooc + kYcccYcoc * Yccc - (kYcocYcoo + kYcocYooc + kYcocYccc) * Ycoc

        dYccodt = kYcccYcco * Yccc + kYocoYcco * Yoco + kYcooYcco * Ycoo - (kYccoYccc + kYccoYoco + kYccoYcoo) * Ycco
        'dYcccdt = kYcciYccc * Ycci + kYccoYccc * Ycco + kYoccYccc * Yocc + kYcocYccc * Ycoc - (kYcccYcci + kYcccYcco + kYcccYocc + kYcccYcoc) * Yccc

        dYocodt = kYoccYoco * Yocc + kYccoYoco * Ycco + kYoooYoco * Yooo - (kYocoYocc + kYocoYcco + kYocoYooo) * Yoco
        dYoccdt = kYcccYocc * Yccc + kYocoYocc * Yoco + kYoocYocc * Yooc - (kYoccYccc + kYoccYoco + kYoccYooc) * Yocc

    End Sub

    Public KL As Double = 4.4 / 1000       'mM
    Public fVmAct As Double
    Dim mEta1 As Double              'component of kco
    Dim mEta2 As Double
    Dim TL As Double = 147.51

    Public Sub LCCCa(ByVal v As Double, ByVal ca As Double, ByRef kco As Double, ByRef koc As Double)
        '*****************  Ca-mediated inactivation of LCC **********************
        koc = (ca / KL) * fVmAct / TL                                           'Ca-mediated inactivation rate  Eta+
        mEta1 = 1 / (8084 * Math.Exp(v / 10) + 158 * Math.Exp(v / 1000))        '
        mEta2 = 1 / (134736 * Math.Exp(v / -5) + 337 * Math.Exp(v / -2000))   '
        kco = mEta1 + mEta2
    End Sub

    Public Ycc_iz As Double     'state of LCC  gateVm closed, gateCa closed
    Public Yco_iz As Double     'state of LCC  gateVm closed, gateCa open
    Public Yoc_iz As Double     'state of LCC  gateVm open, gateCa closed
    Public Yoo_iz As Double     'state of LCC  gateVm open, gateCa open

    Public dYco_izdt As Double     'state of LCC  gateVm closed, gateCa open
    Public dYoc_izdt As Double     'state of LCC  gateVm open, gateCa closed
    Public dYoo_izdt As Double     'state of LCC  gateVm open, gateCa open

    Public Ycc_blk As Double     'state of LCC  gateVm closed, gateCa closed
    Public Yco_blk As Double     'state of LCC  gateVm closed, gateCa open
    Public Yoc_blk As Double     'state of LCC  gateVm open, gateCa closed
    Public Yoo_blk As Double     'state of LCC  gateVm open, gateCa open

    Public dYco_blkdt As Double     'state of LCC  gateVm closed, gateCa open
    Public dYoc_blkdt As Double     'state of LCC  gateVm open, gateCa closed
    Public dYoo_blkdt As Double     'state of LCC  gateVm open, gateCa open

    Dim kccco As Double              'removal of Ca inactivation 
    Dim kcocc As Double              'onset of Ca inactivation
    Dim kocoo As Double             'removal of Ca inactivation
    Dim koooc As Double              'onset of Ca inactivation

    Dim kcooo As Double              'onset of Vm activation
    Dim kooco As Double              'removal of Vm activation
    Dim kccoc As Double               'onset of Vm activation
    Dim koccc As Double               'removal of Vm activation

    Dim Ca_loc As Double             'local [Ca] at the Ca binding site (calmodulin tethered to LCC)   
    Dim product As Double

    Public Sub ICaL4stateGating(ByVal v As Double, ByVal ca_free As Double, ByRef yco As Double, ByRef yoo As Double, ByRef yoc As Double, _
                         ByRef dycodt As Double, ByRef dyoodt As Double, ByRef dyocdt As Double)
        '******************************* calcium gate *******************************************
        fVmAct = 1.0 / (3.734 * Math.Exp(-v / 8.5) + 0.35 * Math.Exp(-v / 3500))    '

        Dim expF As Double = Math.Exp(-v / RTF2)
        LCCVm(v, kcooo, kooco)      'prepare the voltage-dependent transition
        LCCVm(v, kccoc, koccc)      'prepare the voltage-dependent transition

        If v > -0.00001 And v < 0.00001 Then
            v = 0.000011 'to avoid 0 for denominator
            Ca_loc = (ca_free + JLCC / gD_free * Cao * (v / RTF2 * expF) / (1 - expF)) / (1 + JLCC / gD_free * v / RTF2 / (1 - expF))
        Else
            Ca_loc = (ca_free + JLCC / gD_free * Cao * (v / RTF2 * expF) / (1 - expF)) / (1 + JLCC / gD_free * v / RTF2 / (1 - expF))
        End If

        If fixCaconc = True Then               'in case of using any artificial Ca buffer.
            'ca_free = 0.00001
            'Ca_loc = 0.00001
        End If

        LCCCa(v, ca_free, kccco, kcocc)
        LCCCa(v, Ca_loc, kocoo, koooc)
        '***********************************************************
        Dim Ycc As Double = 1 - (yco + yoo + yoc)
        dycodt = kccco * Ycc + kooco * yoo - (kcocc + kcooo) * yco
        dyoodt = kcooo * yco + kocoo * yoc - (kooco + koooc) * yoo
        dyocdt = koooc * yoo + kccoc * Ycc - (kocoo + koccc) * yoc
    End Sub

    Public Sub LCCVm(ByVal v As Double, ByRef kco As Double, ByRef koc As Double)
        kco = fVmAct                                                                                          '
        koc = 1.0 / (4.65 * Exp(v / 15) + 1.363 * Exp(v / 100))                       '
    End Sub

    '********** ICaL **********
    Public oPCaL As Double = 14.21 * 1                                 'original permeability of ICaL 
    Public PCaL As Double = oPCaL                                      ' default value

    Public scaleICaL As Double = 1                'default value =1
    Dim perNa_ICaL As Double
    Dim perK_ICaL As Double                      'the reversal potential of ICaL is shifted negative by this modification
    Dim ATPCaL As Double = 6                    'a concentration of ATP for the CaL gating
    Dim ATPfactor As Double                        'run-down due to ATP depletion

    Public ICaL As Double                              'L-type Ca current composed of Ca, Na, and K components
    Public ICaL_jnc As Double                       'L-type Ca current composed of Ca, Na, and K components
    Public ICaL_iz As Double                          'L-type Ca current composed of Ca, Na, and K components
    Public ICaL_blk As Double                       'L-type Ca current composed of Ca, Na, and K components

    'Public ICaLCa_jnc As Double
    Public ICaLCa_LR As Double              'ICaL due to Yooo in the CaRU
    Public ICaLCa_L0 As Double               'ICaL due to Yooc in the CaRU
    Public ICaLCa_iz As Double
    Public ICaLCa_blk As Double

    Public ICaLNa_jnc As Double
    Public ICaLNa_iz As Double
    Public ICaLNa_blk As Double

    Public ICaLK_jnc As Double
    Public ICaLK_iz As Double
    Public ICaLK_blk As Double

    Public Sub ICaL4stateAmplitude(ByVal fraction As Double, ByVal pOpen As Double, ByVal ghkCa As Double, ByVal ghkNa As Double, ByVal ghkK As Double, ByRef ICaLCa As Double, ByRef ICaLNa As Double, ByRef ICaLK As Double)
        ATPfactor = 1.0 / (1.0 + ((1.4 / ATPCaL) ^ 3))     '
        product = pOpen * ATPfactor * scaleICaL                              '
        perNa_ICaL = ratioNa * PCaL
        perK_ICaL = ratioK * PCaL
        ICaLCa = fraction * PCaL * ghkCa * product                        'pA / pF, amplitude of Ca component  
        ICaLNa = fraction * perNa_ICaL * ghkNa * product            'pA / pF, amplitude of  Na component
        ICaLK = fraction * perK_ICaL * ghkK * product                   'pA / pF, amplitude of K component
    End Sub

    '**************  INaK  ***********************************************************
    Public maxINaK As Double = 25.1779 * 1.24            ' 

    Public fNaK As Double = 1                       'a scaling factor of Na/K pump

    Public INaK As Double
    Public INaK_Na_cyt As Double
    Public INaK_K_cyt As Double

    Public P1_6_NaK As Double
    Public P7_NaK As Double
    Public P8_13_NaK As Double

    Public dP1_6_NaKdt As Double
    Public dP7_NaKdt As Double
    Public dP8_13_NaKdt As Double

    Public dATP_NaK As Double

    Dim k1p_NaK As Double = 0.72        '
    Dim k1m_NaK As Double = 0.08      '
    Dim k2p_NaK As Double = 0.08       '
    Dim k2m_NaK As Double = 0.008    '
    Dim k3p_NaK As Double = 4             '
    Dim k3m_NaK As Double = 8000     '
    Dim k4p_NaK As Double = 0.3         '
    Dim k4m_NaK As Double = 0.2        '

    Dim KdKe0_NaK As Double = 0.8         '
    Dim KdKi0_NaK As Double = 18.8        '
    Dim KdNae0_NaK As Double = 26.8        '
    Dim KdNai0_NaK As Double = 5              '
    Dim KdMgATP_NaK As Double = 0.6     '

    Dim stoiNa_NaK As Double = 3                  '
    Dim stoiK_NaK As Double = -2                   '
    Dim dV_Nae_NaK As Double = 0.44             '
    Dim dV_Nai_NaK As Double = -0.14            '
    Dim dV_Ke_NaK As Double = 0.23            '
    Dim dV_Ki_NaK As Double = -0.14            '

    Public Sub INaKsub(ByVal frc As Double, ByVal v As Double, ByVal Nai As Double, ByVal Ki As Double, ByVal MgATP_cyt As Double, ByVal MgADP_cyt As Double, ByRef INaK As Double, ByRef INaK_Na As Double, ByRef INaK_K As Double, ByRef dNATP_NaK As Double, _
                      ByVal P1_6 As Double, ByVal P7 As Double, ByVal P8_13 As Double, ByRef dP1_6dt As Double, ByRef dP7dt As Double, ByRef dP8_13dt As Double)
        '**********************     Time-dependent model of INaK       *********************************
        '************  rate constants for the 4 state model of Na/K pump *********************

        Dim fVm_NaK As Double = Faraday * v / (R * tempK)                     'VF/R/T
        Dim KdNae_NaK As Double = KdNae0_NaK * Exp(dV_Nae_NaK * fVm_NaK)     'relative Kd 
        Dim KdNai_NaK As Double = KdNai0_NaK * Exp(dV_Nai_NaK * fVm_NaK)         'relative Kd 
        Dim KdKe_NaK As Double = KdKe0_NaK * Exp(dV_Ke_NaK * fVm_NaK)                'relative Kd 
        Dim KdKi_NaK As Double = KdKi0_NaK * Exp(dV_Ki_NaK * fVm_NaK)                    'relative Kd 
        Dim Nai_ As Double = Nai / KdNai_NaK                 'relative concentration referring to the dissociation constant 
        Dim Nae_ As Double = Nao / KdNae_NaK                   'relative concentration referring to the dissociation constant
        Dim Ki_ As Double = Ki / KdKi_NaK                        'relative concentration referring to the dissociation constant
        Dim Ke_ As Double = Ko / KdKe_NaK                         'relative concentration referring to the dissociation constant
        'Dim MgATP_ As Double = ATPMg / kd_MgATP  'relative concentration of ATP referring to the dissociation constant
        Dim Kicongen As Double = 0                                 'used only when inclucing Cs+ to replace K+ 
        Dim Cseffect As Double = Kicongen * (2 * Ki_ + 3 * Nai_ + 1)        'used only when inclucing Cs+ to replace K+ 
        ' transition rates in the forward direction
        Dim a1p_NaK As Double = (k1p_NaK * (Nai_ ^ 3.0)) / (((1 + Nai_) ^ 3) + ((1 + Ki_) ^ 2) - 1)
        Dim a2p_NaK As Double = k2p_NaK
        Dim a3p_NaK As Double = k3p_NaK * (Ke_ ^ 2) / (((1 + Nae_) ^ 3) + ((1 + Ke_) ^ 2) - 1)
        Dim a4p_NaK As Double = k4p_NaK * MgATP_cyt / KdMgATP_NaK / (1 + MgATP_cyt / KdMgATP_NaK)

        ' transition rates in the reverse direction
        Dim a1m_NaK As Double = k1m_NaK * MgADP_cyt
        Dim a2m_NaK As Double = k2m_NaK * (Nae_ ^ 3) / (((1 + Nae_) ^ 3) + ((1 + Ke_) ^ 2) - 1)
        Dim a3m_NaK As Double = k3m_NaK * Pifree_cyt * H_cyt / (1 + MgATP_cyt / KdMgATP_NaK)
        Dim a4m_NaK As Double = k4m_NaK * (Ki_ ^ 2) / (((1 + Nai_) ^ 3) + ((1 + Ki_) ^ 2) - 1)

        Dim P14_15 As Double = (1 - P1_6 - P7 - P8_13)

        Dim vstep0_NaK As Double = (a1p_NaK * P1_6 - a1m_NaK * P7)     'flux of each step within the 4 states model
        Dim vstep1_NaK As Double = (a2p_NaK * P7 - a2m_NaK * P8_13)
        Dim vstep2_NaK As Double = (a3p_NaK * P8_13 - a3m_NaK * P14_15)
        Dim vstep3_NaK As Double = (a4p_NaK * P14_15 - a4m_NaK * P1_6)

        Dim vcyc_NaK As Double = vstep1_NaK                                                    'any step when calculating the steady state distribution

        INaK = frc * maxINaK * vcyc_NaK * fNaK        ' current  density (pA/pF)
        INaK_Na = stoiNa_NaK * INaK        ' calculate ion flux
        INaK_K = stoiK_NaK * INaK

        dNATP_NaK = INaK * Cm / Faraday           'amole / ms

        dP1_6dt = (-a1p_NaK * P1_6 + a1m_NaK * P7 + a4p_NaK * P14_15 - a4m_NaK * P1_6)
        dP7dt = (-a2p_NaK * P7 + a2m_NaK * P8_13 + a1p_NaK * P1_6 - a1m_NaK * P7)
        dP8_13dt = (-a3p_NaK * P8_13 + a3m_NaK * P14_15 + a2p_NaK * P7 - a2m_NaK * P8_13)
    End Sub

    '''''***************************************** IKr
    Public IKr As Double
    Public IKr_K_cyt As Double

    Public y1_IKr As Double
    Public y2_IKr As Double
    Public y3_IKr As Double
    Public dy1_IKrdt As Double
    Public dy2_IKrdt As Double
    Public dy3_IKrdt As Double
    Public pO_IKr As Double

    Public scaleIKr As Double = 1                         'scaling factor for IKr amplitude, defalt = 1

    Public a1_IKr As Double
    Public b1_IKr As Double
    Public a2_IKr As Double
    Public b2_IKr As Double
    Public a3_IKr As Double
    Public b3_IKr As Double

    Public oGIKr As Double = 0.02 * 0.83 * 1.03                            'nS, 0.01 * 0.83 * 2 original GKr
    Public GIKr As Double = oGIKr                                               'default value                                                             '


    Public Sub IKrsub(ByVal v As Double)
        'the fast component of activation
        a1_IKr = 1.0 / (35.0 * Exp(-v / 10.5) + 75.0 * Exp(-v / 100.0))
        b1_IKr = 1.0 / (470.0 * Exp(v / 8.3) + 220.0 * Exp(v / 29.0))
        'the slow component of activation
        a2_IKr = 1.0 / (350.0 * Exp(-v / 10.5) + 300.0 * Exp(-v / 100.0))            '* fixzero
        b2_IKr = 1.0 / (1850.0 * Exp(v / 8.3) + 2200.0 * Exp(v / 29.0))               '* fixzero

        ' inactivation gate (fast gate)
        a3_IKr = 1.0 / (0.015 * Exp(v / 6.0) + 7.0 * Exp(v / 60.0))                      '* fixzero
        b3_IKr = 1.0 / (0.114 * Exp(-v / 9.2) + 2.3 * Exp(-v / 1000.0))                '* fixzero

        pO_IKr = (0.6 * y1_IKr + 0.4 * y2_IKr) * y3_IKr
        Carry(11) = y1_IKr
        Carry(12) = y2_IKr
        Carry(13) = y3_IKr
        IKr_K_cyt = GIKr * (v - EK) * pO_IKr * (Ko / 4.5) ^ 0.2 * scaleIKr

        dy1_IKrdt = a1_IKr * (1.0 - y1_IKr) - b1_IKr * y1_IKr
        dy2_IKrdt = (a2_IKr * (1.0 - y2_IKr) - b2_IKr * y2_IKr)
        dy3_IKrdt = a3_IKr * (1.0 - y3_IKr) - b3_IKr * y3_IKr
    End Sub

    '********** IKsKyoto **********
    Dim b2_IKs As Double = 0.000296 '* fixzero
    Dim a3_IKs As Double = 0.0003 '* fixzero
    Dim b3_IKs As Double = 0.03 '* fixzero

    Public IKs As Double
    Public IKs_iz As Double
    Public IKs_blk As Double

    Public IKs_K_iz As Double
    Public IKs_Na_iz As Double
    Public IKs_K_blk As Double
    Public IKs_Na_blk As Double

    Public pO_IKs_iz As Double
    Public pO_IKs_blk As Double

    Public Ov_IKs As Double        'voltage gate with  two states , common for both iz and blk compartments, 
    Public dOv_IKsdt As Double

    Public C2IKs_iz As Double             'iz compartment, Calcium gate with C1 - C2 - Oc states
    Public OcIKs_iz As Double             'iz compartment, Calcium gate with C1 - C2 - Oc states
    Public dOcIKs_izdt As Double        'iz compartment, Calcium gate with C1 - C2 - Oc states
    Public dC2IKs_izdt As Double        'iz compartment, Calcium gate with C1 - C2 - Oc states

    Public C2IKs_blk As Double              'blk compartment, Calcium gate with C1 - C2 - Oc states
    Public OcIKs_blk As Double              'blk compartment, Calcium gate with C1 - C2 - Oc states
    Public dOcIKs_blkdt As Double        'blk compartment, Calcium gate with C1 - C2 - Oc states
    Public dC2IKs_blkdt As Double        'blk compartment, Calcium gate with C1 - C2 - Oc states

    Public PKs As Double = 0.9                                 '0.002782
    Public perNa_IKs As Double = 0.04 * PKs

    Private Sub VgateIKs(ByVal v As Double, ByVal Ov As Double, ByRef dOvdt As Double)
        Dim a1_IKs As Double = 1.0 / (150.0 * Exp(-v / 25.0) + 900.0 * Exp(-v / 200.0))
        Dim b1_IKs As Double = 1.0 / (1000.0 * Exp(v / 13.0) + 220.0 * Exp(v / 50.0))
        dOvdt = a1_IKs * (1.0 - Ov) - b1_IKs * Ov

    End Sub

    Private Sub IKsKyotosub(ByVal ca As Double, ByVal fraction As Double, ByRef IKs_K As Double, ByRef IKs_Na As Double, ByVal Ov As Double, ByVal C2_IKs As Double, ByVal Oc_IKs As Double, ByRef dOc_IKsdt As Double, ByRef dC2_IKsdt As Double, ByRef pO_IKs As Double)
        Dim C1_IKs As Double = 1 - C2_IKs - Oc_IKs
        Dim a2_IKs As Double = ca * 2.25 '* fixzero

        pO_IKs = (Ov ^ 2) * (0.99 * Oc_IKs + 0.01)
        IKs_Na = fraction * perNa_IKs * GHKNa * pO_IKs
        IKs_K = fraction * PKs * GHKK * pO_IKs
        Carry(5) = (0.99 * Oc_IKs + 0.01)
        dOc_IKsdt = a2_IKs * C2_IKs - b2_IKs * Oc_IKs                                                                          'Calcium gate
        dC2_IKsdt = a3_IKs * C1_IKs - b3_IKs * C2_IKs - a2_IKs * C2_IKs + b2_IKs * Oc_IKs     'Calcium gate
    End Sub

    '**************** activation gate kinetics ********************
    'Public xs1inf As Double                'steady-state value
    'Public xs2inf As Double                'steady-state value
    'Public xs1tau As Double               'ms,   tau
    'Public xs2tau As Double               'ms,   tau
    'Public paraxs1_iz As Double        'iz compartment, Calcium gate with C1 - C2 - Oc states
    'Public paraxs2_iz As Double        'iz compartment, Calcium gate with C1 - C2 - Oc states
    'Public dxs2_izdt As Double        'iz compartment, Calcium gate with C1 - C2 - Oc states
    'Public dxs1_izdt As Double        'iz compartment, Calcium gate with C1 - C2 - Oc states

    'Public paraxs1_blk As Double        'blk compartment, Calcium gate with C1 - C2 - Oc states
    'Public paraxs2_blk As Double        'blk compartment, Calcium gate with C1 - C2 - Oc states
    'Public dxs2_blkdt As Double        'blk compartment, Calcium gate with C1 - C2 - Oc states
    'Public dxs1_blkdt As Double        'blk compartment, Calcium gate with C1 - C2 - Oc states
    ''**************** inactivation gate kinetics ********************
    'Public paraRKs As Double                 'V-dependent inactivation parameter

    'Public pOpen As Double                          'open probability of the channel


    'Public Sub IKsORd(ByVal v As Double, ByVal ca As Double, ByVal fraction As Double, ByRef IKs_K As Double, ByRef IKs_Na As Double, ByRef paraxs1 As Double, ByRef paraxs2 As Double, ByRef dxs1dt As Double, ByRef dxs2dt As Double, ByRef pO_IKs As Double)
    '    xs1inf = 1 / (1 + Math.Exp(-(v + 11.6) / 8.932))       '
    '    xs2inf = xs1inf                                                             '

    '    xs1tau = 817.3 + 1 / (0.0002326 * Math.Exp((v + 48.28) / 17.8) + 0.001292 * Math.Exp(-(v + 210) / 230))     '
    '    xs2tau = 1 / (0.01 * Math.Exp((v - 50) / 20) + 0.0193 * Math.Exp(-(v + 66.54) / 31))                                            '
    '    dxs1dt = (xs1inf - paraxs1) / xs1tau                  '
    '    dxs2dt = (xs2inf - paraxs2) / xs2tau                  '

    '    '***********  Vm-dependent inactivation  RKr  *****************
    '    paraRKs = 1 + 0.6 / (1 + Math.Pow(0.000038 / ca, 1.4))             '

    '    pO_IKs = paraxs1 * paraxs2 * paraRKs                                                      '

    '    IKs_Na = fraction * 0.04 * PKs * GHKNa * pO_IKs      '
    '    IKs_K = fraction * PKs * GHKK * pO_IKs                     '
    'End Sub


    '********** IKp **********
    Public PKpl As Double = 0.000043 * 0.4                  'The plateau current is much reduced in the present model.

    'Public Const GKp As Double = 0.002      'mS/uF
    Public IKpl As Double
    Public IKpl_K_cyt As Double
    Public Vrctf_Kpl As Double
    Public PK_IKpl As Double

    Public Sub IKplsub(ByVal v As Double)
        Dim power_IKpl As Double = 0.16                           '
        PK_IKpl = PKpl * ((Ko / 5.4) ^ power_IKpl)           '

        'To avoid zero denominator
        If v > -0.000001 And v < 0.000001 Then
            Vrctf_Kpl = 13
        Else
            Vrctf_Kpl = v / (1 - Exp(-v / 13))                          '
        End If
        IKpl_K_cyt = PK_IKpl * Vrctf_Kpl * GHKK           '

    End Sub


    '''''''******************************************* IKto
    Public IKto As Double

    Public IKto_Na_cyt As Double
    Public IKto_K_cyt As Double
    Public y1_IKto As Double     'fast inactivation parameter  (57)
    Public y2_IKto As Double     'slow inactivation parameter (58)
    Public dy1_IKtodt As Double
    Public dy2_IKtodt As Double

    Public pO_IKto As Double
    Public pNa_IKto As Double
    Public PKtoEndo As Double = 0.01825 * 0.73


    Private Sub IKtoEndosub(ByVal v As Double, ByRef IKto_K As Double, ByRef IKto_Na As Double, ByRef y1_IKto As Double, ByRef y2_IKto As Double, ByRef dy1_IKtodt As Double, ByRef dy2_IKtodt As Double)

        pNa_IKto = PKtoEndo * 0.09

        pO_IKto = y1_IKto * y2_IKto
        IKto_Na = pNa_IKto * GHKNa * pO_IKto
        IKto_K = PKtoEndo * GHKK * pO_IKto


        Dim a1_IKto As Double = 1.0 / (9 * Exp(-v / 20.0))   'activation gate
        Dim b1_IKto As Double = 1.0 / (2.1 * Exp(v / 60.0))   'activation gate

        Dim a2_IKto As Double = 1.0 / (950 * Exp(v / 500)) '* 0.6                                                'inactivation gate
        Dim b2_IKto As Double = 1.0 / (40.0 * Exp(-v / 9.0) + 13.0 * Exp(-v / 1000.0)) '* 0.6    'inactivation gate

        dy1_IKtodt = (a1_IKto * (1.0 - y1_IKto) - b1_IKto * y1_IKto)
        dy2_IKtodt = (a2_IKto * (1.0 - y2_IKto) - b2_IKto * y2_IKto)
    End Sub


    Public PKtoEpi As Double = 0.08553 * 0.73
    Private Sub IKtoEpisub(ByVal v As Double, ByRef IKto_K As Double, ByRef IKto_Na As Double, ByRef y1_IKto As Double, ByRef y2_IKto As Double, ByRef dy1_IKtodt As Double, ByRef dy2_IKtodt As Double)

        Dim pNa_IKto As Double = PKtoEpi * 0.09

        Dim pO_IKto As Double = y1_IKto * y2_IKto
        IKto_Na = pNa_IKto * GHKNa * pO_IKto
        IKto_K = PKtoEpi * GHKK * pO_IKto

        Dim a1_IKto As Double = 1.0 / (9 * Exp(-v / 20.0))   'activation gate
        Dim b1_IKto As Double = 1.0 / (2.1 * Exp(v / 36.5))   'activation gate

        Dim a2_IKto As Double = 1.0 / (365 * Exp(v / 8.65) + 9.4 * Exp(v / 5000))                                                  'inactivation gate
        Dim b2_IKto As Double = 1.0 / (4.0 * Exp(-v / 8.5) + 8 * Exp(-v / 5000.0))    'inactivation gate

        dy1_IKtodt = (a1_IKto * (1.0 - y1_IKto) - b1_IKto * y1_IKto)
        dy2_IKtodt = (a2_IKto * (1.0 - y2_IKto) - b2_IKto * y2_IKto)
    End Sub

    '**************** activation gate kinetics ********************
    'Public paraa As Double             'parameter of the Vm-dependent activation
    'Public ainf As Double                'steady-state value
    'Public atau As Double               'ms,   tau
    'Dim dadt As Double                   'ms-1,  dydt

    ''**************** inactivation gate kinetics ********************
    ''Public parainet As Double         'over all V-dependent inactivation parameter
    'Public parai As Double                'over all V-dependent inactivation parameter

    'Public iinf As Double                   'steady-state value
    'Dim AiF As Double            'fractional amplitude of the fast component
    'Dim AiS As Double      'fractional amplitude of the slow component

    'Public paraiF As Double              'fast component
    'Public iFtau As Double                'tau
    'Public diFdt As Double

    'Public paraiS As Double       'slow component
    'Public iStau As Double
    'Dim diSdt As Double

    'Public a_IKto As Double         'activation parameter    (56)
    'Public da_IKtodt As Double

    'Public Sub IKtoORd(ByVal v As Double, ByRef IKto_K As Double, ByRef paraa As Double, ByRef paraiF As Double, ByRef paraiS As Double, ByRef dadt As Double, ByRef diFdt As Double, ByRef diSdt As Double)
    '    '***********  Vm-dependent activation m *****************
    '    ainf = 1 / (1 + Math.Exp(-(v - 14.34) / 14.82))                                       '
    '    Dim term1 = 1 / (1.2089 * (1 + Math.Exp(-(v - 18.41) / 29.38)))        '
    '    Dim term2 = 3.5 / (1 + Math.Exp((v + 100) / 29.38))                           '

    '    atau = 1.0515 / (term1 + term2)                                                           '
    '    dadt = (ainf - paraa) / atau                                                                    '

    '    '***********  Vm-dependent inactivation  h  *****************
    '    iinf = 1 / (1 + Math.Exp((v + 43.94) / 5.711))                                                                                                                                    '
    '    iFtau = 4.562 + 1 / (0.3933 * Math.Exp(-(v + 100) / 100) + 0.08004 * Math.Exp((v + 50) / 16.59))                                         '
    '    iStau = 23.62 + 1 / (0.001416 * Math.Exp(-(v + 96.52) / 59.05) + 0.000000017808 * Math.Exp((v + 114.1) / 8.079))         '
    '    AiF = 1 / (1 + Math.Exp((v - 213.6) / 151.2))                                                                                                                                    '
    '    AiS = 1 - AiF                                                                                                                                                                                        '
    '    diFdt = (iinf - paraiF) / iFtau                  ' 
    '    diSdt = (iinf - paraiS) / iStau                  '

    '    parai = AiF * paraiF + AiS * paraiS     '
    '    Dim pO_IKto As Double = paraa * parai            '

    '    IKto_K = 0.0312 * pO_IKto * (v - EK)                '

    'End Sub

    'Private Sub IKtoEpisub(ByVal v As Double, ByRef IKto_K As Double, ByRef IKto_Na As Double, ByRef y1_IKto As Double, ByRef y2_IKto As Double, ByRef dy1_IKtodt As Double, ByRef dy2_IKtodt As Double)

    '    Dim pNa_IKto As Double = PKtoEpi * 0.09

    '    Dim pO_IKto As Double = y1_IKto * y2_IKto
    '    IKto_Na = pNa_IKto * GHKNa * pO_IKto
    '    IKto_K = PKtoEpi * GHKK * pO_IKto

    '    Dim a1_IKto As Double = 1.0 / (9 * Exp(-v / 20.0))   'activation gate
    '    Dim b1_IKto As Double = 1.0 / (2.1 * Exp(v / 36.5))   'activation gate

    '    Dim a2_IKto As Double = 1.0 / (365 * Exp(v / 8.65) + 9.4 * Exp(v / 5000))                                                  'inactivation gate
    '    Dim b2_IKto As Double = 1.0 / (4.0 * Exp(-v / 8.5) + 8 * Exp(-v / 5000.0))    'inactivation gate

    '    dy1_IKtodt = (a1_IKto * (1.0 - y1_IKto) - b1_IKto * y1_IKto)
    '    dy2_IKtodt = (a2_IKto * (1.0 - y2_IKto) - b2_IKto * y2_IKto)
    'End Sub


    '********** IK1 **********
    Public IK1 As Double
    Public oGK1 As Double = 1.353     'nS, conductance of IK1
    Public GK1 As Double = oGK1      'nS, default value of conductance of IK1
    Public IK1_K_cyt As Double
    Public Pbspm As Double
    Public ssPbspm As Double        'steady state probability of spm block   a optional variable
    Public dPbspmdt As Double
    Public pO_IK1 As Double
    Public pO_mode1 As Double
    Public pO_mode2 As Double

    Public fK_IK1 As Double
    Public poMg1 As Double
    Public poMg2 As Double
    Public po_Mggate As Double

    Public alphaSPM As Double
    Public betaSPM As Double
    Public alphaMg As Double
    Public betaMg As Double

    Public SSIK1kinetics As Boolean = False        'default
    Public scale_IK1 As Double = 1                            'dimensionless, scaling factor for IK1 amplitude, default value = 1 


    Public Sub IK1sub(ByVal v As Double, ByVal EK As Double, ByVal Pbspm As Double, ByRef dPbspmdt As Double)
        Dim frac_mode1 As Double = 0.9                                     '
        Dim SPM As Double = 0.005 * 1000                                 ' 
        '************** Mg-block *************************
        alphaMg = 12.0 * Exp(-0.025 * (v - EK))                                            ' 
        betaMg = 28 * Mg_cyt * Exp(0.025 * (v - EK))                                 ' 
        Dim fracO As Double = alphaMg / (alphaMg + betaMg)                  ' 
        Dim fracB As Double = betaMg / (alphaMg + betaMg)                    ' 
        poMg2 = 3.0 * fracO * fracB * fracB                                                  ' 
        poMg1 = 3.0 * fracO * fracO * fracB                                                  '
        po_Mggate = fracO * fracO * fracO                                                   '
        '************** spermin-block *************************
        alphaSPM = 0.17 * Exp(-0.07 * ((v - EK) + 8 * Mg_cyt)) / (1.0 + 0.01 * Exp(0.12 * ((v - EK) + 8 * Mg_cyt)))               '
        betaSPM = 0.28 * SPM * Exp(0.15 * ((v - EK) + 8 * Mg_cyt)) / (1.0 + 0.01 * Exp(0.13 * ((v - EK) + 8 * Mg_cyt)))      '
        dPbspmdt = betaSPM * po_Mggate * (1 - Pbspm) - alphaSPM * Pbspm                                                                               '

        ssPbspm = (betaSPM * po_Mggate) / (alphaSPM + betaSPM * po_Mggate)
        '**************** IK1 amplitude **************************************************
        pO_mode1 = frac_mode1 * (1 - Pbspm) * (po_Mggate + 2 / 3 * poMg1 + 1 / 3 * poMg2)                                                          '
        If SSIK1kinetics = True Then pO_mode1 = frac_mode1 * (1 - ssPbspm) * (po_Mggate + 2 / 3 * poMg1 + 1 / 3 * poMg2) '
        pO_mode2 = (1.0 - frac_mode1) / (1.0 + SPM / (40.0 * Exp(-(v - EK) / 9.1)))                                                                               '

        pO_IK1 = (pO_mode1 + pO_mode2)                                        '

        fK_IK1 = (Ko / 4.5) ^ 0.4 / (1.0 + Exp(-(Ko - 2.2) / 0.6))          '

        IK1_K_cyt = oGK1 * scale_IK1 * (v - EK) * fK_IK1 * pO_IK1                  '

    End Sub

    ''''''*************************************** IbNSC
    Const PNa_IbNSC As Double = 0.00035
    Const PK_IbNSC As Double = 0.00014

    Public IbNSC As Double
    Public IbNSC_Na_cyt As Double
    Public IbNSC_K_cyt As Double

    Private Sub IbNSCsub()                      'background cation current
        IbNSC_Na_cyt = PNa_IbNSC * GHKNa                           '
        IbNSC_K_cyt = PK_IbNSC * GHKK                                  '
    End Sub

    ''''''''''''''''**************************************ILCCa
    Const PNa_ILCa As Double = 0.00273

    Dim perK_ILCa As Double = PNa_ILCa                      '* relPKILCCa
    Public ILCa As Double                              'current amplitude at the time of observation
    Public ILCa_iz As Double                              'current amplitude at the time of observation
    Public ILCa_blk As Double                              'current amplitude at the time of observation

    Public ILCa_Na_iz As Double
    Public ILCa_K_iz As Double
    Public ILCa_Na_blk As Double
    Public ILCa_K_blk As Double
    Public pO_LCa_iz As Double
    Public pO_LCa_blk As Double

    Private Sub ILCasub(ByVal Ca As Double, ByVal fraction As Double, ByRef ILCa_Na As Double, ByRef ILCa_K As Double, ByRef pO_LCa As Double)                        'Ca-dependent non-selective cation current

        pO_LCa = 1.0 / (1.0 + ((0.0012 / Ca) ^ 3))                                          '

        ILCa_Na = PNa_ILCa * fraction * GHKNa * pO_LCa
        ILCa_K = perK_ILCa * fraction * GHKK * pO_LCa
    End Sub

    ''''''''''''''***********************************'IKATP
    Public GKATP As Double = 17.674
    Public IKATP As Double
    Public IKATPK_cyt As Double
    Public pO_IKTP As Double
    Public gamma_IKTP As Double

    Private Sub IKATPsub(ByVal v As Double)                                  'IKATP
        gamma_IKTP = 0.0236 * Ko ^ 0.24       'single channel conductance
        pO_IKTP = 0.8 / (1.0 + ((ATPt_cyt / 0.1) ^ 2))
        IKATPK_cyt = GKATP * (v - EK) * pO_IKTP * gamma_IKTP

    End Sub

    ''''''''*****************************    INCX   **********************************************
    Public maxINCX As Double = 61.06 * 0.5 * 0.95        'NCX 
    '************** parameters for the Ca- and Na-dependent inactivation *************************
    Dim a1off_NCX As Double = 0.0015
    Dim a1on_NCX As Double = 0.002
    Dim b1off_NCX As Double = 0.0000005
    Dim b1on_NCX As Double = 0.0012
    Dim a2off_NCX As Double = 0.02
    Dim a2on_NCX As Double = 0.00006
    Dim b2off_NCX As Double = 0.0002
    Dim b2on_NCX As Double = 0.18
    '************** parameters for the Ca- and Na-binding to transport sites *************************
    Dim KmNao_NCX As Double = 87.5                        'mM
    Dim KmNai_NCX As Double = 20.74854
    Dim KmCao_NCX As Double = 1.38
    Dim KmCai_NCX As Double = 0.0184
    Dim Kmact_NCX As Double = 0.004
    Dim nHNa_NCX As Double = 3
    Dim partition_NCX As Double = 0.32

    Dim k3_NCX As Double = 1.0
    Dim k4_NCX As Double = 1.0

    Public INCX As Double

    '********** iz ***********************
    Public INCX_iz As Double
    Public INCXNa_iz As Double
    Public INCXCa_iz As Double

    Public I1NCX_iz As Double
    Public I2NCX_iz As Double
    Public E1NCX_iz As Double

    Public dE1NCX_izdt As Double
    Public dI1NCX_izdt As Double
    Public dI2NCX_izdt As Double

    '************** blk *******************
    Public INCX_blk As Double
    Public INCXNa_blk As Double
    Public INCXCa_blk As Double

    Public I1NCX_blk As Double
    Public I2NCX_blk As Double
    Public E1NCX_blk As Double

    Public dE1NCX_blkdt As Double
    Public dI1NCX_blkdt As Double
    Public dI2NCX_blkdt As Double

    Public fNaCa As Double = 1

    Public Sub INCXsub(ByVal frc As Double, ByVal v As Double, ByVal Nai As Double, ByVal Cai As Double, ByRef INCX As Double, ByRef INCX_Na As Double, ByRef INCX_Ca As Double, _
                         ByVal E1_NCX As Double, ByVal I1_NCX As Double, ByVal I2_NCX As Double, ByRef dE1_NCXdt As Double, ByRef dI1_NCXdt As Double, ByRef dI2_NCXdt As Double)

        Dim k1_NCX As Double = Exp(partition_NCX * v / (R * tempK / Faraday))                            'voltage-dependent rate
        Dim k2_NCX As Double = Exp((partition_NCX - 1.0) * v / (R * tempK / Faraday))                 'voltage-dependent rate

        Dim E2Na As Double = 1.0 / (1.0 + ((KmNao_NCX / Nao) ^ nHNa_NCX) * (1.0 + Cao / KmCao_NCX))    'Na binding probability
        Dim E2Ca As Double = 1.0 / (1.0 + (KmCao_NCX / Cao) * (1.0 + ((Nao / KmNao_NCX) ^ nHNa_NCX)))   'Ca binding probability

        Dim E1Na As Double = 1.0 / (1.0 + ((KmNai_NCX / Nai) ^ nHNa_NCX) * (1 + Cai / KmCai_NCX))   'Na binding probability
        Dim E1Ca As Double = 1.0 / (1.0 + (KmCai_NCX / Cai) * (1.0 + ((Nai / KmNai_NCX) ^ nHNa_NCX)))   'Ca binding probability

        Dim fCaina As Double = Cai / (Cai + Kmact_NCX)          'Ca-dependency of the inactivation step

        ' I1 inactivation gate (Na-dependent)
        Dim alpha1 As Double = (E1Na * (fCaina * a1on_NCX + (1 - fCaina) * a1off_NCX)) * fixzero_slowVariable  'rate from E1 to I1
        Dim beta1 As Double = (fCaina * b1on_NCX + (1 - fCaina) * b1off_NCX) * fixzero_slowVariable                    'rate from I1 to E1

        ' I2 inactivation gate (Ca-dependent)
        Dim alpha2 As Double = (fCaina * a2on_NCX + (1 - fCaina) * a2off_NCX)              'rate from E1 to I2
        Dim beta2 As Double = (fCaina * b2on_NCX + (1 - fCaina) * b2off_NCX)                 'rate from I2 to E1

        Dim alphaE As Double = k2_NCX * E2Na + k4_NCX * E2Ca                  'overall transition rate from E2 to E1
        Dim betaE As Double = k1_NCX * E1Na + k3_NCX * E1Ca                    'overall transition rate from E1 to E2

        Dim E2_NCX As Double = 1 - E1_NCX - I1_NCX - I2_NCX

        INCX = frc * maxINCX * fNaCa * (k1_NCX * E1Na * E1_NCX - k2_NCX * E2Na * E2_NCX)
        INCX_Na = 3 * INCX
        INCX_Ca = -2 * INCX

        'If isIVcurve <> True Then                       'when no IV-curve is calculated
        dE1_NCXdt = E2_NCX * alphaE + I1_NCX * beta1 + I2_NCX * beta2 - E1_NCX * (betaE + alpha1 + alpha2)
        dI1_NCXdt = E1_NCX * alpha1 - I1_NCX * beta1
        dI2_NCXdt = E1_NCX * alpha2 - I2_NCX * beta2
        'End If
    End Sub

    '********** PMCA  **********
    Public maxIPMCA As Double = 0.19
    Const KmPMCA As Double = 0.0005       'mM
    Public IPMCA As Double
    Public IPMCA_Ca_iz As Double
    Public IPMCA_Ca_blk As Double

    Public dATP_PMCA_iz As Double        'mM   consumption of ATP by PMCA
    Public dATP_PMCA_blk As Double

    Private Sub IPMCAsub(ByVal frc, ByVal Ca, ByRef IPMCA, ByRef dNATP_PMCA)
        IPMCA = frc * maxIPMCA * Math.Pow(Ca, 1.6) / (Math.Pow(KmPMCA, 1.6) + Math.Pow(Ca, 1.6))
        dNATP_PMCA = IPMCA * Cm / 2.0 / Faraday                         ' amole / ms
    End Sub

    '********** ICabk **********
    Public PCab As Double = 0.0006822 * 0.1
    Public ICab As Double
    Public ICab_iz As Double
    Public ICab_blk As Double

    Private Sub ICabsub(ByVal frc As Double, ByVal GHKCa As Double, ByRef ICab As Double)
        ICab = PCab * frc * GHKCa                                                  '
    End Sub

    Dim Gtot As Double                          ' nS,  Whole cell conductance for all cations      Gtot(n+1)   
    Dim dV As Double = 0.2                  'half interval to measure the slope conductance

    Public LeadP As Double = -87                 'mV, Lead potential (n)
    Public oldLeadP As Double                  'mV, Lead potential (n-1)
    Public dVLdt As Double                    'dVL/dt

    Public sumGx As Double                  'total G
    Public sumGxEx As Double
    Public SCtbdVLdt As Double                   'summator of contributions of individual current components to dVL/dt
    Public SCtbVL As Double                     'summator of contributions of individual current components to VL
    Dim Ctb As Double
    Dim CtbEx As Double                        'contribution attributable to changes in Ex

    Public Sub calculateLeadP(ByVal dt As Double)     'Lead potential analysis based on the linearlized whole cell ion conductances
        Dim I As Integer
        For I = 1 To 35
            DA(3, I) = DA(2, I)                  'copy the Gion(n) as Gion(n-1)
            DA(5, I) = DA(4, I)                  'copy the Ex(n) as Ex(n-1)
        Next
        DA(3, 36) = DA(1, 36)                 'copy the IPMCA_iz(n) as IPMCA(n-1)
        DA(3, 37) = DA(1, 37)                 'copy the IPMCA_blk as IPMCA(n-1)
        oldLeadP = LeadP                      'copy Lead Potential to obtain dVL/dt directly if necessary

        FillDarray()       'fill DAR(1,I) including IPMCA  

        '*********** Calculate Gx, VL,  and finally VL  *****************
        sumGx = 0                                                     'prepare the summator of Gx
        sumGxEx = 0                     'prepare the summator of GxEx

        For I = 1 To 35                                             'Firstly, determine the sumGx of individual ionic current
            sumGx += DA(2, I)                                  'Sum of Gx
        Next I

        For I = 1 To 35                                                  'contribution to VL
            DA(7, I) = DA(2, I) * DA(4, I) / sumGx          'Gx*Ex / sumGx      
            sumGxEx += DA(7, I)                                    'Sum of contributions of ion channels to VL
        Next I

        DA(7, 36) = -DA(1, 36) / sumGx
        sumGxEx += DA(7, 36)                            'Sum of contributions of PMCA to VL
        DA(7, 37) = -DA(1, 37) / sumGx
        sumGxEx += DA(7, 37)                            'Sum of contributions of PMCA to VL

        LeadP = sumGxEx                                    'Lead Potential

        '***********      contributions of individual current component to dVL/dt (mV/msec) *********
        SCtbdVLdt = 0                        'summator of dVL/dt
        For I = 1 To 35                         'contribution of Ion channels to VL based on the analytical solution dVL/dt
            DA(6, I) = ((DA(2, I) - DA(3, I)) * (DA(4, I) - LeadP) + DA(2, I) * (DA(4, I) - DA(5, I))) / sumGx / dt  'mV/ms
            SCtbdVLdt += DA(6, I)   'mV/ms
            DA(9, I) = (DA(2, I) * (DA(4, I) - DA(5, I))) / sumGx / dt      'contribution attributed to Ex change
            DA(8, I) = DA(9, I) / Ctb
        Next I

        DA(6, 36) = -(DA(1, 36) - DA(3, 36)) / sumGx / dt      'IPMCA_iz
        SCtbdVLdt += DA(6, 36)
        DA(6, 37) = -(DA(1, 37) - DA(3, 37)) / sumGx / dt      'IPMCA_blk
        SCtbdVLdt += DA(6, 37)

        dVLdt = (LeadP - oldLeadP) / dt       'ms/mV    direct calculation of dVL/dt

    End Sub

    Public Sub FillDarray()

        '*****  Ion channel current,  Gion(n) and Ex(n) **********************
        DA(1, 1) = INaT_Na_cyt
        DA(2, 1) = PNa * (1 - LSMratio) * O_TM * dGHKNa    'slope conductance
        DA(4, 1) = ErevNa

        DA(1, 2) = INaT_K_cyt
        DA(2, 2) = PNa * (1 - LSMratio) * relativePK_INa * O_TM * dGHKK   'slope conductance
        DA(4, 2) = ErevK

        DA(1, 3) = INaL_Na_cyt
        DA(2, 3) = PNa * LSMratio * O_LSM * dGHKNa   'slope conductance
        DA(4, 3) = ErevNa

        DA(1, 4) = INaL_K_cyt
        DA(2, 4) = PNa * LSMratio * relativePK_INa * O_LSM * dGHKK   'slope conductance
        DA(4, 4) = ErevK

        DA(1, 5) = ICaLCa_LR
        DA(2, 5) = FrcCaL_jnc * PCaL * Yooo * ATPfactor * dGHKCa_LR   'slope conductance
        DA(4, 5) = ErevCa_LR

        DA(1, 6) = ICaLCa_L0
        DA(2, 6) = FrcCaL_jnc * PCaL * Yooc * ATPfactor * dGHKCa_L0   'slope conductance
        DA(4, 6) = ErevCa_L0

        DA(1, 7) = ICaLNa_jnc
        DA(2, 7) = FrcCaL_jnc * perNa_ICaL * (Yooo + Yooc) * ATPfactor * dGHKNa   'slope conductance
        DA(4, 7) = ErevNa

        DA(1, 8) = ICaLK_jnc
        DA(2, 8) = FrcCaL_jnc * perK_ICaL * (Yooo + Yooc) * ATPfactor * dGHKK   'slope conductance
        DA(4, 8) = ErevK

        DA(1, 9) = ICaLCa_iz
        DA(2, 9) = FrcCaL_iz * PCaL * Yoo_iz * ATPfactor * dGHKCa_iz   'slope conductance
        DA(4, 9) = ErevCa_iz

        DA(1, 10) = ICaLNa_iz
        DA(2, 10) = FrcCaL_iz * perNa_ICaL * Yoo_iz * ATPfactor * dGHKNa   'slope conductance
        DA(4, 10) = ErevNa

        DA(1, 11) = ICaLK_iz
        DA(2, 11) = FrcCaL_iz * perK_ICaL * Yoo_iz * ATPfactor * dGHKK   'slope conductance
        DA(4, 11) = ErevK

        DA(1, 12) = ICaLCa_blk
        DA(2, 12) = FrcCaL_blk * PCaL * Yoo_blk * ATPfactor * dGHKCa_blk   'slope conductance
        DA(4, 12) = ErevCa_blk

        DA(1, 13) = ICaLNa_blk
        DA(2, 13) = FrcCaL_blk * perNa_ICaL * Yoo_blk * ATPfactor * dGHKNa   'slope conductance
        DA(4, 13) = ErevNa

        DA(1, 14) = ICaLK_blk
        DA(2, 14) = FrcCaL_blk * perK_ICaL * Yoo_blk * ATPfactor * dGHKK   'slope conductance
        DA(4, 14) = ErevK

        DA(1, 15) = ICab_iz
        DA(2, 15) = Frc_iz * PCab * dGHKCa_iz   'slope conductance
        DA(4, 15) = ErevCa_iz

        DA(1, 16) = ICab_blk
        DA(2, 16) = Frc_blk * PCab * dGHKCa_blk   'slope conductance
        DA(4, 16) = ErevCa_blk

        DA(1, 17) = IK1_K_cyt
        DA(2, 17) = GK1 * fK_IK1 * pO_IK1   'chord  conductance by Ohmic equation
        DA(4, 17) = EK

        DA(1, 18) = IKr_K_cyt
        DA(2, 18) = GIKr * (Ko / 4.5) ^ 0.5 * pO_IKr   'chord conductance by OHmic equation
        DA(4, 18) = EK

        DA(1, 19) = IKs_K_iz
        DA(2, 19) = Frc_iz * PKs / 300 * pO_IKs_iz * dGHKK   'slope conductance
        DA(4, 19) = ErevK

        DA(1, 20) = IKs_Na_iz
        DA(2, 20) = Frc_iz * perNa_IKs / 300 * pO_IKs_iz * dGHKNa   'slope conductance
        DA(4, 20) = ErevNa

        DA(1, 21) = IKs_K_blk
        DA(2, 21) = Frc_blk * PKs / 300 * pO_IKs_blk * dGHKK   'slope conductance
        DA(4, 21) = ErevK

        DA(1, 22) = IKs_Na_blk
        DA(2, 22) = Frc_blk * perNa_IKs / 300 * pO_IKs_blk * dGHKNa   'slope conductance
        DA(4, 22) = ErevNa

        DA(1, 23) = IKpl_K_cyt
        IKplsub(Vm + dV)                                           'IK plateau  at Vm + dV mV
        Dim IKplpdV As Double = IKpl_K_cyt
        IKplsub(Vm - dV)                                             'IK plateau  at Vm + dV mV
        Dim IKplndV As Double = IKpl_K_cyt
        DA(2, 23) = (IKplpdV - IKplndV) / (2 * dV)      'slope conductance at the Vm depending on both GHK and Frectification
        DA(4, 23) = Vm - DA(1, 23) / DA(2, 23)             'Ex obtained with Vrctf_KPl and GHK

        DA(1, 24) = IKto_Na_cyt
        DA(2, 24) = pNa_IKto * pO_IKto * dGHKNa   'slope conductance
        DA(4, 24) = ErevNa

        DA(1, 25) = IKto_K_cyt
        DA(2, 25) = PKtoEndo * pO_IKto * dGHKK   'slope conductance
        DA(4, 25) = ErevK

        DA(1, 26) = IbNSC_K_cyt
        DA(2, 26) = PK_IbNSC * dGHKK   'slope conductance
        DA(4, 26) = ErevK

        DA(1, 27) = IbNSC_Na_cyt
        DA(2, 27) = PNa_IbNSC * dGHKNa   'slope conductance
        DA(4, 27) = ErevNa

        DA(1, 28) = ILCa_Na_iz
        DA(2, 28) = PNa_ILCa * Frc_iz * pO_LCa_iz * dGHKNa   'slope conductance
        DA(4, 28) = ErevNa

        DA(1, 29) = ILCa_K_iz
        DA(2, 29) = perK_ILCa * Frc_iz * pO_LCa_iz * dGHKK   'slope conductance
        DA(4, 29) = ErevK

        DA(1, 30) = ILCa_Na_blk
        DA(2, 30) = PNa_ILCa * Frc_blk * pO_LCa_blk * dGHKNa   'slope conductance
        DA(4, 30) = ErevNa

        DA(1, 31) = ILCa_K_blk
        DA(2, 31) = perK_ILCa * Frc_blk * pO_LCa_blk * dGHKK   'slope conductance
        DA(4, 31) = ErevK

        DA(1, 32) = IKATPK_cyt
        DA(2, 32) = GKATP * gamma_IKTP * pO_IKTP     'chord conductance by Ohm equation
        DA(4, 32) = EK

        DA(1, 33) = INCX_iz                                    ' INCX_iz(n)
        INCXsub(FrcNCX_iz, Vm + dV, Nai, Cafree_iz, INCX_iz, INCXNa_iz, INCXCa_iz, E1NCX_iz, I1NCX_iz, I2NCX_iz, dE1NCX_izdt, dI1NCX_izdt, dI2NCX_izdt)
        Dim INCXpdV As Double = INCX_iz
        INCXsub(FrcNCX_iz, Vm - dV, Nai, Cafree_iz, INCX_iz, INCXNa_iz, INCXCa_iz, E1NCX_iz, I1NCX_iz, I2NCX_iz, dE1NCX_izdt, dI1NCX_izdt, dI2NCX_izdt)
        Dim INCXndV As Double = INCX_iz
        DA(2, 33) = (INCXpdV - INCXndV) / (2 * dV)         'slope conductance of INCX at  Vm
        DA(4, 33) = Vm - DA(1, 33) / DA(2, 33)              'Ex obtained with the slope conductance

        DA(1, 34) = INCX_blk                                   ' INCX_iz(n)
        INCXsub(FrcNCX_blk, Vm + dV, Nai, Cafree_blk, INCX_blk, INCXNa_blk, INCXCa_blk, E1NCX_blk, I1NCX_blk, I2NCX_blk, dE1NCX_blkdt, dI1NCX_blkdt, dI2NCX_blkdt)
        INCXpdV = INCX_blk
        INCXsub(FrcNCX_blk, Vm - dV, Nai, Cafree_blk, INCX_blk, INCXNa_blk, INCXCa_blk, E1NCX_blk, I1NCX_blk, I2NCX_blk, dE1NCX_blkdt, dI1NCX_blkdt, dI2NCX_blkdt)
        INCXndV = INCX_blk
        DA(2, 34) = (INCXpdV - INCXndV) / (2 * dV)   'slope conductance of INCX at  Vm
        DA(4, 34) = Vm - DA(1, 34) / DA(2, 34)              'Ex obtained with the slope conductance

        DA(1, 35) = INaK
        INaKsub(1, Vm + dV, Nai, Ki, MgATP_cyt, MgADP_cyt, INaK, INaK_Na_cyt, INaK_K_cyt, dATP_NaK, P1_6_NaK, P7_NaK, P8_13_NaK, dP1_6_NaKdt, dP7_NaKdt, dP8_13_NaKdt)
        Dim INaKpdV As Double = INaK
        INaKsub(1, Vm - dV, Nai, Ki, MgATP_cyt, MgADP_cyt, INaK, INaK_Na_cyt, INaK_K_cyt, dATP_NaK, P1_6_NaK, P7_NaK, P8_13_NaK, dP1_6_NaKdt, dP7_NaKdt, dP8_13_NaKdt)
        Dim INaKndV As Double = INaK
        DA(2, 35) = (INaKpdV - INaKndV) / (2 * dV)         'slope conductance of INCX at  Vm
        DA(4, 35) = Vm - DA(1, 35) / DA(2, 35)              'Ex obtained with the slope conductance

        '*****  Pump Current amplitude (n) **********************
        DA(1, 36) = IPMCA_Ca_iz
        DA(1, 37) = IPMCA_Ca_blk

    End Sub

    '********** contration of the NL model 2008  **********
    Public IsoMetric As Boolean = False
    Public Ca As Double                             'mM,  concentration of free Ca 

    Public halfSL As Double                       'um, half sarcomere length
    Public crsBrLnX As Double                  'um, X in the NL model

    Public TSt = 23                                     'uM  total TS,  = number of Tn / 3

    Public TS As Double                            'uM, troponin-system Ca-free
    Public TSCa3 As Double                     'uM, TS bound with 3 Ca
    Public TSCa3W As Double
    Public TSCa3S As Double
    Public TSS As Double
    Public TSW As Double
    Public dydtTS As Double                     'uM/msec,  rate of change 
    Public dTSCa3dt As Double
    Public dTSCa3Wdt As Double
    Public dTSCa3Sdt As Double
    Public dTSSdt As Double
    Public dTSWdt As Double

    Public dydtCa As Double                     'uM/msec, rate of Ca change

    Public Yb As Double = 0.1816            'Ca-binding rate
    Public Zb As Double = 0.1397            'Ca-unbinding rate

    Public ratef As Double = 0.0023          'rate for weak CB
    Public eqvhalfSL As Double = 1.15     'a half sarcomere length, giving a maximum rate f
    Public convertF As Double = 15          'L dependency of rate f

    Public Yp As Double = 0.1397           'forming strong CB
    Public Zp As Double = 0.2095           'relaxing to weak CB

    Public Yr As Double = 0.1397           'Ca unbinding rate from strong CB
    Public Zr As Double = 7.2626          'Ca binding rate to strong CB  
    Public Yq As Double = 0.2328          'relaxing to weak CB without Ca
    Public Zq As Double = 0.3724          'forming strong CB without Ca

    Public Za As Double = 0.0023
    Public Yv As Double = 1.5
    Public Yvd As Double = Yv                '1.5   dependency on h of gd, magnifying factor

    Public rateg As Double                       'releasing CB weak
    Public rategd As Double                           'relaxation rate
    Public Yd As Double = 0.0333           'primary rate of gd
    Public Yc As Double = 1                     'dependency on half sarcomere length of Yd 
    Public Lc As Double = 1.2                  'reference length of sarcomere length


    Public Fbpeak As Double                           ' mN mm-2  force of cross bridge
    Public Fb As Double                           ' mN mm-2  force of cross bridge
    Public Fbw As Double                           ' mN mm-2  force of cross bridge
    Public Fbp As Double                           ' mN mm-2  force of cross bridge
    Public hw As Double                           'um, elongation of weak cross bridge
    Public dhwdt As Double                     'um/msec, rate of hw change
    Public hwr As Double = 0.0001         'um, reference value of h
    Public hp As Double                           'um, elongation of strong cross bridge
    Public dhpdt As Double                      'um/msec, rate of hw change
    Public hpr As Double = 0.006            'reference value of h
    Public rateB As Double = 0.5             '1.2    '0.5    'amplitude factor for h change

    Public Ap As Double = 2700             'mN mm-2 um-1 uM-1      force of myofilaments strong cross bridge
    Public Aw As Double = 540              'mN mm-2 um-1 uM-1       weak cross bridge force of myofilaments

    Public Fp As Double                        'mN mm-2                           force of parallel elastic component
    Public Ke As Double = 105000      'mN mm-2 um-5
    Public L0 As Double = 0.97            'um    resting half sarcomere length giving Fp = 0
    Public Le As Double = 10               'mN mm-2 um-5                 'spring constant for a linear elastic component

    Public Fext As Double                    'mN mm-2                external force
    Public isometricL As Double
    Public dATP_contraction As Double
    Const RatioATP_contraction As Double = 0.08             '0.08         0.025   
    Public JCaTnC_3Ca As Double                         'Ca flux due to Ca binding to Troponin system 

    Public Sub Contraction(ByVal mMca As Double)
        crsBrLnX = halfSL - hp                                                         'renew crsBrLnX
        TS = TSt - TSCa3 - TSCa3W - TSCa3S - TSS - TSW    'renew TS

        Dim uMCa As Double = mMca * 1000                          'convert from mM to microM
        Dim propFh As Double = 28000    'converting factor
        rateg = Za + Yv * (1 - Exp(-propFh * (hw - hwr) ^ 2))                                              'rateg dependent on h 
        rategd = Yd + Yc * (halfSL - Lc) ^ 2 + Yvd * (1 - Exp(-propFh * (hw - hwr) ^ 2))    'rate gd dependent on both L and h
        '************ State transition according to a new [Ca] *******************************
        dTSCa3dt = Yb * TS * uMCa ^ 3 - Zb * TSCa3 + rateg * TSCa3W - ratef * Exp(-convertF * (halfSL - eqvhalfSL) ^ 2) * TSCa3
        dTSCa3Wdt = ratef * Exp(-convertF * (halfSL - eqvhalfSL) ^ 2) * TSCa3 - rateg * TSCa3W + Zp * TSCa3S - Yp * TSCa3W
        dTSCa3Sdt = Yp * TSCa3W - Zp * TSCa3S + Zr * TSS * uMCa ^ 3 - Yr * TSCa3S
        dTSSdt = Yr * TSCa3S - Zr * TSS * uMCa ^ 3 + Zq * TSW - Yq * TSS
        dTSWdt = Yq * TSS - Zq * TSW - rategd * TSW

        '************ Time-dependent changes in hw and hs  *******************************
        dhwdt = -rateB * (hw - hwr)
        dhpdt = -rateB * (hp - hpr)

        '************ Time-dependent change in [Ca]  *******************************
        'Dim JTSCa3 As Double = Yb * TS * uMCa ^ 3 - Zb * TSCa3 + rateg * TSCa3W - ratef * Exp(-convertF * (halfSL - eqvhalfSL) ^ 2) * TSCa3
        'Dim JTSCa3W As Double = ratef * Exp(-convertF * (halfSL - eqvhalfSL) ^ 2) * TSCa3 - rateg * TSCa3W + Zp * TSCa3S - Yp * TSCa3W
        'Dim JTSCa3S As Double = Yp * TSCa3W - Zp * TSCa3S + Zr * TSS * uMCa ^ 3 - Yr * TSCa3S

        'JCaTnC_3Ca = 3 * (dTSCa3dt + dTSCa3Wdt + dTSCa3Sdt) / 1000        'in mM
        'JCaTnC_3Ca = 3 * (JTSCa3 + JTSCa3W + JTSCa3S) / 1000        'in mM

        dATP_contraction = RatioATP_contraction * (Yp * TSCa3W + Zq * TSW) * Vol_cyt            ' amole / ms

        If IsoMetric = True Then                       'determine halfSL according to the isometric or isotonic contraction
            halfSL = isometricL
        Else
            Dim newhalfSL As Double    'temporary halfSL to calculate the balance in forces, Fb, Fp and Fext
            Dim temphs As Double = hp          'temporary hs
            Dim temphw As Double = hw         'temporary hw
            Dim Resolution As Double
            'Fb = 3 * (Aw * (TSCa3W + TSW) * hw + Ap * (TSCa3S + TSS) * hp)      'force of cross bridge
            'Fp = Ke * (halfSL - L0) ^ 5 + Le * (halfSL - L0)
            Dim FnetA As Double = Fext - Fp - Fb    'net force
            Dim FnetB As Double                               '= old FnetA
            If FnetA > 0 Then
                Resolution = 0.000005                        'in determining h
            Else
                Resolution = -0.000005
            End If
            Do                                                               'determine newhalfSL  
                FnetB = FnetA                                        'copy the old FnetA
                temphs = temphs + Resolution             'a step forward
                temphw = temphw + Resolution
                If FnetB = 0 Then Exit Do 'already at the evquilibrium
                newhalfSL = crsBrLnX + temphs               'crsBrLnX is fixed in this Do Loop
                If newhalfSL > 1.3 Then Exit Do 'error and stop calculation
                Fp = Ke * (newhalfSL - L0) ^ 5 + Le * (newhalfSL - L0)
                Fbw = Aw * 3 * (TSCa3W + TSW) * temphw
                Fbp = Ap * 3 * (TSCa3S + TSS) * temphs
                Fb = Fbw + Fbp
                FnetA = Fext - Fp - Fb                              'the balance sheet
            Loop While (FnetA * FnetB) > 0                  'loop while force balance is not reversed
            hp = temphs - Resolution                    'a step backward
            hw = temphw - Resolution                   'a step backward
            halfSL = crsBrLnX + hp                       ' solution of the Do Loop 
        End If

        '*************** developed tension ********************************************
        Fb = 3 * (Aw * (TSCa3W + TSW) * hw + Ap * (TSCa3S + TSS) * hp)      'force of cross bridge
        Fp = Ke * (halfSL - L0) ^ 5 + Le * (halfSL - L0)                                         'force of parallel elastic component
        If Fb > Fbpeak Then Fbpeak = Fb 'peak search
        'Carry(10) = Fb

    End Sub

    Public Sub dydt_mit()                            ' 14 variables

        ddpsidt = (-4 * vC1 - 2 * vC3 - 4 * vC4 + nA * vSN + vANT + vLK + vKuni) * Faraday / Cm_mit       ' mV / ms

        dATPt_cytdt = (vANT - vCons + vAK + vCK + vglyc) / Vol_cyt                                                  ' mM / ms
        dADPt_cytdt = (vCons - vANT - 2 * vAK - vCK - vglyc) / Vol_cyt
        dPi_cytdt = (vCons - vPiT - vglyc) / Vol_cyt
        dPCr_cytdt = (-vCK) / Vol_cyt
        'dH_cytdt = (4 * vC1 + 4 * vC3 + 2 * vC4 - nA * vSN - vPiT - vLK - vKHex - vEFF) / HBuff_cyt / Vol_cyt

        dATPt_mitdt = (vSN - vANT) / Vol_mit
        dPi_mitdt = (vPiT - vSN) / Vol_mit
        dNADH_mitdt = (vDH - vC1) / Vol_mit / NADbuf_mit
        dH_mitdt = (-4 * vC1 - 2 * vC3 - 4 * vC4 + nA * vSN + vPiT + vLK + vKHex) / Hbuff_mit / Vol_mit
        dK_mitdt = (vKuni - vKHex) / Vol_mit
        dUQr_mitdt = (vC1 - vC3) / Vol_mit
        dctCrd_mitdt = (vC3 - vC4) * 2 / Vol_mit
        'dO2_mitdt = 0

    End Sub

    '********************************  Mitochondria ********************************
    Public Hbuff_mit As Double                 'unitless, H+ buffer power
    Public pH_mit As Double                    '  unitless,  pH 

    Public Pifree_mit As Double                       'mM concentration
    Public ADPt_mit As Double                   'mM concentration
    Public ADPfr_mit As Double                  'mM concentration
    Public MgADP_mit As Double               'mM concentration
    Public ATPfr_mit As Double                   'mM concentration
    Public MgATP_mit As Double                'mM concentration

    Public NAD_mit As Double                     'mM concentration
    Public UQo_mit As Double                     'mM concentration

    Public ctC_ox_mit As Double                   'mM concentration
    Public ctA_rd_mit As Double                    'mM concentration
    Public ctA_ox_mit As Double                   'mM concentration

    Public dpH As Double                             'mV    delta pH
    Public dp As Double                               'mV    delta H motive force

    Public EmNAD_mit As Double                'mV, redox potential
    Public EmUQ_mit As Double                  'mV, redox potential
    Public EmcytC_mit As Double                'mV, redox potential
    Public EmcytA_mit As Double                'mV, redox potential

    '******************************** cytosol ********************************
    Public HBuff_cyt As Double                 'H+ buffer power 
    Public pH_cyt As Double

    Public Pifree_cyt As Double                      'mM concentration
    Public AMP_cyt As Double                  'mM concentration
    Public ADPfr_cyt As Double                'mM concentration
    Public MgADP_cyt As Double             'mM concentration
    Public ATPfr_cyt As Double                 'mM concentration
    Public MgATP_cyt As Double              'mM concentration
    Public Cr_cyt As Double                      'mM concentration

    Public Sub cytSubstrateConcentrations()
        '****************** fixed *****************
        Pifree_cyt = 0.50872066859173026
        AMP_cyt = 0.00033459021041526427
        AMP_cyt = 0.00033459021041526427
        ADPfr_cyt = 0.0022536111580301241
        MgADP_cyt = 0.025978226605534577
        ATPfr_cyt = 0.039789862258604494
        MgATP_cyt = 6.631643709767415
        Cr_cyt = 12.6772372798697

        ''***************************** Pi, AMP, ADP free, MgADP, ATP free, MgATP, Cr, ***************************************** 
        'pH_cyt = -Log10(H_cyt / 1000)
        ' ''HBuff_cyt = 25 / 2.3 / H_cyt

        'Pifree_cyt = Pi_cyt / (1 + Math.Pow(10, (pH_cyt - pKa_Pi)))                 '   

        'AMP_cyt = AXPSUM_cyt - (ATPt_cyt + ADPt_cyt)                                 'mM, AMP concentration  total                            
        'ADPfr_cyt = ADPt_cyt / (1 + Mg_cyt / KdADP_cyt)                               'mM, Instantaneous equilibrium for Mg + ADP binding   
        'MgADP_cyt = ADPt_cyt - ADPfr_cyt                                                        '
        'ATPfr_cyt = ATPt_cyt / (1 + Mg_cyt / KdATP_cyt)                                'mM,  Instantaneous equilibrium for Mg + ATP binding   
        'MgATP_cyt = ATPt_cyt - ATPfr_cyt                                                         '
        'Cr_cyt = CrSUM_cyt - PCr_cyt                                                                'mM, creatine concentration             

    End Sub

    '********** mitochondria   15 variables **********
    Public ATPt_mit As Double             'mM  matrix ATP concentration
    Public Pi_mit As Double                  'mM  matrix Pi concentration
    Public NADH_mit As Double           'mM  matrix NADH concentration
    Public H_mit As Double                   'mM  matrix H concentration
    Public K_mit As Double                   'mM matrix K concentration
    Public UQr_mit As Double               'mM matrix UQH concentration
    Public ctCrd_mit As Double              'mM matrix cytcrome concentration
    Public O2_mit As Double = 0.24       'mM O2 concentration   This is constant in the present model.

    Public dpsi As Double                          'mV  matrix membrane potential

    Public dATPt_mitdt As Double             'mM/ms,  rate of matrix ATP concentration
    Public dPi_mitdt As Double                  'mM/ms,  rate of matrix Pi concentration
    Public dNADH_mitdt As Double           'mM/ms,  rate of matrix NADH concentration
    Public dH_mitdt As Double                   'mM/ms,  rate of matrix H concentration
    Public dK_mitdt As Double                   'mM/ms,  rate of matrix K concentration
    Public dUQr_mitdt As Double               'mM/ms,  rate of matrix UQH concentration
    Public dctCrd_mitdt As Double              'mM/ms,  rate of matrix cytcrome concentration
    'Public dO2_mitdt As Double                 'mM/ms,  rate of O2 concentration

    Public ddpsidt As Double                       'mV/ms, rate of matrix membrane potential

    '******************************** Mitochondria MODEL CONSTANTS  ********************************
    Public pKa_Pi As Double = 6.8                                                'unitless, pK for Pi      
    Public AXPSUM_mit As Double = 16.26                         'mM     
    Public NADt_mit As Double = 2.79                                  'mM     
    Public UQt_mit As Double = 1.35                                    'mM    
    Public cytCt_mit As Double = 0.27                                  'mM    
    Public cytAt_mit As Double = 0.135                                 'mM    
    Public Mg_mit As Double = 0.38                                      'mM          corrected from 0.3 mM by AN on 24 May
    Public KdADP_mit As Double = 0.282
    Public kdATP_mit As Double = 0.017
    Public EmNAD0_mit As Double = -320                           'mV   
    Public EmUQ0_mit As Double = 40                                 'mV   
    Public EmctC0_mit As Double = 250                             'mV   
    Public EmctA0_mit As Double = 540                              'mV  

    Public nA As Double = 2.5                                                 'unitless    
    Public NADbuf_mit As Double = 5                                   'unitless,   buffering capacity coefficient 

    Public Sub mitConcentrations()

        pH_mit = -Log10(H_mit / 1000)
        'Hbuff_mit = 0.022 / ((Math.Pow(10, -pH_mit) - Math.Pow(10, -(pH_mit + 0.001))) / 0.001)
        Hbuff_mit = 22 / 2.3 / H_mit

        '******************************free Pi, free ADP, MgADP, free ATP, MgATP **********************************************************
        Pifree_mit = Pi_mit / (1 + 10 ^ (pH_mit - pKa_Pi))            'phosphate-,    

        ADPt_mit = AXPSUM_mit - ATPt_mit                         'determine mit_tADP    

        ADPfr_mit = ADPt_mit / (1 + Mg_mit / KdADP_mit)          'equilibrium for Mg + ADP binding  
        MgADP_mit = ADPt_mit - ADPfr_mit                                   'MgADP concentration (mM)        
        ATPfr_mit = ATPt_mit / (1 + Mg_mit / kdATP_mit)             'equilibrium for Mg + ATP binding   
        MgATP_mit = ATPt_mit - ATPfr_mit                                     'MgATP concentration (mM)          

        '****************************** NAD, NADH,EmNaD  **********************************************************
        NAD_mit = NADt_mit - NADH_mit           'determine [NAD]          

        EmNAD_mit = EmNAD0_mit + RTF / 2 * Log(NAD_mit / NADH_mit)      'redox potential for NAD/NADH c   

        '****************************** UQo, UQr, EmUQ  **********************************************************
        UQo_mit = UQt_mit - UQr_mit                'determine [UQ]                                          

        EmUQ_mit = EmUQ0_mit + RTF / 2 * Log(UQo_mit / UQr_mit)       'redox potential of UQ/UQH2 complex

        '****************************** cytCo, cytCr  EmcytC  **********************************************************
        ctC_ox_mit = cytCt_mit - ctCrd_mit                     'determine mit_c3p                                 

        EmcytC_mit = EmctC0_mit + RTF * Log(ctC_ox_mit / ctCrd_mit)           'redox potential of cytochrome  

        '****************************** pH, dpH, dp  **********************************************************
        dpH = RTF * Log(H_cyt / H_mit)           'determine H equilibrium potential 
        dp = dpH - dpsi                                               'corrected by AN

        '****************************** EmcytA, cytAr, cytAo  **********************************************************
        EmcytA_mit = EmcytC_mit + dp - dpsi           'redox potential of cytochrome A3  
        'EmcytA_mit = EmcytC_mit + dpH - dpsi 

        ctA_rd_mit = cytAt_mit / (1 + Exp((EmcytA_mit - EmctA0_mit) / RTF))
        ctA_ox_mit = cytAt_mit - ctA_rd_mit

    End Sub

    '****************** Krebs cycle ***************************
    Public vDH As Double                                                    '  rate of NADH production
    Public k_DH As Double = 96293 / 1000 / 60 / 1000 * ampRatio '   mM . ms-1     
    Public Km_DH As Double = 100                                     'unitless           
    Public pD_DH As Double = 0.8                                       'unitless            

    '******************************** INNER MITOCHONDRIAL MEMBRANE PROCESSES  ********************************
    Public vC1 As Double                                                      '   rate of Complex I
    Public k_C1 As Double = 819.61 / 1000 / 60 / 1000 * ampRatio ' 'mM/ms/ mV-1    
    Public dG_C1 As Double

    Public vC3 As Double                                                      ' rate of Complex III
    Public k_C3 As Double = 467.9 / 1000 / 60 / 1000 * ampRatio ' 'mM. ms-1. mV-1    
    Public dG_C3 As Double

    Public vC4 As Double                                                     'rate of Complex IV
    Public k_C4 As Double = 24.696 / 60 * ampRatio           ' 'mM-1. ms-1   
    Public Km_C4 As Double = 0.15                                      'mM                 

    Public vSN As Double                                                    '   rate of ATP synthase
    Public k_SN As Double = 117706 / 1000 / 60 / 1000 * ampRatio ' 'mM. ms-1       
    Public Gamma As Double
    Public dG_SN0 As Double = 31.9 * 1000                                 'J mol^1         

    Public vANT As Double                                                    '  rate of ATP/ADP exchange
    Public k_ANT As Double = 187185 / 1000 / 60 / 1000 * ampRatio    ' mM. ms-1             
    Public Km_ANT As Double = 0.0025                                '  mM from source code   10.08.03 Cha 
    Public fANT As Double = 0.65                                            'unitless           

    Public vPiT As Double                                                        '  rate of Pi uniporter
    Public k_PiT As Double = 238.113 / 60 * ampRatio             ' '69421 / 60 / 1000 * mit_Fscale

    '****************** [K]mit dynamics *******************************************
    Public vLK As Double                                                       '  rate of Pi leak
    Public k_LK1 As Double = 8.5758 / 1000 / 60 / 1000         '* Sc_Cell   'mM-1. ms-1            
    Public k_LK2 As Double = 0.038                                       'mV-1   

    Public vKuni As Double                                                    ' rate of K uniporter
    Public k_Kuni As Double = 0.00001 / 60 / 1000                  '* Sc_Cell       ' = 0.00001 / 60 / 1000          'ms-1                   

    Public vKHex As Double                                                    '  rate of K/H exchange
    Public k_KHex As Double = 0.003453 / 60                          '* Sc_Cell     '      


    Public Sub mitRate()
        ' unit     amole / ms
        '****************************** rate of NADH production   **********************************************************
        vDH = k_DH / (1 + Km_DH / (NAD_mit / NADH_mit)) ^ pD_DH   'rate of NADH production             

        '**************************   rate of  complex I ********************************************************************
        dG_C1 = EmUQ_mit - EmNAD_mit - dp * 2
        vC1 = k_C1 * dG_C1                         'velocity of complexI to complexIII 

        '**************************    rate of complex III ********************************************************************
        dG_C3 = EmcytC_mit - EmUQ_mit - dp * 2 - dpsi
        vC3 = k_C3 * dG_C3

        '**************************    rate of complex IV ********************************************************************
        'vC4 = k_C4 * ctCrd_mit * ctA_rd_mit / (1 + Km_C4 / O2_mit)
        vC4 = k_C4 * ctCrd_mit * ctA_rd_mit * O2_mit / (O2_mit + Km_C4)

        ''**************************   rate of ATP synthase    ***************************************************************
        'Gamma = Math.Exp(nA * dp / RTF - (dG_SN0 / (R * tempK) + Math.Log( ATPt_mit / (ADPt_mit * Pi_mit))))
        Gamma = Exp(nA * dp / RTF - (dG_SN0 / (R * tempK) + Log(1000 * ATPt_mit / (ADPt_mit * Pi_mit))))
        vSN = k_SN * (Gamma - 1) / (Gamma + 1)

        '**************************   rate of Proton leak********************************************************************
        vLK = k_LK1 * (Exp(k_LK2 * dp) - 1)

        '**************************   rate of ATP/ADP antiport **************************************************************
        Dim tmp1 As Double
        Dim tmp2 As Double

        tmp1 = ADPfr_cyt / (ADPfr_cyt + ATPfr_cyt * Exp((1 - fANT) * dpsi / RTF))
        tmp2 = ADPfr_mit / (ADPfr_mit + ATPfr_mit * Exp(-fANT * dpsi / RTF))
        vANT = k_ANT / (1 + (Km_ANT / ADPfr_cyt)) * (tmp1 - tmp2)

        '**************************   rate of Phosphate translocator ********************************************************
        vPiT = k_PiT * (Pifree_cyt * H_cyt - Pifree_mit * H_mit)

        '**************************   rate of potassium uniport *************************************************************
        vKuni = k_Kuni * (Ki * Exp(-dpsi / RTF / 2) - K_mit * Exp(dpsi / RTF / 2))

        '**************************   rate of H+/K+ exchanger ***************************************************************
        vKHex = k_KHex * (K_mit * H_cyt - Ki * H_mit)
    End Sub

    '********* cytosolic substrates for ATP homeostasis *******************
    Public ATPt_cyt As Double             'mM ATP concentration
    Public ADPt_cyt As Double             'mM  ADP concentration
    Public Pi_cyt As Double                   'mM cytosol Pi concentration
    Public PCr_cyt As Double               'mM PCr concentration
    Public H_cyt As Double = 0.0001     '0038276520463351       'mM cytosol H concentration   Constant at present 

    Public dATPt_cytdt As Double             'mM/ms,  rate of cytosol ATP concentration
    Public dADPt_cytdt As Double             'mM/ms,  rate of cytosol ADP concentration
    Public dPi_cytdt As Double                   'mM/ms,  rate of cytosol Pi concentration
    Public dPCr_cytdt As Double               'mM/ms,  rate of cytosol PCr concentration
    'Public dH_cytdt As Double                   'mM/ms,  rate of cytosol H concentration

    Public AXPSUM_cyt As Double = 6.7                              'mM    
    Public CrSUM_cyt As Double = 25                                   'mM    
    Public Mg_cyt As Double = 0.8                                             '
    Public KdATP_cyt As Double = 0.024
    Public KdADP_cyt As Double = 0.347

    Public vCons As Double                                                           'mM / ms,   rate of ATP consumption
    Public vCK As Double                                             'mM / ms    rate of Cr kinase

    Public kCKf As Double = 6.6056 * 1000 / 60 * Vol_cyt '* vRatio_mit    'mM-1. ms-1  from source code   10.08.03 Cha 
    Public kCKb As Double = 0.00005 * Vol_cyt '* vRatio_mit   'mM-1. ms-1   

    Public vAK As Double                                   'mM / ms,   rate of adenylate kinase
    Public kAKf As Double = 2956.98 / 60 * Vol_cyt '* vRatio_mit      'mM-1. ms-1    
    Public kAKb As Double = 78.0208 / 60 * Vol_cyt  '* vRatio_mit     'mM-1. ms-1 

    Public vglyc As Double                                    'mM. ms-1  rate of glycolysis in cytosol

    Public vEFF As Double
    Public kEFF As Double = 10000 / 1000 / 60 / 1000 * Vol_cyt '* vRatio_mit           'mM.pH-1. ms-1    
    Public Sub cytRate()
        ' unit     amole / ms
        '***************************** rate of ATP consumption ********************************************
        vCons = (dATP_NaK + dATP_PMCA_blk + dATP_PMCA_iz + dATP_SERCA) + dATP_contraction

        '***************************** rate of creatine kinase reaction  ************************************
        vCK = (kCKf * ADPt_cyt * PCr_cyt * H_cyt - kCKb * ATPt_cyt * Cr_cyt)                       ' from source code 10. 08. 03 Cha

        '***************************** rate of adenylate kinase reaction  ************************************
        vAK = (kAKf * ADPfr_cyt * MgADP_cyt - kAKb * MgATP_cyt * AMP_cyt)            'MgATP production by adenylate kinase

        '***************************** rate of pH change ??  ************************************
        vEFF = kEFF * (7.0 - pH_cyt)         'proton efflux from cytosol to extracellular space

        vglyc = 0.2 * vDH                                   'mM. ms-1,        rate of glycolysis
    End Sub

    Public Sub calculateNCXSteadyState(ByVal frc As Double, ByVal v As Double, ByVal Nai As Double, ByVal Cai As Double, ByRef INCX As Double, ByRef INCX_Na As Double, ByRef INCX_Ca As Double, _
                 ByVal E1_NCX As Double, ByVal I1_NCX As Double, ByVal I2_NCX As Double, ByRef dE1_NCXdt As Double, ByRef dI1_NCXdt As Double, ByRef dI2_NCXdt As Double)

        Dim k1_NCX As Double = Exp(partition_NCX * v / (R * tempK / Faraday))                            'voltage-dependent rate
        Dim k2_NCX As Double = Exp((partition_NCX - 1.0) * v / (R * tempK / Faraday))                 'voltage-dependent rate

        Dim E2Na As Double = 1.0 / (1.0 + ((KmNao_NCX / Nao) ^ nHNa_NCX) * (1.0 + Cao / KmCao_NCX))    'Na binding probability
        Dim E2Ca As Double = 1.0 / (1.0 + (KmCao_NCX / Cao) * (1.0 + ((Nao / KmNao_NCX) ^ nHNa_NCX)))   'Ca binding probability

        Dim E1Na As Double = 1.0 / (1.0 + ((KmNai_NCX / Nai) ^ nHNa_NCX) * (1 + Cai / KmCai_NCX))   'Na binding probability
        Dim E1Ca As Double = 1.0 / (1.0 + (KmCai_NCX / Cai) * (1.0 + ((Nai / KmNai_NCX) ^ nHNa_NCX)))   'Ca binding probability

        Dim fCaina As Double = Cai / (Cai + Kmact_NCX)          'Ca-dependency of the inactivation step

        ' I1 inactivation gate (Na-dependent)
        Dim alpha1 As Double = (E1Na * (fCaina * a1on_NCX + (1 - fCaina) * a1off_NCX)) * fixzero_slowVariable  'rate from E1 to I1
        Dim beta1 As Double = (fCaina * b1on_NCX + (1 - fCaina) * b1off_NCX) * fixzero_slowVariable                    'rate from I1 to E1

        ' I2 inactivation gate (Ca-dependent)
        Dim alpha2 As Double = (fCaina * a2on_NCX + (1 - fCaina) * a2off_NCX)              'rate from E1 to I2
        Dim beta2 As Double = (fCaina * b2on_NCX + (1 - fCaina) * b2off_NCX)                          'rate from I2 to E1

        Dim alphaE As Double = k2_NCX * E2Na + k4_NCX * E2Ca                  'overall transition rate from E2 to E1
        Dim betaE As Double = k1_NCX * E1Na + k3_NCX * E1Ca                    'overall transition rate from E1 to E2

        Dim E2_NCX As Double = 1 - E1_NCX - I1_NCX - I2_NCX

        '********************   model excluding pl1 state  (three state model ) of NCX *********************************************
        I2_NCX = (1 - I1_NCX) * alphaE * alpha2 / (alphaE * alpha2 + alphaE * beta2 + beta2 * betaE)    ' I2 in the  (1 - pI1_NCX)
        E1_NCX = (1 - I1_NCX) * alphaE * beta2 / (alphaE * alpha2 + alphaE * beta2 + beta2 * betaE)    ' E1 in the  (1 - pI1_NCX)
        E2_NCX = (1 - I1_NCX) * beta2 * betaE / (alphaE * alpha2 + alphaE * beta2 + beta2 * betaE)      ' E2 in the  (1 - pI1_NCX)

    End Sub

    Public Sub calculateINaSteadyState(ByVal v As Double, ByVal Iinact_tm As Double, ByVal Iinact_lsm As Double)
        Dim kC2O As Double = 1.0 / (0.0025 * Exp(v / (-8.0)) + 0.15 * Exp(v / (-100.0)))
        Dim kOC As Double = 1.0 / (30.0 * Exp(v / 12.0) + 0.53 * Exp(v / 50.0))
        Dim kOI2 As Double = 1.0 / (0.0433 * Exp(v / (-27)) + 0.34 * Exp(v / (-2000.0)))
        Dim kI2O As Double = 0.0001312
        Dim kC2I2 As Double = 1.0 / (1.0 + kI2O * kOC / kOI2 / kC2O)
        Dim kI2C As Double = 1.0 - kC2I2
        Dim fC As Double = 1 / (1 + Exp((v + 44.1) / -6.27))            ' fraction of C2 in (C1 + C2)

        Dim deno As Double                        'denominator
        Dim A As Double = fC * kC2O * kI2O + kI2C * fC * kC2O + fC * kC2I2 * kI2O     'O
        Dim B As Double = fC * kC2I2 * kOI2 + fC * kC2O * kOI2 + kOC * fC * kC2I2     'I2
        Dim C As Double = kI2C * kOC + kOI2 * kI2C + kI2O * kOC                                  'C
        deno = A + B + C
        C_TM = (1 - Iinact_tm) * C / deno              'y(8)   distribution in the the pC state within (1 - pIs_TM)
        O_TM = (1 - Iinact_tm) * A / deno              'y(9)   distribution in the pO state within (1 - pIs_TM) 
        I2_TM = (1 - Iinact_tm) * B / deno              'y(10)  distributtion in the pI2 state within (1 - pIs_TM)

        '****************  Calculate C-O-I1 of the gating of LSM *********** using common C1-C2-O gating with the transient mode ***********
        Dim kOI1 As Double = kOI2      'the same as the transient mode
        Dim kI1O As Double = 0.01
        Dim kI1C As Double = kI2C             'the same as the transient mode
        Dim kC2I1 As Double = kC2I2         'the same as the transient mode

        A = fC * kC2I1 * kI1O + kI1C * fC * kC2O + kI1O * fC * kC2O      'probability of pO within (pC + pO + pI1)
        B = fC * kC2I1 * kOI1 + kOC * fC * kC2I1 + fC * kC2O * kOI1        'probability of I1 within (pC + pO + pI1)
        C = kI1C * kOC + kOI1 * kI1C + kI1O * kOC                                   'probability of C within (pC + pO + pI1) 
        deno = A + B + C
        C_LSM = (1 - Iinact_lsm) * C / deno             'y(12)    distribute the pC_LSM  within the triangle of  (1 - pIs_LSM - pI2_LSM) 
        O_LSM = (1 - Iinact_lsm) * A / deno             'y(13)     distribute the pO_LSM  within the triangle of (1 - pIs_LSM - pI2_LSM) 
        I1_LSM = (1 - Iinact_lsm) * B / deno             'y(14)    distribute the pI1_LSM  within the triangle of  (1 - pIs_LSM - pI2_LSM) 

    End Sub

    Public Sub DefaultValues()
        Vm = -91.55752

        TnChCa = 0.1118547
        CaMCa = 0.0002271417
        bufferSRCa = 0.001727959

        Lb_jnc = 0.02380029
        Lb_iz = 0.00829182
        Hb_jnc = 0.1938908
        Hb_iz = 0.08161925

        Nai = 6.277602
        Ki = 139.6088
        Catot_jnc = 0.2179754
        Catot_iz = 0.09009045
        Catot_blk = 0.1139043
        Ca_SRup = 0.857742
        Catot_SRrl = 2.370628

        O_TM = 0.0000006879646
        I2_TM = 0.01156846
        Is_TM = 0.3015598

        O_LSM = 0.00000285718
        I1_LSM = 0.002465517
        I2_LSM = 0.01161808
        Is_LSM = 0.3006655

        Yco_iz = 0.970303
        Yoc_iz = 0.00000009280585
        Yoo_iz = 0.000003032515
        Yco_blk = 0.9709757
        Yoc_blk = 0.00000009070315
        Yoo_blk = 0.000003034618

        Yooo = 0.000001390504
        Yooc = 0.000001640441
        Ycoo = 0.0000177527
        Ycoc = 0.969783
        Ycco = 0.0000005069495
        Yoco = 0.000000000002534132
        Yocc = 0.0000000943734

        y1_IKr = 0.00004368053
        y2_IKr = 0.002862531
        y3_IKr = 0.9993651

        Ov_IKs = 0.004949171
        C2IKs_iz = 0.00976066
        OcIKs_iz = 0.04797447
        C2IKs_blk = 0.00984898
        OcIKs_blk = 0.01399778

        y1_IKto = 0.000521063
        y2_IKto = 0.5166466

        Pbspm = 0.5937188

        E1NCX_iz = 0.2649876
        I1NCX_iz = 0.1138406
        I2NCX_iz = 0.6204557
        E1NCX_blk = 0.1178701
        I1NCX_blk = 0.1720314
        I2NCX_blk = 0.7098817

        P1_6_NaK = 0.4560799
        P7_NaK = 0.07488691
        P8_13_NaK = 0.2582757


        halfSL = 1.0
        Fb = 0.02795932
        Fp = 0.3025515
        TSCa3 = 0.00890031
        TSCa3W = 0.0002772057
        TSCa3S = 0.0001146444
        TSS = 0.0004588994
        TSW = 0.0002418054
        hw = 0.0001
        hp = 0.006

        ATPt_cyt = 6.675219
        ADPt_cyt = 0.02406609
        Pi_cyt = 1.127577
        PCr_cyt = 12.75247

        '*********Variables described below for mitochondria *************
        ATPt_mit = 6.948746
        Pi_mit = 3.359158
        NADH_mit = 1.377339
        H_mit = 0.00003432782
        K_mit = 117.8002
        UQr_mit = 0.1785899
        ctCrd_mit = 0.05031882
        dpsi = -162.5778

        'Vm = -91.585940109504136

        'TnChCa = 0.11172521916169439
        'CaMCa = 0.00021453335215003443
        'bufferSRCa = 0.0016394735930375668

        'Lb_jnc = 0.023794729537137238
        'Lb_iz = 0.0082001494251161544
        'Hb_jnc = 0.19386705782160793
        'Hb_iz = 0.081044922100698608

        'Nai = 6.278104330556352
        'Ki = 139.60590200295147
        'Catot_jnc = 0.21793936725372437
        'Catot_iz = 0.089417855689152156
        'Catot_blk = 0.11366557313239029
        'Ca_SRup = 0.87309190463550856
        'Catot_SRrl = 2.3981827640291455

        'O_TM = 0.00000068326406838961989
        'I2_TM = 0.011516177448930394
        'Is_TM = 0.30088023610050896

        'O_LSM = 0.0000028335889586009951
        'I1_LSM = 0.0024463581737484258
        'I2_LSM = 0.011565224688528837
        'Is_LSM = 0.29999150959366688

        'Yco_iz = 0.9702833067312242
        'Yoc_iz = 0.000000092528668089729066
        'Yoo_iz = 0.000003021396885272064
        'Yco_blk = 0.97096754020807363
        'Yoc_blk = 0.000000090398008973362157
        'Yoo_blk = 0.0000030235279913233234

        'Yooo = 0.0000013974262979472623
        'Yooc = 0.0000016223937962347114
        'Ycoo = 0.000018020370648994048
        'Ycoc = 0.96975902767171973
        'Ycco = 0.00000051545921246758266
        'Yoco = 0.0000000000025648468931287271
        'Yocc = 0.000000094103275682083628

        'y1_IKr = 0.000043523663860484064
        'y2_IKr = 0.0028634715748834128
        'y3_IKr = 0.999367398320787

        'Ov_IKs = 0.0049423372238274555
        'C2IKs_iz = 0.0097621321689196332
        'OcIKs_iz = 0.048241445029556365
        'C2IKs_blk = 0.0098518925427404126
        'OcIKs_blk = 0.013759728630691586

        'y1_IKto = 0.00052007830620183417
        'y2_IKto = 0.51694523530461689

        'Pbspm = 0.5928766627906471

        'E1NCX_iz = 0.26301847516123134
        'I1NCX_blk = 0.17219845176931417
        'I2NCX_iz = 0.62271600990195719
        'E1NCX_blk = 0.11231329878175683
        'I1NCX_blk = 0.17146351207804253
        'I2NCX_blk = 0.71528501866035143

        'P1_6_NaK = 0.48071934639306474
        'P7_NaK = 0.074801949460253148
        'P8_13_NaK = 0.22202853434133574


        'halfSL = 1.0
        'Fb = 0.0232314500574
        'Fp = 0.30255150000000025
        'TSCa3 = 0.0074816776644491137
        'TSCa3W = 0.00023028653395111692
        'TSCa3S = 0.000094800674431182343
        'TSS = 0.00038157479408221765
        'TSW = 0.00020101618712560002
        'hw = 0.0001
        'hp = 0.006

        'ATPt_cyt = 6.678558415010059
        'ADPt_cyt = 0.02090266659840348
        'Pi_cyt = 0.29175202342229561
        'PCr_cyt = 13.529616006692681

        ''*********Variables described below for mitochondria *************
        'ATPt_mit = 7.3367206117008541
        'Pi_mit = 3.2111519490084248
        'NADH_mit = 1.3887878589301346
        'H_mit = 0.000035195462736798312
        'K_mit = 117.80020000000209
        'UQr_mit = 0.15960973582376589
        'ctCrd_mit = 0.04821561627799667
        'dpsi = -164.46419609752397


    End Sub
    'Public Sub DefaultValues()
    '    Vm = -91.597661630336063

    '    TnChCa = 0.111558664301216
    '    CaMCa = 0.00020159855079
    '    bufferSRCa = 0.0015521111742

    '    Lb_jnc = 0.0218215322629436
    '    Lb_iz = 0.0075621764602356
    '    Hb_jnc = 0.185094540066232
    '    Hb_iz = 0.0769149150028914

    '    Nai = 6.2526658568804461
    '    Ki = 139.64400016892722
    '    Catot_jnc = 0.21632822993076958
    '    Catot_iz = 0.088385376342780084
    '    Catot_blk = 0.11339256386092261
    '    Ca_SRup = 0.81532115796704774
    '    Catot_SRrl = 2.2692289756972372

    '    O_TM = 0.00000068135813460357759
    '    I2_TM = 0.011492842312812529
    '    Is_TM = 0.30054399898494838

    '    O_LSM = 0.0000028240321467868546
    '    I1_LSM = 0.0024386999769378664
    '    I2_LSM = 0.011541375325648934
    '    Is_LSM = 0.29965222266596425

    '    Yco_iz = 0.96951490203350077
    '    Yoc_iz = 0.0000000947782929249469
    '    Yoo_iz = 0.0000030144508647702051
    '    Yco_blk = 0.97019855489823625
    '    Yoc_blk = 0.000000092652600941220067
    '    Yoo_blk = 0.00000314619469048683

    '    Yooo = 0.0000013372774590452455
    '    Yooc = 0.0000016756444981754105
    '    Ycoo = 0.000016274853331264134
    '    Ycoc = 0.969006920418768
    '    Ycco = 0.00000047496639159387847
    '    Yoco = 0.0000000000024604892624143223
    '    Yocc = 0.000000096305367534005485

    '    y1_IKr = 0.00004345699010120553
    '    y2_IKr = 0.0030080758894566674
    '    y3_IKr = 0.99936830005053279

    '    Ov_IKs = 0.0049391363691572926
    '    C2IKs_iz = 0.0097638781635053084
    '    OcIKs_iz = 0.050122020015373692
    '    C2IKs_blk = 0.0098546163805510647
    '    OcIKs_blk = 0.013458943293890937

    '    y1_IKto = 0.000519672138759823
    '    y2_IKto = 0.515195124385403

    '    Pbspm = 0.59274525027013814

    '    E1NCX_iz = 0.26143255551097355
    '    I1NCX_iz = 0.10980404928718691
    '    I2NCX_iz = 0.62807189532382746
    '    E1NCX_blk = 0.10728920489258556
    '    I1NCX_blk = 0.17146351207804253
    '    I2NCX_blk = 0.721057887302995

    '    P1_6_NaK = 0.48891349499375208
    '    P7_NaK = 0.074118984853335956
    '    P8_13_NaK = 0.21075057669861089


    '    halfSL = 1.09840500012898
    '    Fb = 0.0502092089156129
    '    Fp = 4.94926096641491
    '    TSCa3 = 0.0062458694504374905
    '    TSCa3W = 0.00019187875784269291
    '    TSCa3S = 0.00007876873429833259
    '    TSS = 0.00032344101179246739
    '    TSW = 0.0001704952723738
    '    hw = 0.0001
    '    hp = 0.006

    '    ATPt_cyt = 6.6785980115785231
    '    ADPt_cyt = 0.020865112627078384
    '    Pi_cyt = 0.25602449722861048
    '    PCr_cyt = 13.638506610287504

    '    '****************** Variables described below for mitochondria are not used in the present version *************
    '    ATPt_mit = 7.2841097646983117
    '    Pi_mit = 2.9452776474456521
    '    NADH_mit = 1.4686608311773375
    '    H_mit = 0.00003414910083070998
    '    K_mit = 117.80020000000016
    '    UQr_mit = 0.16273250194322272
    '    ctCrd_mit = 0.04686672943773533
    '    dpsi = -164.36578552721812


    'End Sub


End Class
